/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Binary.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
/* Qore class Qore::<binary> */

qore_classid_t CID_PSEUDOBINARY;
QoreClass* QC_PSEUDOBINARY;

// bool PseudoBinary::empty(){}
static bool PseudoBinary_empty(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 74 "Pseudo_QC_Binary.qpp"
return b->empty();
}

// int PseudoBinary::size(){}
static int64 PseudoBinary_size(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 60 "Pseudo_QC_Binary.qpp"
return b->size();
}

// bool PseudoBinary::sizep(){}
static bool PseudoBinary_sizep(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 434 "Pseudo_QC_Binary.qpp"
return true;
}

// list PseudoBinary::split(binary sep){}
static AbstractQoreNode* PseudoBinary_split_Vy(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
   const BinaryNode* sep = HARD_QORE_BINARY(args, 0);
# 111 "Pseudo_QC_Binary.qpp"
return split_intern((const char*)sep->getPtr(), sep->size(), (const char*)b->getPtr(), b->size(), 0);
}

// binary PseudoBinary::substr(softint start){}
static AbstractQoreNode* PseudoBinary_substr_vi(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
# 377 "Pseudo_QC_Binary.qpp"
BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start);
   return b1;
}

// binary PseudoBinary::substr(softint start, softint len){}
static AbstractQoreNode* PseudoBinary_substr_vivi(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
   int64 len = HARD_QORE_INT(args, 1);
# 401 "Pseudo_QC_Binary.qpp"
BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start, len);
   return b1;
}

// string PseudoBinary::toBase64(softint maxlinelen = -1){}
static AbstractQoreNode* PseudoBinary_toBase64_vi(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
   int64 maxlinelen = HARD_QORE_INT(args, 0);
# 356 "Pseudo_QC_Binary.qpp"
return new QoreStringNode(b, maxlinelen);
}

// string PseudoBinary::toHex(){}
static AbstractQoreNode* PseudoBinary_toHex(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 332 "Pseudo_QC_Binary.qpp"
QoreStringNode *rv = new QoreStringNode;
   rv->concatHex(b);
   return rv;
}

// string PseudoBinary::toMD5(){}
static AbstractQoreNode* PseudoBinary_toMD5(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 137 "Pseudo_QC_Binary.qpp"
DigestHelper dh(*b);
   if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
      return 0;

   return dh.getString();
}

// string PseudoBinary::toSHA1(){}
static AbstractQoreNode* PseudoBinary_toSHA1(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 165 "Pseudo_QC_Binary.qpp"
DigestHelper dh(*b);
   if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
      return 0;

   return dh.getString();
}

// string PseudoBinary::toSHA224(){}
static AbstractQoreNode* PseudoBinary_toSHA224(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 196 "Pseudo_QC_Binary.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA224", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoBinary::toSHA256(){}
static AbstractQoreNode* PseudoBinary_toSHA256(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 232 "Pseudo_QC_Binary.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA256", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoBinary::toSHA384(){}
static AbstractQoreNode* PseudoBinary_toSHA384(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 268 "Pseudo_QC_Binary.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA384", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoBinary::toSHA512(){}
static AbstractQoreNode* PseudoBinary_toSHA512(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 304 "Pseudo_QC_Binary.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA512", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoBinary::toString(*string encoding){}
static AbstractQoreNode* PseudoBinary_toString_Ns(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 0));
# 420 "Pseudo_QC_Binary.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   if (!b->size())
      return new QoreStringNode(qe);
   return new QoreStringNode((const char *)b->getPtr(), b->size(), qe);
}

// int PseudoBinary::typeCode(){}
static int64 PseudoBinary_typeCode(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 45 "Pseudo_QC_Binary.qpp"
return NT_BINARY;
}

// bool PseudoBinary::val(){}
static bool PseudoBinary_val(QoreObject* self, const BinaryNode* b, const QoreListNode* args, ExceptionSink* xsink) {
# 92 "Pseudo_QC_Binary.qpp"
return !b->empty();
}

DLLLOCAL QoreClass* initPseudoBinaryClass() {
   QC_PSEUDOBINARY = new QoreClass("<binary>", QDOM_DEFAULT);
   CID_PSEUDOBINARY = QC_PSEUDOBINARY->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOBINARY->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // bool PseudoBinary::empty(){}
   QC_PSEUDOBINARY->addMethodExtended3("empty", (q_method_bool_t)PseudoBinary_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoBinary::size(){}
   QC_PSEUDOBINARY->addMethodExtended3("size", (q_method_int64_t)PseudoBinary_size, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoBinary::sizep(){}
   QC_PSEUDOBINARY->addMethodExtended3("sizep", (q_method_bool_t)PseudoBinary_sizep, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // list PseudoBinary::split(binary sep){}
   QC_PSEUDOBINARY->addMethodExtended3("split", (q_method_t)PseudoBinary_split_Vy, false, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, binaryTypeInfo, NULL, "sep");

   // binary PseudoBinary::substr(softint start){}
   QC_PSEUDOBINARY->addMethodExtended3("substr", (q_method_t)PseudoBinary_substr_vi, false, QC_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 1, softBigIntTypeInfo, NULL, "start");

   // binary PseudoBinary::substr(softint start, softint len){}
   QC_PSEUDOBINARY->addMethodExtended3("substr", (q_method_t)PseudoBinary_substr_vivi, false, QC_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 2, softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len");

   // string PseudoBinary::toBase64(softint maxlinelen = -1){}
   QC_PSEUDOBINARY->addMethodExtended3("toBase64", (q_method_t)PseudoBinary_toBase64_vi, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");

   // string PseudoBinary::toHex(){}
   QC_PSEUDOBINARY->addMethodExtended3("toHex", (q_method_t)PseudoBinary_toHex, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toMD5(){}
   QC_PSEUDOBINARY->addMethodExtended3("toMD5", (q_method_t)PseudoBinary_toMD5, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toSHA1(){}
   QC_PSEUDOBINARY->addMethodExtended3("toSHA1", (q_method_t)PseudoBinary_toSHA1, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toSHA224(){}
   QC_PSEUDOBINARY->addMethodExtended3("toSHA224", (q_method_t)PseudoBinary_toSHA224, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toSHA256(){}
   QC_PSEUDOBINARY->addMethodExtended3("toSHA256", (q_method_t)PseudoBinary_toSHA256, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toSHA384(){}
   QC_PSEUDOBINARY->addMethodExtended3("toSHA384", (q_method_t)PseudoBinary_toSHA384, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toSHA512(){}
   QC_PSEUDOBINARY->addMethodExtended3("toSHA512", (q_method_t)PseudoBinary_toSHA512, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoBinary::toString(*string encoding){}
   QC_PSEUDOBINARY->addMethodExtended3("toString", (q_method_t)PseudoBinary_toString_Ns, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringOrNothingTypeInfo, NULL, "encoding");

   // int PseudoBinary::typeCode(){}
   QC_PSEUDOBINARY->addMethodExtended3("typeCode", (q_method_int64_t)PseudoBinary_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoBinary::val(){}
   QC_PSEUDOBINARY->addMethodExtended3("val", (q_method_bool_t)PseudoBinary_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDOBINARY;
}
