/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Date.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_TimeZone.h>
/* Qore class Qore::<date> */

qore_classid_t CID_PSEUDODATE;
QoreClass* QC_PSEUDODATE;

// bool PseudoDate::absolute(){}
static bool PseudoDate_absolute(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 59 "Pseudo_QC_Date.qpp"
return dt->isAbsolute();
}

// *string PseudoDate::currentZoneName(){}
static AbstractQoreNode* PseudoDate_currentZoneName(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 363 "Pseudo_QC_Date.qpp"
if (dt->isRelative())
      return 0;

   bool is_dst;
   const char *zone_name;

   dt->getZone()->getUTCOffset(dt->getEpochSecondsUTC(), is_dst, zone_name);
   return new QoreStringNode(zone_name);
}

// int PseudoDate::days(){}
static int64 PseudoDate_days(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 121 "Pseudo_QC_Date.qpp"
return dt->getDay();
}

// int PseudoDate::durationMicroseconds(){}
static int64 PseudoDate_durationMicroseconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 300 "Pseudo_QC_Date.qpp"
return dt->getRelativeMicroseconds();
}

// int PseudoDate::durationMilliseconds(){}
static int64 PseudoDate_durationMilliseconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 272 "Pseudo_QC_Date.qpp"
return dt->getRelativeMilliseconds();
}

// int PseudoDate::durationSeconds(){}
static int64 PseudoDate_durationSeconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 242 "Pseudo_QC_Date.qpp"
return dt->getRelativeSeconds();
}

// string PseudoDate::format(string format){}
static AbstractQoreNode* PseudoDate_format_Vs(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* format = HARD_QORE_STRING(args, 0);
# 319 "Pseudo_QC_Date.qpp"
QoreStringNode *rv = new QoreStringNode;
   dt->format(*rv, format->getBuffer());
   return rv;
}

// int PseudoDate::getEpochSeconds(){}
static int64 PseudoDate_getEpochSeconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 460 "Pseudo_QC_Date.qpp"
return dt->isAbsolute() ? dt->getEpochSecondsUTC() : 0;
}

// int PseudoDate::getEpochSecondsLocalTime(){}
static int64 PseudoDate_getEpochSecondsLocalTime(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 474 "Pseudo_QC_Date.qpp"
return dt->isAbsolute() ? dt->getEpochSeconds() : 0;
}

// int PseudoDate::getUtcOffset(){}
static int64 PseudoDate_getUtcOffset(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 383 "Pseudo_QC_Date.qpp"
return dt->getZone()->getUTCOffset();
}

// int PseudoDate::hours(){}
static int64 PseudoDate_hours(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 138 "Pseudo_QC_Date.qpp"
return dt->getHour();
}

// hash PseudoDate::info(){}
static AbstractQoreNode* PseudoDate_info(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 490 "Pseudo_QC_Date.qpp"
return date_info(*dt);
}

// bool PseudoDate::intp(){}
static bool PseudoDate_intp(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 413 "Pseudo_QC_Date.qpp"
return true;
}

// bool PseudoDate::isDst(){}
static bool PseudoDate_isDst(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 334 "Pseudo_QC_Date.qpp"
bool is_dst;

   dt->getZone()->getUTCOffset(dt->getEpochSecondsUTC(), is_dst);
   return is_dst;
}

// int PseudoDate::microseconds(){}
static int64 PseudoDate_microseconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 212 "Pseudo_QC_Date.qpp"
return dt->getMicrosecond();
}

// date PseudoDate::midnight(){}
static AbstractQoreNode* PseudoDate_midnight(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 444 "Pseudo_QC_Date.qpp"
DateTimeNode* rv = new DateTimeNode(*dt);
   rv->setTime(0, 0, 0);
   return rv;
}

// int PseudoDate::milliseconds(){}
static int64 PseudoDate_milliseconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 193 "Pseudo_QC_Date.qpp"
return dt->getMillisecond();
}

// int PseudoDate::minutes(){}
static int64 PseudoDate_minutes(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 155 "Pseudo_QC_Date.qpp"
return dt->getMinute();
}

// int PseudoDate::months(){}
static int64 PseudoDate_months(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 104 "Pseudo_QC_Date.qpp"
return dt->getMonth();
}

// bool PseudoDate::relative(){}
static bool PseudoDate_relative(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 72 "Pseudo_QC_Date.qpp"
return dt->isRelative();
}

// int PseudoDate::seconds(){}
static int64 PseudoDate_seconds(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 174 "Pseudo_QC_Date.qpp"
return dt->getSecond();
}

// bool PseudoDate::strp(){}
static bool PseudoDate_strp(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 427 "Pseudo_QC_Date.qpp"
return true;
}

// int PseudoDate::typeCode(){}
static int64 PseudoDate_typeCode(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 46 "Pseudo_QC_Date.qpp"
return NT_DATE;
}

// bool PseudoDate::val(){}
static bool PseudoDate_val(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 399 "Pseudo_QC_Date.qpp"
return dt->hasValue();
}

// int PseudoDate::years(){}
static int64 PseudoDate_years(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 87 "Pseudo_QC_Date.qpp"
return dt->getYear();
}

// *TimeZone PseudoDate::zone(){}
static QoreObject* PseudoDate_zone(QoreObject* self, const DateTimeNode *dt, const QoreListNode* args, ExceptionSink* xsink) {
# 350 "Pseudo_QC_Date.qpp"
return dt->isAbsolute() ? new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(dt->getZone())) : 0;
}

DLLLOCAL QoreClass* initPseudoDateClass() {
   QC_PSEUDODATE = new QoreClass("<date>", QDOM_DEFAULT);
   CID_PSEUDODATE = QC_PSEUDODATE->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDODATE->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // bool PseudoDate::absolute(){}
   QC_PSEUDODATE->addMethodExtended3("absolute", (q_method_bool_t)PseudoDate_absolute, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // *string PseudoDate::currentZoneName(){}
   QC_PSEUDODATE->addMethodExtended3("currentZoneName", (q_method_t)PseudoDate_currentZoneName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // int PseudoDate::days(){}
   QC_PSEUDODATE->addMethodExtended3("days", (q_method_int64_t)PseudoDate_days, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::durationMicroseconds(){}
   QC_PSEUDODATE->addMethodExtended3("durationMicroseconds", (q_method_int64_t)PseudoDate_durationMicroseconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::durationMilliseconds(){}
   QC_PSEUDODATE->addMethodExtended3("durationMilliseconds", (q_method_int64_t)PseudoDate_durationMilliseconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::durationSeconds(){}
   QC_PSEUDODATE->addMethodExtended3("durationSeconds", (q_method_int64_t)PseudoDate_durationSeconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // string PseudoDate::format(string format){}
   QC_PSEUDODATE->addMethodExtended3("format", (q_method_t)PseudoDate_format_Vs, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "format");

   // int PseudoDate::getEpochSeconds(){}
   QC_PSEUDODATE->addMethodExtended3("getEpochSeconds", (q_method_int64_t)PseudoDate_getEpochSeconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::getEpochSecondsLocalTime(){}
   QC_PSEUDODATE->addMethodExtended3("getEpochSecondsLocalTime", (q_method_int64_t)PseudoDate_getEpochSecondsLocalTime, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::getUtcOffset(){}
   QC_PSEUDODATE->addMethodExtended3("getUtcOffset", (q_method_int64_t)PseudoDate_getUtcOffset, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::hours(){}
   QC_PSEUDODATE->addMethodExtended3("hours", (q_method_int64_t)PseudoDate_hours, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // hash PseudoDate::info(){}
   QC_PSEUDODATE->addMethodExtended3("info", (q_method_t)PseudoDate_info, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

   // bool PseudoDate::intp(){}
   QC_PSEUDODATE->addMethodExtended3("intp", (q_method_bool_t)PseudoDate_intp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool PseudoDate::isDst(){}
   QC_PSEUDODATE->addMethodExtended3("isDst", (q_method_bool_t)PseudoDate_isDst, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoDate::microseconds(){}
   QC_PSEUDODATE->addMethodExtended3("microseconds", (q_method_int64_t)PseudoDate_microseconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // date PseudoDate::midnight(){}
   QC_PSEUDODATE->addMethodExtended3("midnight", (q_method_t)PseudoDate_midnight, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);

   // int PseudoDate::milliseconds(){}
   QC_PSEUDODATE->addMethodExtended3("milliseconds", (q_method_int64_t)PseudoDate_milliseconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::minutes(){}
   QC_PSEUDODATE->addMethodExtended3("minutes", (q_method_int64_t)PseudoDate_minutes, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoDate::months(){}
   QC_PSEUDODATE->addMethodExtended3("months", (q_method_int64_t)PseudoDate_months, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoDate::relative(){}
   QC_PSEUDODATE->addMethodExtended3("relative", (q_method_bool_t)PseudoDate_relative, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoDate::seconds(){}
   QC_PSEUDODATE->addMethodExtended3("seconds", (q_method_int64_t)PseudoDate_seconds, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoDate::strp(){}
   QC_PSEUDODATE->addMethodExtended3("strp", (q_method_bool_t)PseudoDate_strp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoDate::typeCode(){}
   QC_PSEUDODATE->addMethodExtended3("typeCode", (q_method_int64_t)PseudoDate_typeCode, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoDate::val(){}
   QC_PSEUDODATE->addMethodExtended3("val", (q_method_bool_t)PseudoDate_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoDate::years(){}
   QC_PSEUDODATE->addMethodExtended3("years", (q_method_int64_t)PseudoDate_years, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // *TimeZone PseudoDate::zone(){}
   QC_PSEUDODATE->addMethodExtended3("zone", (q_method_t)PseudoDate_zone, false, QC_CONSTANT, QDOM_DEFAULT, QC_TIMEZONE->getOrNothingTypeInfo());

   return QC_PSEUDODATE;
}
