/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Float.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
/* Qore class Qore::<float> */

qore_classid_t CID_PSEUDOFLOAT;
QoreClass* QC_PSEUDOFLOAT;

// float PseudoFloat::abs(){}
static double PseudoFloat_abs(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
# 145 "Pseudo_QC_Float.qpp"
return fabs(f->f);
}

// string PseudoFloat::format(string fmt){}
static AbstractQoreNode* PseudoFloat_format_Vs(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* fmt = HARD_QORE_STRING(args, 0);
# 113 "Pseudo_QC_Float.qpp"
return format_float_intern(*fmt, f->f);
}

// bool PseudoFloat::intp(){}
static bool PseudoFloat_intp(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
# 76 "Pseudo_QC_Float.qpp"
return true;
}

// int PseudoFloat::sign(){}
static int64 PseudoFloat_sign(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
# 127 "Pseudo_QC_Float.qpp"
if (!f->f)
      return 0;
   return f->f > 0.0 ? 1 : -1;
}

// bool PseudoFloat::strp(){}
static bool PseudoFloat_strp(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
# 90 "Pseudo_QC_Float.qpp"
return true;
}

// int PseudoFloat::typeCode(){}
static int64 PseudoFloat_typeCode(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
# 45 "Pseudo_QC_Float.qpp"
return NT_FLOAT;
}

// bool PseudoFloat::val(){}
static bool PseudoFloat_val(QoreObject* self, const QoreFloatNode* f, const QoreListNode* args, ExceptionSink* xsink) {
# 62 "Pseudo_QC_Float.qpp"
return (bool)f->f == 0.0 ? false : true;
}

DLLLOCAL QoreClass* initPseudoFloatClass() {
   QC_PSEUDOFLOAT = new QoreClass("<float>", QDOM_DEFAULT);
   CID_PSEUDOFLOAT = QC_PSEUDOFLOAT->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOFLOAT->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // float PseudoFloat::abs(){}
   QC_PSEUDOFLOAT->addMethodExtended3("abs", (q_method_double_t)PseudoFloat_abs, false, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo);

   // string PseudoFloat::format(string fmt){}
   QC_PSEUDOFLOAT->addMethodExtended3("format", (q_method_t)PseudoFloat_format_Vs, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");

   // bool PseudoFloat::intp(){}
   QC_PSEUDOFLOAT->addMethodExtended3("intp", (q_method_bool_t)PseudoFloat_intp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoFloat::sign(){}
   QC_PSEUDOFLOAT->addMethodExtended3("sign", (q_method_int64_t)PseudoFloat_sign, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoFloat::strp(){}
   QC_PSEUDOFLOAT->addMethodExtended3("strp", (q_method_bool_t)PseudoFloat_strp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoFloat::typeCode(){}
   QC_PSEUDOFLOAT->addMethodExtended3("typeCode", (q_method_int64_t)PseudoFloat_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoFloat::val(){}
   QC_PSEUDOFLOAT->addMethodExtended3("val", (q_method_bool_t)PseudoFloat_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDOFLOAT;
}
