/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Hash.cpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>

#include <qore/intern/QoreHashNodeIntern.h>
#include <qore/intern/QoreHashIterator.h>
#include <qore/intern/QoreHashListIterator.h>
/* Qore class Qore::<hash> */

qore_classid_t CID_PSEUDOHASH;
QoreClass* QC_PSEUDOHASH;

// bool PseudoHash::compareKeys(hash oh){}
static bool PseudoHash_compareKeys_Vh(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* oh = HARD_QORE_HASH(args, 0);
# 351 "Pseudo_QC_Hash.qpp"
if (h->size() != oh->size())
      return false;

   ConstHashIterator hi1(h);
   ConstHashIterator hi2(oh);
   while (hi1.next()) {
      hi2.next();
      if (strcmp(hi1.getKey(), hi2.getKey()))
         return false;
   }

   return true;
}

// HashListIterator PseudoHash::contextIterator(){}
static QoreObject* PseudoHash_contextIterator(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 333 "Pseudo_QC_Hash.qpp"
return new QoreObject(QC_HASHLISTITERATOR, 0, new QoreHashListIterator(h));
}

// bool PseudoHash::empty(){}
static bool PseudoHash_empty(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 226 "Pseudo_QC_Hash.qpp"
return h->empty();
}

// *string PseudoHash::firstKey(){}
static AbstractQoreNode* PseudoHash_firstKey(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 110 "Pseudo_QC_Hash.qpp"
const char *key = h->getFirstKey();
   return key ? new QoreStringNode(key) : 0;
}

// any PseudoHash::firstValue(){}
static AbstractQoreNode* PseudoHash_firstValue(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 128 "Pseudo_QC_Hash.qpp"
AbstractQoreNode* rv = qore_hash_private::getFirstKeyValue(h);
   return rv ? rv->refSelf() : 0;
}

// bool PseudoHash::hasKey(softstring key){}
static bool PseudoHash_hasKey_vs(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 184 "Pseudo_QC_Hash.qpp"
TempEncodingHelper str(key, QCS_DEFAULT, xsink);
   if (!str)
      return false;

   return h->existsKey(str->getBuffer());
}

// bool PseudoHash::hasKeyValue(softstring key){}
static bool PseudoHash_hasKeyValue_vs(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 207 "Pseudo_QC_Hash.qpp"
TempEncodingHelper str(key, QCS_DEFAULT, xsink);
   if (!str)
      return false;

   return h->existsKeyValue(str->getBuffer());
}

// AbstractIterator PseudoHash::iterator(){}
static QoreObject* PseudoHash_iterator(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 280 "Pseudo_QC_Hash.qpp"
return new QoreObject(QC_HASHITERATOR, 0, new QoreHashIterator(h));
}

// HashKeyIterator PseudoHash::keyIterator(){}
static QoreObject* PseudoHash_keyIterator(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 298 "Pseudo_QC_Hash.qpp"
return new QoreObject(QC_HASHKEYITERATOR, 0, new QoreHashIterator(h));
}

// list PseudoHash::keys(){}
static AbstractQoreNode* PseudoHash_keys(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 68 "Pseudo_QC_Hash.qpp"
return h->getKeys();
}

// *string PseudoHash::lastKey(){}
static AbstractQoreNode* PseudoHash_lastKey(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 146 "Pseudo_QC_Hash.qpp"
const char *key = h->getLastKey();
   return key ? new QoreStringNode(key) : 0;
}

// any PseudoHash::lastValue(){}
static AbstractQoreNode* PseudoHash_lastValue(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 164 "Pseudo_QC_Hash.qpp"
AbstractQoreNode* rv = qore_hash_private::getLastKeyValue(h);
   return rv ? rv->refSelf() : 0;
}

// HashPairIterator PseudoHash::pairIterator(){}
static QoreObject* PseudoHash_pairIterator(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 314 "Pseudo_QC_Hash.qpp"
return new QoreObject(QC_HASHPAIRITERATOR, 0, new QoreHashIterator(h));
}

// int PseudoHash::size(){}
static int64 PseudoHash_size(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 244 "Pseudo_QC_Hash.qpp"
return h->size();
}

// bool PseudoHash::sizep(){}
static bool PseudoHash_sizep(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 373 "Pseudo_QC_Hash.qpp"
return true;
}

// int PseudoHash::typeCode(){}
static int64 PseudoHash_typeCode(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 51 "Pseudo_QC_Hash.qpp"
return NT_HASH;
}

// bool PseudoHash::val(){}
static bool PseudoHash_val(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 264 "Pseudo_QC_Hash.qpp"
return !h->empty();
}

// list PseudoHash::values(){}
static AbstractQoreNode* PseudoHash_values(QoreObject* self, const QoreHashNode* h, const QoreListNode* args, ExceptionSink* xsink) {
# 87 "Pseudo_QC_Hash.qpp"
QoreListNode *l = new QoreListNode;

   ConstHashIterator hi(h);
   while (hi.next())
      l->push(hi.getReferencedValue());

   return l;
}

DLLLOCAL QoreClass* initPseudoHashClass() {
   QC_PSEUDOHASH = new QoreClass("<hash>", QDOM_DEFAULT);
   CID_PSEUDOHASH = QC_PSEUDOHASH->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOHASH->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // bool PseudoHash::compareKeys(hash oh){}
   QC_PSEUDOHASH->addMethodExtended3("compareKeys", (q_method_bool_t)PseudoHash_compareKeys_Vh, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, hashTypeInfo, NULL, "oh");

   // HashListIterator PseudoHash::contextIterator(){}
   QC_PSEUDOHASH->addMethodExtended3("contextIterator", (q_method_t)PseudoHash_contextIterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_HASHLISTITERATOR->getTypeInfo());

   // bool PseudoHash::empty(){}
   QC_PSEUDOHASH->addMethodExtended3("empty", (q_method_bool_t)PseudoHash_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // *string PseudoHash::firstKey(){}
   QC_PSEUDOHASH->addMethodExtended3("firstKey", (q_method_t)PseudoHash_firstKey, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // any PseudoHash::firstValue(){}
   QC_PSEUDOHASH->addMethodExtended3("firstValue", (q_method_t)PseudoHash_firstValue, false, QC_CONSTANT, QDOM_DEFAULT, anyTypeInfo);

   // bool PseudoHash::hasKey(softstring key){}
   QC_PSEUDOHASH->addMethodExtended3("hasKey", (q_method_bool_t)PseudoHash_hasKey_vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, softStringTypeInfo, NULL, "key");

   // bool PseudoHash::hasKeyValue(softstring key){}
   QC_PSEUDOHASH->addMethodExtended3("hasKeyValue", (q_method_bool_t)PseudoHash_hasKeyValue_vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, softStringTypeInfo, NULL, "key");

   // AbstractIterator PseudoHash::iterator(){}
   QC_PSEUDOHASH->addMethodExtended3("iterator", (q_method_t)PseudoHash_iterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

   // HashKeyIterator PseudoHash::keyIterator(){}
   QC_PSEUDOHASH->addMethodExtended3("keyIterator", (q_method_t)PseudoHash_keyIterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_HASHKEYITERATOR->getTypeInfo());

   // list PseudoHash::keys(){}
   QC_PSEUDOHASH->addMethodExtended3("keys", (q_method_t)PseudoHash_keys, false, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo);

   // *string PseudoHash::lastKey(){}
   QC_PSEUDOHASH->addMethodExtended3("lastKey", (q_method_t)PseudoHash_lastKey, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // any PseudoHash::lastValue(){}
   QC_PSEUDOHASH->addMethodExtended3("lastValue", (q_method_t)PseudoHash_lastValue, false, QC_CONSTANT, QDOM_DEFAULT, anyTypeInfo);

   // HashPairIterator PseudoHash::pairIterator(){}
   QC_PSEUDOHASH->addMethodExtended3("pairIterator", (q_method_t)PseudoHash_pairIterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_HASHPAIRITERATOR->getTypeInfo());

   // int PseudoHash::size(){}
   QC_PSEUDOHASH->addMethodExtended3("size", (q_method_int64_t)PseudoHash_size, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoHash::sizep(){}
   QC_PSEUDOHASH->addMethodExtended3("sizep", (q_method_bool_t)PseudoHash_sizep, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoHash::typeCode(){}
   QC_PSEUDOHASH->addMethodExtended3("typeCode", (q_method_int64_t)PseudoHash_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoHash::val(){}
   QC_PSEUDOHASH->addMethodExtended3("val", (q_method_bool_t)PseudoHash_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // list PseudoHash::values(){}
   QC_PSEUDOHASH->addMethodExtended3("values", (q_method_t)PseudoHash_values, false, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo);

   return QC_PSEUDOHASH;
}
