/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Int.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/qore_number_private.h>
/* Qore class Qore::<int> */

qore_classid_t CID_PSEUDOINT;
QoreClass* QC_PSEUDOINT;

// int PseudoInt::abs(){}
static int64 PseudoInt_abs(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 274 "Pseudo_QC_Int.qpp"
return i->val < 0 ? -i->val : i->val;
}

// binary PseudoInt::encodeLsb(int size = 4){}
static AbstractQoreNode* PseudoInt_encodeLsb_Vi(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
   int64 size = HARD_QORE_INT(args, 0);
# 214 "Pseudo_QC_Int.qpp"
SimpleRefHolder<BinaryNode> b(new BinaryNode);
   switch (size) {
      case 1: {
         unsigned char c = i->val;
         if (i->val != c) {
            xsink->raiseException("SIZE-ERROR", "the integer value called with <int>::encodeLsb(size=1) ("QLLD") is too large to fit in 1 byte", i->val);
            return 0;
         }
         b->append((void*)&c, 1);
         break;
      }

      case 2: {
         uint16_t num = i->val;
         if (i->val != num) {
            xsink->raiseException("SIZE-ERROR", "the integer value called with <int>::encodeLsb(size=2) ("QLLD") is too large to fit in 2 bytes", i->val);
            return 0;
         }
         num = i2LSB(num);
         b->append((void*)&num, 2);
         break;
      }

      case 4: {
         uint32_t num = i->val;
         if (i->val != num) {
            xsink->raiseException("SIZE-ERROR", "the integer value called with <int>::encodeLsb(size=4) ("QLLD") is too large to fit in 4 bytes", i->val);
            return 0;
         }
         num = i4LSB(num);
         b->append((void*)&num, 4);
         break;
      }

      case 8: {
         int64 num = i8LSB(i->val);
         b->append((void*)&num, 8);
         break;
      }

      default:
         xsink->raiseException("SIZE-ERROR", "the size given to <int>::encodeLsb("QLLD") is not 1, 2, 4, or 8", size);
         return 0;
   }
   return b.release();
}

// binary PseudoInt::encodeMsb(int size = 4){}
static AbstractQoreNode* PseudoInt_encodeMsb_Vi(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
   int64 size = HARD_QORE_INT(args, 0);
# 152 "Pseudo_QC_Int.qpp"
SimpleRefHolder<BinaryNode> b(new BinaryNode);
   switch (size) {
      case 1: {
         unsigned char c = i->val;
         if (i->val != c) {
            xsink->raiseException("SIZE-ERROR", "the integer value called with <int>::encodeMsb(size=1) ("QLLD") is too large to fit in 1 byte", i->val);
            return 0;
         }
         b->append((void*)&c, 1);
         break;
      }

      case 2: {
         uint16_t num = i->val;
         if (i->val != num) {
            xsink->raiseException("SIZE-ERROR", "the integer value called with <int>::encodeMsb(size=2) ("QLLD") is too large to fit in 2 bytes", i->val);
            return 0;
         }
         num = htons(num);
         b->append((void*)&num, 2);
         break;
      }

      case 4: {
         uint32_t num = i->val;
         if (i->val != num) {
            xsink->raiseException("SIZE-ERROR", "the integer value called with <int>::encodeMsb(size=4) ("QLLD") is too large to fit in 4 bytes", i->val);
            return 0;
         }
         num = htonl(num);
         b->append((void*)&num, 4);
         break;
      }

      case 8: {
         int64 num = i8MSB(i->val);
         b->append((void*)&num, 8);
         break;
      }

      default:
         xsink->raiseException("SIZE-ERROR", "the size given to <int>::encodeMsb("QLLD") is not 1, 2, 4, or 8", size);
         return 0;
   }
   return b.release();
}

// string PseudoInt::format(string fmt){}
static AbstractQoreNode* PseudoInt_format_Vs(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* fmt = HARD_QORE_STRING(args, 0);
# 116 "Pseudo_QC_Int.qpp"
SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(fmt->getEncoding()));
   rv->sprintf(QLLD, i->val);
   return qore_number_private::formatNumberString(**rv, *fmt, xsink) ? 0 : rv.release();
}

// bool PseudoInt::intp(){}
static bool PseudoInt_intp(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 77 "Pseudo_QC_Int.qpp"
return true;
}

// int PseudoInt::sign(){}
static int64 PseudoInt_sign(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 132 "Pseudo_QC_Int.qpp"
if (!i->val)
      return 0;
   return i->val > 0 ? 1 : -1;
}

// bool PseudoInt::strp(){}
static bool PseudoInt_strp(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 91 "Pseudo_QC_Int.qpp"
return true;
}

// string PseudoInt::toUnicode(){}
static AbstractQoreNode* PseudoInt_toUnicode(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 290 "Pseudo_QC_Int.qpp"
SimpleRefHolder<QoreStringNode> str(new QoreStringNode(QCS_UTF8));
   str->concatUnicode(i->val);
   return *xsink ? 0 : str.release();
}

// int PseudoInt::typeCode(){}
static int64 PseudoInt_typeCode(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 46 "Pseudo_QC_Int.qpp"
return NT_INT;
}

// bool PseudoInt::val(){}
static bool PseudoInt_val(QoreObject* self, const QoreBigIntNode* i, const QoreListNode* args, ExceptionSink* xsink) {
# 63 "Pseudo_QC_Int.qpp"
return (bool)i->val;
}

DLLLOCAL QoreClass* initPseudoIntClass() {
   QC_PSEUDOINT = new QoreClass("<int>", QDOM_DEFAULT);
   CID_PSEUDOINT = QC_PSEUDOINT->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOINT->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // int PseudoInt::abs(){}
   QC_PSEUDOINT->addMethodExtended3("abs", (q_method_int64_t)PseudoInt_abs, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // binary PseudoInt::encodeLsb(int size = 4){}
   QC_PSEUDOINT->addMethodExtended3("encodeLsb", (q_method_t)PseudoInt_encodeLsb_Vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, bigIntTypeInfo, new QoreBigIntNode(4), "size");

   // binary PseudoInt::encodeMsb(int size = 4){}
   QC_PSEUDOINT->addMethodExtended3("encodeMsb", (q_method_t)PseudoInt_encodeMsb_Vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, bigIntTypeInfo, new QoreBigIntNode(4), "size");

   // string PseudoInt::format(string fmt){}
   QC_PSEUDOINT->addMethodExtended3("format", (q_method_t)PseudoInt_format_Vs, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");

   // bool PseudoInt::intp(){}
   QC_PSEUDOINT->addMethodExtended3("intp", (q_method_bool_t)PseudoInt_intp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoInt::sign(){}
   QC_PSEUDOINT->addMethodExtended3("sign", (q_method_int64_t)PseudoInt_sign, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoInt::strp(){}
   QC_PSEUDOINT->addMethodExtended3("strp", (q_method_bool_t)PseudoInt_strp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // string PseudoInt::toUnicode(){}
   QC_PSEUDOINT->addMethodExtended3("toUnicode", (q_method_t)PseudoInt_toUnicode, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // int PseudoInt::typeCode(){}
   QC_PSEUDOINT->addMethodExtended3("typeCode", (q_method_int64_t)PseudoInt_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoInt::val(){}
   QC_PSEUDOINT->addMethodExtended3("val", (q_method_bool_t)PseudoInt_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDOINT;
}
