/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_List.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreLibIntern.h>
#include <qore/intern/QoreListIterator.h>
#include <qore/intern/RangeIterator.h>
/* Qore class Qore::<list> */

qore_classid_t CID_PSEUDOLIST;
QoreClass* QC_PSEUDOLIST;

// bool PseudoList::contains(any arg){}
static bool PseudoList_contains_Va(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
# 184 "Pseudo_QC_List.qpp"
return inlist_intern(arg, l, xsink);
}

// bool PseudoList::empty(){}
static bool PseudoList_empty(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 77 "Pseudo_QC_List.qpp"
return l->empty();
}

// any PseudoList::first(){}
static AbstractQoreNode* PseudoList_first(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 111 "Pseudo_QC_List.qpp"
return l->get_referenced_entry(0);
}

// AbstractIterator PseudoList::iterator(){}
static QoreObject* PseudoList_iterator(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 198 "Pseudo_QC_List.qpp"
return new QoreObject(QC_LISTITERATOR, 0, new QoreListIterator(l));
}

// string PseudoList::join(string str){}
static AbstractQoreNode* PseudoList_join_Vs(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* str = HARD_QORE_STRING(args, 0);
# 149 "Pseudo_QC_List.qpp"
return join_intern(str, l, 0, xsink);
}

// any PseudoList::last(){}
static AbstractQoreNode* PseudoList_last(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 127 "Pseudo_QC_List.qpp"
return l->get_referenced_entry((qore_size_t)(l->size() - 1));
}

// int PseudoList::lsize(){}
static int64 PseudoList_lsize(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 166 "Pseudo_QC_List.qpp"
return l->size();
}

// AbstractIterator PseudoList::rangeIterator(){}
static QoreObject* PseudoList_rangeIterator(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 212 "Pseudo_QC_List.qpp"
return l->empty() 
      ? new QoreObject(QC_SINGLEVALUEITERATOR, 0, new SingleValueIterator(0))
      : new QoreObject(QC_RANGEITERATOR, 0, new RangeIterator(0, l->size() - 1, 1, xsink));
   return 0;
}

// int PseudoList::size(){}
static int64 PseudoList_size(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 63 "Pseudo_QC_List.qpp"
return l->size();
}

// bool PseudoList::sizep(){}
static bool PseudoList_sizep(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 225 "Pseudo_QC_List.qpp"
return true;
}

// int PseudoList::typeCode(){}
static int64 PseudoList_typeCode(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 48 "Pseudo_QC_List.qpp"
return NT_LIST;
}

// bool PseudoList::val(){}
static bool PseudoList_val(QoreObject* self, const QoreListNode* l, const QoreListNode* args, ExceptionSink* xsink) {
# 95 "Pseudo_QC_List.qpp"
return !l->empty();
}

DLLLOCAL QoreClass* initPseudoListClass() {
   QC_PSEUDOLIST = new QoreClass("<list>", QDOM_DEFAULT);
   CID_PSEUDOLIST = QC_PSEUDOLIST->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOLIST->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // bool PseudoList::contains(any arg){}
   QC_PSEUDOLIST->addMethodExtended3("contains", (q_method_bool_t)PseudoList_contains_Va, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, anyTypeInfo, NULL, "arg");

   // bool PseudoList::empty(){}
   QC_PSEUDOLIST->addMethodExtended3("empty", (q_method_bool_t)PseudoList_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // any PseudoList::first(){}
   QC_PSEUDOLIST->addMethodExtended3("first", (q_method_t)PseudoList_first, false, QC_CONSTANT, QDOM_DEFAULT, anyTypeInfo);

   // AbstractIterator PseudoList::iterator(){}
   QC_PSEUDOLIST->addMethodExtended3("iterator", (q_method_t)PseudoList_iterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

   // string PseudoList::join(string str){}
   QC_PSEUDOLIST->addMethodExtended3("join", (q_method_t)PseudoList_join_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");

   // any PseudoList::last(){}
   QC_PSEUDOLIST->addMethodExtended3("last", (q_method_t)PseudoList_last, false, QC_CONSTANT, QDOM_DEFAULT, anyTypeInfo);

   // int PseudoList::lsize(){}
   QC_PSEUDOLIST->addMethodExtended3("lsize", (q_method_int64_t)PseudoList_lsize, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // AbstractIterator PseudoList::rangeIterator(){}
   QC_PSEUDOLIST->addMethodExtended3("rangeIterator", (q_method_t)PseudoList_rangeIterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

   // int PseudoList::size(){}
   QC_PSEUDOLIST->addMethodExtended3("size", (q_method_int64_t)PseudoList_size, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoList::sizep(){}
   QC_PSEUDOLIST->addMethodExtended3("sizep", (q_method_bool_t)PseudoList_sizep, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoList::typeCode(){}
   QC_PSEUDOLIST->addMethodExtended3("typeCode", (q_method_int64_t)PseudoList_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoList::val(){}
   QC_PSEUDOLIST->addMethodExtended3("val", (q_method_bool_t)PseudoList_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDOLIST;
}
