/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Number.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/qore_number_private.h>
/* Qore class Qore::<number> */

qore_classid_t CID_PSEUDONUMBER;
QoreClass* QC_PSEUDONUMBER;

// number PseudoNumber::abs(){}
static AbstractQoreNode* PseudoNumber_abs(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 233 "Pseudo_QC_Number.qpp"
return qore_number_private::doUnary(*n, mpfr_abs);
}

// string PseudoNumber::format(string fmt){}
static AbstractQoreNode* PseudoNumber_format_Vs(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* fmt = HARD_QORE_STRING(args, 0);
# 216 "Pseudo_QC_Number.qpp"
SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(fmt->getEncoding()));
   return qore_number_private::get(*n)->format(**rv, *fmt, xsink) ? 0 : rv.release();
}

// bool PseudoNumber::infp(){}
static bool PseudoNumber_infp(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 150 "Pseudo_QC_Number.qpp"
return qore_number_private::get(*n)->inf();
}

// bool PseudoNumber::intp(){}
static bool PseudoNumber_intp(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 102 "Pseudo_QC_Number.qpp"
return true;
}

// bool PseudoNumber::nanp(){}
static bool PseudoNumber_nanp(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 134 "Pseudo_QC_Number.qpp"
return qore_number_private::get(*n)->nan();
}

// int PseudoNumber::prec(){}
static int64 PseudoNumber_prec(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 178 "Pseudo_QC_Number.qpp"
return (int64)qore_number_private::get(*n)->getPrec();
}

// int PseudoNumber::sign(){}
static int64 PseudoNumber_sign(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 164 "Pseudo_QC_Number.qpp"
return qore_number_private::get(*n)->sign();
}

// bool PseudoNumber::strp(){}
static bool PseudoNumber_strp(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 118 "Pseudo_QC_Number.qpp"
return true;
}

// string PseudoNumber::toString(int fmt){}
static AbstractQoreNode* PseudoNumber_toString_Vi(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
   int64 fmt = HARD_QORE_INT(args, 0);
# 189 "Pseudo_QC_Number.qpp"
QoreStringNode* rv = new QoreStringNode;
   n->toString(*rv, fmt);
   return rv;
}

// int PseudoNumber::typeCode(){}
static int64 PseudoNumber_typeCode(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 67 "Pseudo_QC_Number.qpp"
return NT_NUMBER;
}

// bool PseudoNumber::val(){}
static bool PseudoNumber_val(QoreObject* self, const QoreNumberNode* n, const QoreListNode* args, ExceptionSink* xsink) {
# 86 "Pseudo_QC_Number.qpp"
return !n->zero();
}

DLLLOCAL QoreClass* initPseudoNumberClass() {
   QC_PSEUDONUMBER = new QoreClass("<number>", QDOM_DEFAULT);
   CID_PSEUDONUMBER = QC_PSEUDONUMBER->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDONUMBER->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // number PseudoNumber::abs(){}
   QC_PSEUDONUMBER->addMethodExtended3("abs", (q_method_t)PseudoNumber_abs, false, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo);

   // string PseudoNumber::format(string fmt){}
   QC_PSEUDONUMBER->addMethodExtended3("format", (q_method_t)PseudoNumber_format_Vs, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");

   // bool PseudoNumber::infp(){}
   QC_PSEUDONUMBER->addMethodExtended3("infp", (q_method_bool_t)PseudoNumber_infp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool PseudoNumber::intp(){}
   QC_PSEUDONUMBER->addMethodExtended3("intp", (q_method_bool_t)PseudoNumber_intp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool PseudoNumber::nanp(){}
   QC_PSEUDONUMBER->addMethodExtended3("nanp", (q_method_bool_t)PseudoNumber_nanp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoNumber::prec(){}
   QC_PSEUDONUMBER->addMethodExtended3("prec", (q_method_int64_t)PseudoNumber_prec, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int PseudoNumber::sign(){}
   QC_PSEUDONUMBER->addMethodExtended3("sign", (q_method_int64_t)PseudoNumber_sign, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoNumber::strp(){}
   QC_PSEUDONUMBER->addMethodExtended3("strp", (q_method_bool_t)PseudoNumber_strp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // string PseudoNumber::toString(int fmt){}
   QC_PSEUDONUMBER->addMethodExtended3("toString", (q_method_t)PseudoNumber_toString_Vi, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, NULL, "fmt");

   // int PseudoNumber::typeCode(){}
   QC_PSEUDONUMBER->addMethodExtended3("typeCode", (q_method_int64_t)PseudoNumber_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoNumber::val(){}
   QC_PSEUDONUMBER->addMethodExtended3("val", (q_method_bool_t)PseudoNumber_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDONUMBER;
}

DLLLOCAL void init_QC_Number_constants(QoreNamespace& ns) {
# 28 "Pseudo_QC_Number.qpp"
# 34 "Pseudo_QC_Number.qpp"
   ns.addConstant("NF_Default", new QoreBigIntNode(QORE_NF_DEFAULT));
   ns.addConstant("NF_Raw", new QoreBigIntNode(QORE_NF_RAW));
   ns.addConstant("NF_Scientific", new QoreBigIntNode(QORE_NF_SCIENTIFIC));

}
