/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Object.cpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreObjectIntern.h>
#include <qore/intern/QoreObjectIterator.h>
#include <qore/intern/QoreClassIntern.h>
/* Qore class Qore::<object> */

qore_classid_t CID_PSEUDOOBJECT;
QoreClass* QC_PSEUDOOBJECT;

// string PseudoObject::className(){}
static AbstractQoreNode* PseudoObject_className(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 133 "Pseudo_QC_Object.qpp"
return new QoreStringNode(obj->getClassName());
}

// bool PseudoObject::empty(){}
static bool PseudoObject_empty(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 105 "Pseudo_QC_Object.qpp"
return !obj->size(xsink);
}

// *string PseudoObject::firstKey(){}
static AbstractQoreNode* PseudoObject_firstKey(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 76 "Pseudo_QC_Object.qpp"
return qore_object_private::firstKey(obj, xsink);
}

// bool PseudoObject::hasCallableMethod(string name){}
static bool PseudoObject_hasCallableMethod_Vs(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* name = HARD_QORE_STRING(args, 0);
# 232 "Pseudo_QC_Object.qpp"
return qore_class_private::hasCallableMethod(*(obj->getClass()), name->getBuffer());
}

// bool PseudoObject::hasCallableNormalMethod(string name){}
static bool PseudoObject_hasCallableNormalMethod_Vs(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* name = HARD_QORE_STRING(args, 0);
# 255 "Pseudo_QC_Object.qpp"
return qore_class_private::hasCallableNormalMethod(*(obj->getClass()), name->getBuffer());
}

// bool PseudoObject::hasCallableStaticMethod(string name){}
static bool PseudoObject_hasCallableStaticMethod_Vs(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* name = HARD_QORE_STRING(args, 0);
# 277 "Pseudo_QC_Object.qpp"
return qore_class_private::hasCallableStaticMethod(*(obj->getClass()), name->getBuffer());
}

// bool PseudoObject::isSystem(){}
static bool PseudoObject_isSystem(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 146 "Pseudo_QC_Object.qpp"
return get_bool_node(obj->isSystemObject());
}

// AbstractIterator PseudoObject::iterator(){}
static QoreObject* PseudoObject_iterator(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 181 "Pseudo_QC_Object.qpp"
return new QoreObject(QC_OBJECTITERATOR, 0, new QoreObjectIterator(obj));
}

// ObjectKeyIterator PseudoObject::keyIterator(){}
static QoreObject* PseudoObject_keyIterator(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 195 "Pseudo_QC_Object.qpp"
return new QoreObject(QC_OBJECTKEYITERATOR, 0, new QoreObjectIterator(obj));
}

// list PseudoObject::keys(){}
static AbstractQoreNode* PseudoObject_keys(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 61 "Pseudo_QC_Object.qpp"
return obj->getMemberList(xsink);
}

// *string PseudoObject::lastKey(){}
static AbstractQoreNode* PseudoObject_lastKey(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 91 "Pseudo_QC_Object.qpp"
return qore_object_private::lastKey(obj, xsink);
}

// ObjectPairIterator PseudoObject::pairIterator(){}
static QoreObject* PseudoObject_pairIterator(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 209 "Pseudo_QC_Object.qpp"
return new QoreObject(QC_OBJECTPAIRITERATOR, 0, new QoreObjectIterator(obj));
}

// int PseudoObject::size(){}
static int64 PseudoObject_size(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 120 "Pseudo_QC_Object.qpp"
return obj->size(xsink);
}

// bool PseudoObject::sizep(){}
static bool PseudoObject_sizep(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 288 "Pseudo_QC_Object.qpp"
return true;
}

// int PseudoObject::typeCode(){}
static int64 PseudoObject_typeCode(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 48 "Pseudo_QC_Object.qpp"
return NT_OBJECT;
}

// bool PseudoObject::val(){}
static bool PseudoObject_val(QoreObject* self, QoreObject* obj, const QoreListNode* args, ExceptionSink* xsink) {
# 164 "Pseudo_QC_Object.qpp"
return (bool)obj->size(xsink);
}

DLLLOCAL QoreClass* initPseudoObjectClass() {
   QC_PSEUDOOBJECT = new QoreClass("<object>", QDOM_DEFAULT);
   CID_PSEUDOOBJECT = QC_PSEUDOOBJECT->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOOBJECT->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // string PseudoObject::className(){}
   QC_PSEUDOOBJECT->addMethodExtended3("className", (q_method_t)PseudoObject_className, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // bool PseudoObject::empty(){}
   QC_PSEUDOOBJECT->addMethodExtended3("empty", (q_method_bool_t)PseudoObject_empty, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

   // *string PseudoObject::firstKey(){}
   QC_PSEUDOOBJECT->addMethodExtended3("firstKey", (q_method_t)PseudoObject_firstKey, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // bool PseudoObject::hasCallableMethod(string name){}
   QC_PSEUDOOBJECT->addMethodExtended3("hasCallableMethod", (q_method_bool_t)PseudoObject_hasCallableMethod_Vs, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");

   // bool PseudoObject::hasCallableNormalMethod(string name){}
   QC_PSEUDOOBJECT->addMethodExtended3("hasCallableNormalMethod", (q_method_bool_t)PseudoObject_hasCallableNormalMethod_Vs, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");

   // bool PseudoObject::hasCallableStaticMethod(string name){}
   QC_PSEUDOOBJECT->addMethodExtended3("hasCallableStaticMethod", (q_method_bool_t)PseudoObject_hasCallableStaticMethod_Vs, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");

   // bool PseudoObject::isSystem(){}
   QC_PSEUDOOBJECT->addMethodExtended3("isSystem", (q_method_bool_t)PseudoObject_isSystem, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // AbstractIterator PseudoObject::iterator(){}
   QC_PSEUDOOBJECT->addMethodExtended3("iterator", (q_method_t)PseudoObject_iterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

   // ObjectKeyIterator PseudoObject::keyIterator(){}
   QC_PSEUDOOBJECT->addMethodExtended3("keyIterator", (q_method_t)PseudoObject_keyIterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_OBJECTKEYITERATOR->getTypeInfo());

   // list PseudoObject::keys(){}
   QC_PSEUDOOBJECT->addMethodExtended3("keys", (q_method_t)PseudoObject_keys, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo);

   // *string PseudoObject::lastKey(){}
   QC_PSEUDOOBJECT->addMethodExtended3("lastKey", (q_method_t)PseudoObject_lastKey, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // ObjectPairIterator PseudoObject::pairIterator(){}
   QC_PSEUDOOBJECT->addMethodExtended3("pairIterator", (q_method_t)PseudoObject_pairIterator, false, QC_CONSTANT, QDOM_DEFAULT, QC_OBJECTPAIRITERATOR->getTypeInfo());

   // int PseudoObject::size(){}
   QC_PSEUDOOBJECT->addMethodExtended3("size", (q_method_int64_t)PseudoObject_size, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoObject::sizep(){}
   QC_PSEUDOOBJECT->addMethodExtended3("sizep", (q_method_bool_t)PseudoObject_sizep, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoObject::typeCode(){}
   QC_PSEUDOOBJECT->addMethodExtended3("typeCode", (q_method_int64_t)PseudoObject_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoObject::val(){}
   QC_PSEUDOOBJECT->addMethodExtended3("val", (q_method_bool_t)PseudoObject_val, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDOOBJECT;
}
// bool <object>::hasKey(softstring $key) [flags=RET_VALUE_ONLY] {}
// bool <object>::hasKeyValue(softstring $key) [flags=RET_VALUE_ONLY] {}
