/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_String.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/ql_crypto.h>

#include <ctype.h>
/* Qore class Qore::<string> */

qore_classid_t CID_PSEUDOSTRING;
QoreClass* QC_PSEUDOSTRING;

// int PseudoString::comparePartial(string ostr){}
static int64 PseudoString_comparePartial_Vs(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* ostr = HARD_QORE_STRING(args, 0);
# 764 "Pseudo_QC_String.qpp"
// convert pattern encoding to string if necessary
   TempEncodingHelper temp(ostr, str->getEncoding(), xsink);
   if (*xsink)
      return 0;

   int rc = strncmp(str->getBuffer(), temp->getBuffer(), temp->size());
   return !rc ? 0 : (rc < 0 ? -1 : 1);
}

// bool PseudoString::empty(){}
static bool PseudoString_empty(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 123 "Pseudo_QC_String.qpp"
return str->empty();
}

// string PseudoString::encoding(){}
static AbstractQoreNode* PseudoString_encoding(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 138 "Pseudo_QC_String.qpp"
return new QoreStringNode(str->getEncoding()->getCode());
}

// bool PseudoString::equalPartial(string ostr){}
static bool PseudoString_equalPartial_Vs(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* ostr = HARD_QORE_STRING(args, 0);
# 788 "Pseudo_QC_String.qpp"
return str->equalPartialSoft(ostr, xsink);
}

// bool PseudoString::equalPartialPath(string ostr){}
static bool PseudoString_equalPartialPath_Vs(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* ostr = HARD_QORE_STRING(args, 0);
# 804 "Pseudo_QC_String.qpp"
return str->equalPartialPath(ostr, xsink);
}

// int PseudoString::find(softstring substr, softint pos = 0){}
static int64 PseudoString_find_vsvi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* substr = HARD_QORE_STRING(args, 0);
   int64 pos = HARD_QORE_INT(args, 1);
# 226 "Pseudo_QC_String.qpp"
return str->index(*substr, pos, xsink);
}

// *string PseudoString::getLine(int offset = 0, *string eol, bool trim = True, *reference size){}
static AbstractQoreNode* PseudoString_getLine_ViNsVbNr(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   int64 offset = HARD_QORE_INT(args, 0);
   const QoreStringNode* eol = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
   bool trim = HARD_QORE_BOOL(args, 2);
   const ReferenceNode* size = reinterpret_cast<const ReferenceNode*>(get_param(args, 3));
# 871 "Pseudo_QC_String.qpp"
if (str->empty())
      return 0;

   if (offset < 0) {
      offset = str->size() + offset;
      if (offset < 0)
         offset = 0;
   }
   else if (offset >= (int64)str->size())
      return 0;

   TempEncodingHelper eolstr;
   if (eol) {
      eolstr.set(eol, str->getEncoding(), xsink);
      if (*xsink)
         return 0;

      if (eolstr->size() > str->size())
         return 0;
   }

   int64 strsize = 0;

   const char* start = str->getBuffer() + offset;
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (eolstr) {
      const char* p = strstr(start, eolstr->getBuffer());
      if (p) {
         strsize = p - start + eolstr->size();
         if (trim) {
            if (p != start)
               rv->concat(start, p - start);
         }
         else
            rv->concat(start, strsize);
      }
      else {
         strsize = rv->size();
         rv->concat(start);
         strsize = rv->size() - strsize;
      }
   }
   else {
      while (*start) {
         if (*start == '\r') {
            ++strsize;
            if (!trim)
               rv->concat(*start);
            if (*(start + 1) == '\n') {
               ++strsize;
               if (!trim)
                  rv->concat(*start);
            }
            break;
         }
         if (*start == '\n') {
            ++strsize;
            if (!trim)
               rv->concat(*start);
            break;
         }
         rv->concat(*start);
         ++strsize;
         ++start;
      }
   }

   if (size) {
      QoreTypeSafeReferenceHelper r(size, xsink);
      if (!r)
         return 0;

      if (r.assign(new QoreBigIntNode(strsize), xsink))
         return 0;
   }

   return rv.release();
}

// int PseudoString::getUnicode(int offset = 0){}
static int64 PseudoString_getUnicode_Vi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   int64 offset = HARD_QORE_INT(args, 0);
# 841 "Pseudo_QC_String.qpp"
return str->getUnicodePoint(offset, xsink);
}

// bool PseudoString::intp(){}
static bool PseudoString_intp(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 282 "Pseudo_QC_String.qpp"
if (str->empty())
      return false;
   char c = (*str)[0];
   if (c == '-')
      c = (*str)[1];
   return (isdigit(c)) ? true : false;
}

// bool PseudoString::isDataAscii(){}
static bool PseudoString_isDataAscii(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 690 "Pseudo_QC_String.qpp"
return str->isDataAscii();
}

// bool PseudoString::isDataPrintableAscii(){}
static bool PseudoString_isDataPrintableAscii(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 704 "Pseudo_QC_String.qpp"
return str->isDataPrintableAscii();
}

// int PseudoString::length(){}
static int64 PseudoString_length(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 109 "Pseudo_QC_String.qpp"
return str->length();
}

// string PseudoString::lwr(){}
static AbstractQoreNode* PseudoString_lwr(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 163 "Pseudo_QC_String.qpp"
SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_tolower(*(*rv), *str, xsink))
      return 0;

   return rv.release();
}

// bool PseudoString::regex(string regex, int options = 0){}
static bool PseudoString_regex_VsVi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* regex = HARD_QORE_STRING(args, 0);
   int64 options = HARD_QORE_INT(args, 1);
# 436 "Pseudo_QC_String.qpp"
QoreRegexNode qr(*regex, options, xsink);
   if (*xsink)
      return 0;

   return qr.exec(str, xsink);
}

// *list PseudoString::regexExtract(string regex, int options = 0){}
static AbstractQoreNode* PseudoString_regexExtract_VsVi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* regex = HARD_QORE_STRING(args, 0);
   int64 options = HARD_QORE_INT(args, 1);
# 470 "Pseudo_QC_String.qpp"
QoreRegexNode qr(*regex, options, xsink);
   if (*xsink)
      return 0;

   return qr.extractSubstrings(str, xsink);
}

// int PseudoString::rfind(softstring substr, softint pos = -1){}
static int64 PseudoString_rfind_vsvi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* substr = HARD_QORE_STRING(args, 0);
   int64 pos = HARD_QORE_INT(args, 1);
# 256 "Pseudo_QC_String.qpp"
return str->rindex(*substr, (qore_offset_t)pos, xsink);
}

// int PseudoString::size(){}
static int64 PseudoString_size(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 88 "Pseudo_QC_String.qpp"
return str->size();
}

// bool PseudoString::sizep(){}
static bool PseudoString_sizep(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 852 "Pseudo_QC_String.qpp"
return true;
}

// list PseudoString::split(string sep, bool with_separator = False){}
static AbstractQoreNode* PseudoString_split_VsVb(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* sep = HARD_QORE_STRING(args, 0);
   bool with_separator = HARD_QORE_BOOL(args, 1);
# 375 "Pseudo_QC_String.qpp"
// convert pattern encoding to string if necessary
   TempEncodingHelper temp(sep, str->getEncoding(), xsink);
   if (*xsink)
      return 0;

   return split_intern(temp->getBuffer(), temp->strlen(), str->getBuffer(), str->strlen(), str->getEncoding(), with_separator);
}

// list PseudoString::split(string sep, string quote, bool trim_unquoted = False){}
static AbstractQoreNode* PseudoString_split_VsVsVb(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* sep = HARD_QORE_STRING(args, 0);
   const QoreStringNode* quote = HARD_QORE_STRING(args, 1);
   bool trim_unquoted = HARD_QORE_BOOL(args, 2);
# 409 "Pseudo_QC_String.qpp"
return split_with_quote(sep, str, quote, trim_unquoted, xsink);
}

// int PseudoString::strlen(){}
static int64 PseudoString_strlen(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 67 "Pseudo_QC_String.qpp"
return str->strlen();
}

// bool PseudoString::strp(){}
static bool PseudoString_strp(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 301 "Pseudo_QC_String.qpp"
return true;
}

// string PseudoString::substr(softint start){}
static AbstractQoreNode* PseudoString_substr_vi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
# 322 "Pseudo_QC_String.qpp"
QoreStringNode* rv = str->substr(start, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// string PseudoString::substr(softint start, softint len){}
static AbstractQoreNode* PseudoString_substr_vivi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
   int64 len = HARD_QORE_INT(args, 1);
# 349 "Pseudo_QC_String.qpp"
QoreStringNode* rv = str->substr(start, len, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// string PseudoString::toBase64(softint maxlinelen = -1){}
static AbstractQoreNode* PseudoString_toBase64_vi(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
   int64 maxlinelen = HARD_QORE_INT(args, 0);
# 746 "Pseudo_QC_String.qpp"
QoreStringNode *rv = new QoreStringNode;
   rv->concatBase64(str, maxlinelen);
   return rv;
}

// string PseudoString::toHex(){}
static AbstractQoreNode* PseudoString_toHex(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 722 "Pseudo_QC_String.qpp"
QoreStringNode *rv = new QoreStringNode;
   rv->concatHex(str);
   return rv;
}

// string PseudoString::toMD5(){}
static AbstractQoreNode* PseudoString_toMD5(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 500 "Pseudo_QC_String.qpp"
DigestHelper dh(*str);
   if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
      return 0;

   return dh.getString();
}

// string PseudoString::toSHA1(){}
static AbstractQoreNode* PseudoString_toSHA1(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 528 "Pseudo_QC_String.qpp"
DigestHelper dh(*str);
   if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
      return 0;

   return dh.getString();
}

// string PseudoString::toSHA224(){}
static AbstractQoreNode* PseudoString_toSHA224(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 559 "Pseudo_QC_String.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA224", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoString::toSHA256(){}
static AbstractQoreNode* PseudoString_toSHA256(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 595 "Pseudo_QC_String.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA256", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoString::toSHA384(){}
static AbstractQoreNode* PseudoString_toSHA384(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 631 "Pseudo_QC_String.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA384", xsink);
   return 0;
#endif
   return 0;
}

// string PseudoString::toSHA512(){}
static AbstractQoreNode* PseudoString_toSHA512(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 667 "Pseudo_QC_String.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
      return 0;

   return dh.getString();
#else
   missing_openssl_feature("SHA512", xsink);
   return 0;
#endif
   return 0;
}

// int PseudoString::typeCode(){}
static int64 PseudoString_typeCode(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 48 "Pseudo_QC_String.qpp"
return NT_STRING;
}

// string PseudoString::unaccent(){}
static AbstractQoreNode* PseudoString_unaccent(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 821 "Pseudo_QC_String.qpp"
SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_unaccent(*(*rv), *str, xsink))
      return 0;

   return rv.release();
}

// string PseudoString::upr(){}
static AbstractQoreNode* PseudoString_upr(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 192 "Pseudo_QC_String.qpp"
SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_toupper(*(*rv), *str, xsink))
      return 0;

   return rv.release();
}

// bool PseudoString::val(){}
static bool PseudoString_val(QoreObject* self, const QoreStringNode* str, const QoreListNode* args, ExceptionSink* xsink) {
# 268 "Pseudo_QC_String.qpp"
return !str->empty();
}

DLLLOCAL QoreClass* initPseudoStringClass() {
   QC_PSEUDOSTRING = new QoreClass("<string>", QDOM_DEFAULT);
   CID_PSEUDOSTRING = QC_PSEUDOSTRING->getID();

   // set parent class
   assert(QC_PSEUDOVALUE);
   QC_PSEUDOSTRING->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

   // int PseudoString::comparePartial(string ostr){}
   QC_PSEUDOSTRING->addMethodExtended3("comparePartial", (q_method_int64_t)PseudoString_comparePartial_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, NULL, "ostr");

   // bool PseudoString::empty(){}
   QC_PSEUDOSTRING->addMethodExtended3("empty", (q_method_bool_t)PseudoString_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // string PseudoString::encoding(){}
   QC_PSEUDOSTRING->addMethodExtended3("encoding", (q_method_t)PseudoString_encoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // bool PseudoString::equalPartial(string ostr){}
   QC_PSEUDOSTRING->addMethodExtended3("equalPartial", (q_method_bool_t)PseudoString_equalPartial_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "ostr");

   // bool PseudoString::equalPartialPath(string ostr){}
   QC_PSEUDOSTRING->addMethodExtended3("equalPartialPath", (q_method_bool_t)PseudoString_equalPartialPath_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "ostr");

   // int PseudoString::find(softstring substr, softint pos = 0){}
   QC_PSEUDOSTRING->addMethodExtended3("find", (q_method_int64_t)PseudoString_find_vsvi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, zero(), "pos");

   // *string PseudoString::getLine(int offset = 0, *string eol, bool trim = True, *reference size){}
   QC_PSEUDOSTRING->addMethodExtended3("getLine", (q_method_t)PseudoString_getLine_ViNsVbNr, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, bigIntTypeInfo, zero(), "offset", stringOrNothingTypeInfo, NULL, "eol", boolTypeInfo, &True, "trim", anyTypeInfo, NULL, "size");

   // int PseudoString::getUnicode(int offset = 0){}
   QC_PSEUDOSTRING->addMethodExtended3("getUnicode", (q_method_int64_t)PseudoString_getUnicode_Vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, zero(), "offset");

   // bool PseudoString::intp(){}
   QC_PSEUDOSTRING->addMethodExtended3("intp", (q_method_bool_t)PseudoString_intp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool PseudoString::isDataAscii(){}
   QC_PSEUDOSTRING->addMethodExtended3("isDataAscii", (q_method_bool_t)PseudoString_isDataAscii, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool PseudoString::isDataPrintableAscii(){}
   QC_PSEUDOSTRING->addMethodExtended3("isDataPrintableAscii", (q_method_bool_t)PseudoString_isDataPrintableAscii, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int PseudoString::length(){}
   QC_PSEUDOSTRING->addMethodExtended3("length", (q_method_int64_t)PseudoString_length, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // string PseudoString::lwr(){}
   QC_PSEUDOSTRING->addMethodExtended3("lwr", (q_method_t)PseudoString_lwr, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // bool PseudoString::regex(string regex, int options = 0){}
   QC_PSEUDOSTRING->addMethodExtended3("regex", (q_method_bool_t)PseudoString_regex_VsVi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, stringTypeInfo, NULL, "regex", bigIntTypeInfo, zero(), "options");

   // *list PseudoString::regexExtract(string regex, int options = 0){}
   QC_PSEUDOSTRING->addMethodExtended3("regexExtract", (q_method_t)PseudoString_regexExtract_VsVi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listOrNothingTypeInfo, 2, stringTypeInfo, NULL, "regex", bigIntTypeInfo, zero(), "options");

   // int PseudoString::rfind(softstring substr, softint pos = -1){}
   QC_PSEUDOSTRING->addMethodExtended3("rfind", (q_method_int64_t)PseudoString_rfind_vsvi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, new QoreBigIntNode(-1), "pos");

   // int PseudoString::size(){}
   QC_PSEUDOSTRING->addMethodExtended3("size", (q_method_int64_t)PseudoString_size, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoString::sizep(){}
   QC_PSEUDOSTRING->addMethodExtended3("sizep", (q_method_bool_t)PseudoString_sizep, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // list PseudoString::split(string sep, bool with_separator = False){}
   QC_PSEUDOSTRING->addMethodExtended3("split", (q_method_t)PseudoString_split_VsVb, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, stringTypeInfo, NULL, "sep", boolTypeInfo, &False, "with_separator");

   // list PseudoString::split(string sep, string quote, bool trim_unquoted = False){}
   QC_PSEUDOSTRING->addMethodExtended3("split", (q_method_t)PseudoString_split_VsVsVb, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "sep", stringTypeInfo, NULL, "quote", boolTypeInfo, &False, "trim_unquoted");

   // int PseudoString::strlen(){}
   QC_PSEUDOSTRING->addMethodExtended3("strlen", (q_method_int64_t)PseudoString_strlen, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool PseudoString::strp(){}
   QC_PSEUDOSTRING->addMethodExtended3("strp", (q_method_bool_t)PseudoString_strp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // string PseudoString::substr(softint start){}
   QC_PSEUDOSTRING->addMethodExtended3("substr", (q_method_t)PseudoString_substr_vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, NULL, "start");

   // string PseudoString::substr(softint start, softint len){}
   QC_PSEUDOSTRING->addMethodExtended3("substr", (q_method_t)PseudoString_substr_vivi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len");

   // string PseudoString::toBase64(softint maxlinelen = -1){}
   QC_PSEUDOSTRING->addMethodExtended3("toBase64", (q_method_t)PseudoString_toBase64_vi, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");

   // string PseudoString::toHex(){}
   QC_PSEUDOSTRING->addMethodExtended3("toHex", (q_method_t)PseudoString_toHex, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::toMD5(){}
   QC_PSEUDOSTRING->addMethodExtended3("toMD5", (q_method_t)PseudoString_toMD5, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::toSHA1(){}
   QC_PSEUDOSTRING->addMethodExtended3("toSHA1", (q_method_t)PseudoString_toSHA1, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::toSHA224(){}
   QC_PSEUDOSTRING->addMethodExtended3("toSHA224", (q_method_t)PseudoString_toSHA224, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::toSHA256(){}
   QC_PSEUDOSTRING->addMethodExtended3("toSHA256", (q_method_t)PseudoString_toSHA256, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::toSHA384(){}
   QC_PSEUDOSTRING->addMethodExtended3("toSHA384", (q_method_t)PseudoString_toSHA384, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::toSHA512(){}
   QC_PSEUDOSTRING->addMethodExtended3("toSHA512", (q_method_t)PseudoString_toSHA512, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // int PseudoString::typeCode(){}
   QC_PSEUDOSTRING->addMethodExtended3("typeCode", (q_method_int64_t)PseudoString_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // string PseudoString::unaccent(){}
   QC_PSEUDOSTRING->addMethodExtended3("unaccent", (q_method_t)PseudoString_unaccent, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // string PseudoString::upr(){}
   QC_PSEUDOSTRING->addMethodExtended3("upr", (q_method_t)PseudoString_upr, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // bool PseudoString::val(){}
   QC_PSEUDOSTRING->addMethodExtended3("val", (q_method_bool_t)PseudoString_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_PSEUDOSTRING;
}
