/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractDatasource.qpp AbstractDatasource class (interface) definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2014 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>

class AbstractDatasource {};
/* Qore class Qore::SQL::AbstractDatasource */

qore_classid_t CID_ABSTRACTDATASOURCE;
QoreClass* QC_ABSTRACTDATASOURCE;

// bool AbstractDatasource::currentThreadInTransaction(){}
static bool AbstractDatasource_currentThreadInTransaction(QoreObject* self, AbstractDatasource* ds, const QoreListNode* args, ExceptionSink* xsink) {
# 401 "QC_AbstractDatasource.qpp"
xsink->raiseException("ABSTRACT-DATASOURCE-ERROR", "This method must be re-implemented in subclasses to provide the desired functionality; it is implemented in Datasource and DatasourcePool for example");
   return false;
}

DLLLOCAL QoreClass* initAbstractDatasourceClass(QoreNamespace& ns) {
   QC_ABSTRACTDATASOURCE = new QoreClass("AbstractDatasource", QDOM_DATABASE);
   CID_ABSTRACTDATASOURCE = QC_ABSTRACTDATASOURCE->getID();

   // nothing AbstractDatasource::beginTransaction();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("beginTransaction", false, QC_NO_FLAGS, nothingTypeInfo);

   // nothing AbstractDatasource::commit();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("commit", false, QC_NO_FLAGS, nothingTypeInfo);

   // bool AbstractDatasource::currentThreadInTransaction(){}
   QC_ABSTRACTDATASOURCE->addMethodExtended3("currentThreadInTransaction", (q_method_bool_t)AbstractDatasource_currentThreadInTransaction, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // any AbstractDatasource::exec(string sql, ...);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("exec", false, QC_USES_EXTRA_ARGS, anyTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // any AbstractDatasource::execRaw(string sql);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("execRaw", false, QC_NO_FLAGS, anyTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // any AbstractDatasource::getClientVersion();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getClientVersion", false, QC_NO_FLAGS, anyTypeInfo);

   // hash AbstractDatasource::getConfigHash();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getConfigHash", false, QC_NO_FLAGS, hashTypeInfo);

   // string AbstractDatasource::getConfigString();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getConfigString", false, QC_NO_FLAGS, stringTypeInfo);

   // string AbstractDatasource::getDBEncoding();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getDBEncoding", false, QC_NO_FLAGS, stringTypeInfo);

   // *string AbstractDatasource::getDBName();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getDBName", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

   // string AbstractDatasource::getDriverName();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getDriverName", false, QC_NO_FLAGS, stringTypeInfo);

   // *string AbstractDatasource::getHostName();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getHostName", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

   // *string AbstractDatasource::getOSEncoding();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getOSEncoding", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

   // *string AbstractDatasource::getPassword();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getPassword", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

   // *int AbstractDatasource::getPort();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getPort", false, QC_NO_FLAGS, bigIntOrNothingTypeInfo);

   // any AbstractDatasource::getServerVersion();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getServerVersion", false, QC_NO_FLAGS, anyTypeInfo);

   // *string AbstractDatasource::getUserName();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getUserName", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

   // bool AbstractDatasource::inTransaction();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("inTransaction", false, QC_NO_FLAGS, boolTypeInfo);

   // nothing AbstractDatasource::rollback();
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("rollback", false, QC_NO_FLAGS, nothingTypeInfo);

   // any AbstractDatasource::select(string sql, ...);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("select", false, QC_USES_EXTRA_ARGS, anyTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // any AbstractDatasource::selectRow(string sql, ...);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("selectRow", false, QC_USES_EXTRA_ARGS, anyTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // any AbstractDatasource::selectRows(string sql, ...);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("selectRows", false, QC_USES_EXTRA_ARGS, anyTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // any AbstractDatasource::vexec(string sql, *softlist vargs);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vexec", false, QC_NO_FLAGS, anyTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

   // any AbstractDatasource::vselect(string sql, *softlist vargs);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vselect", false, QC_NO_FLAGS, anyTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

   // any AbstractDatasource::vselectRow(string sql, *softlist vargs);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vselectRow", false, QC_NO_FLAGS, anyTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

   // any AbstractDatasource::vselectRows(string sql, *softlist vargs);
   QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vselectRows", false, QC_NO_FLAGS, anyTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

   return QC_ABSTRACTDATASOURCE;
}
