/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
 QC_AutoLock.qpp
 
 Qore Programming Language
 
 Copyright 2003 - 2013 David Nichols
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <qore/Qore.h>
#include <qore/intern/QC_AutoLock.h>
#include <qore/intern/QC_Mutex.h>
/* Qore class Qore::Thread::AutoLock */

qore_classid_t CID_AUTOLOCK;
QoreClass* QC_AUTOLOCK;

// AutoLock::constructor(Mutex mutex) {}
static void AutoLock_constructor_C5Mutex(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(mutex, SmartMutex, args, 0, CID_MUTEX, "AutoLock::constructor()", "Mutex", xsink);
   if (*xsink)
      return;
# 74 "QC_AutoLock.qpp"
QoreAutoLock *qsl = new QoreAutoLock(mutex, xsink);
   if (*xsink)
      qsl->deref(xsink);
   else
      self->setPrivate(CID_AUTOLOCK, qsl);
}

// AutoLock::copy() {}
static void AutoLock_copy(QoreObject* self, QoreObject* old, QoreAutoLock* al, ExceptionSink* xsink) {
# 99 "QC_AutoLock.qpp"
xsink->raiseException("AUTOLOCK-COPY-ERROR", "objects of this class cannot be copied");
}

// AutoLock::destructor() {}
static void AutoLock_destructor(QoreObject* self, QoreAutoLock* al, ExceptionSink* xsink) {
# 90 "QC_AutoLock.qpp"
al->destructor(xsink);
   al->deref(xsink);
}

// nothing AutoLock::lock(){}
static AbstractQoreNode* AutoLock_lock(QoreObject* self, QoreAutoLock* al, const QoreListNode* args, ExceptionSink* xsink) {
# 114 "QC_AutoLock.qpp"
al->lock(xsink);
   return 0;
}

// int AutoLock::trylock(){}
static int64 AutoLock_trylock(QoreObject* self, QoreAutoLock* al, const QoreListNode* args, ExceptionSink* xsink) {
# 130 "QC_AutoLock.qpp"
return al->trylock();
}

// nothing AutoLock::unlock(){}
static AbstractQoreNode* AutoLock_unlock(QoreObject* self, QoreAutoLock* al, const QoreListNode* args, ExceptionSink* xsink) {
# 143 "QC_AutoLock.qpp"
al->unlock(xsink);
   return 0;
}

DLLLOCAL QoreClass* initAutoLockClass(QoreNamespace& ns) {
   QC_AUTOLOCK = new QoreClass("AutoLock", QDOM_THREAD_CLASS);
   CID_AUTOLOCK = QC_AUTOLOCK->getID();

   // AutoLock::constructor(Mutex mutex) {}
   QC_AUTOLOCK->setConstructorExtended3(AutoLock_constructor_C5Mutex, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, QC_MUTEX->getTypeInfo(), NULL, "mutex");

   // AutoLock::copy() {}
   QC_AUTOLOCK->setCopy((q_copy_t)AutoLock_copy);

   // AutoLock::destructor() {}
   QC_AUTOLOCK->setDestructor((q_destructor_t)AutoLock_destructor);

   // nothing AutoLock::lock(){}
   QC_AUTOLOCK->addMethodExtended3("lock", (q_method_t)AutoLock_lock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int AutoLock::trylock(){}
   QC_AUTOLOCK->addMethodExtended3("trylock", (q_method_int64_t)AutoLock_trylock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // nothing AutoLock::unlock(){}
   QC_AUTOLOCK->addMethodExtended3("unlock", (q_method_t)AutoLock_unlock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   return QC_AUTOLOCK;
}
