/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
 QC_AutoReadLock.qpp
 
 Qore Programming Language
 
 Copyright 2003 - 2013 David Nichols
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <qore/Qore.h>
#include <qore/intern/QC_AutoReadLock.h>
#include <qore/intern/QC_RWLock.h>
/* Qore class Qore::Thread::AutoReadLock */

qore_classid_t CID_AUTOREADLOCK;
QoreClass* QC_AUTOREADLOCK;

// AutoReadLock::constructor(RWLock rwl) {}
static void AutoReadLock_constructor_C6RWLock(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(rwl, RWLock, args, 0, CID_RWLOCK, "AutoReadLock::constructor()", "RWLock", xsink);
   if (*xsink)
      return;
# 65 "QC_AutoReadLock.qpp"
ReferenceHolder<QoreAutoReadLock> arwl(new QoreAutoReadLock(rwl, xsink), xsink);
   if (!*xsink)
      self->setPrivate(CID_AUTOREADLOCK, arwl.release());
}

// AutoReadLock::copy() {}
static void AutoReadLock_copy(QoreObject* self, QoreObject* old, QoreAutoReadLock* arwl, ExceptionSink* xsink) {
# 89 "QC_AutoReadLock.qpp"
xsink->raiseException("AUTOREADLOCK-COPY-ERROR", "objects of this class cannot be copied");
}

// AutoReadLock::destructor() {}
static void AutoReadLock_destructor(QoreObject* self, QoreAutoReadLock* arwl, ExceptionSink* xsink) {
# 80 "QC_AutoReadLock.qpp"
arwl->destructor(xsink);
   arwl->deref(xsink);
}

DLLLOCAL QoreClass* initAutoReadLockClass(QoreNamespace& ns) {
   QC_AUTOREADLOCK = new QoreClass("AutoReadLock", QDOM_THREAD_CLASS);
   CID_AUTOREADLOCK = QC_AUTOREADLOCK->getID();

   // AutoReadLock::constructor(RWLock rwl) {}
   QC_AUTOREADLOCK->setConstructorExtended3(AutoReadLock_constructor_C6RWLock, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, QC_RWLOCK->getTypeInfo(), NULL, "rwl");

   // AutoReadLock::copy() {}
   QC_AUTOREADLOCK->setCopy((q_copy_t)AutoReadLock_copy);

   // AutoReadLock::destructor() {}
   QC_AUTOREADLOCK->setDestructor((q_destructor_t)AutoReadLock_destructor);

   return QC_AUTOREADLOCK;
}
