/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Condition.qpp

  Qore Programming Language
  
  Copyright (C) 2003 - 2014 David Nichols
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_Condition.h>
#include <qore/intern/QC_AbstractSmartLock.h>
#include <qore/ReferenceHolder.h>

#include <errno.h>
/* Qore class Qore::Thread::Condition */

qore_classid_t CID_CONDITION;
QoreClass* QC_CONDITION;

// nothing Condition::broadcast(){}
static AbstractQoreNode* Condition_broadcast(QoreObject* self, Condition* c, const QoreListNode* args, ExceptionSink* xsink) {
# 90 "QC_Condition.qpp"
if (c->broadcast())
      xsink->raiseException("CONDITION-BROADCAST-ERROR", q_strerror(errno));
   return 0;
}

// Condition::constructor() {}
static void Condition_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 46 "QC_Condition.qpp"
self->setPrivate(CID_CONDITION, new Condition);
}

// Condition::copy() {}
static void Condition_copy(QoreObject* self, QoreObject* old, Condition* c, ExceptionSink* xsink) {
# 57 "QC_Condition.qpp"
self->setPrivate(CID_CONDITION, new Condition);
}

// nothing Condition::signal(){}
static AbstractQoreNode* Condition_signal(QoreObject* self, Condition* c, const QoreListNode* args, ExceptionSink* xsink) {
# 73 "QC_Condition.qpp"
if (c->signal())
      xsink->raiseException("CONDITION-SIGNAL-ERROR", q_strerror(errno));
   return 0;
}

// int Condition::wait(AbstractSmartLock lock, timeout timeout_ms = 0){}
static int64 Condition_wait_C17AbstractSmartLockVt(QoreObject* self, Condition* c, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(lock, AbstractSmartLock, args, 0, CID_ABSTRACTSMARTLOCK, "Condition::wait()", "AbstractSmartLock", xsink);
   if (*xsink)
      return 0;
   int64 timeout_ms = HARD_QORE_INT(args, 1);
# 115 "QC_Condition.qpp"
ReferenceHolder<AbstractSmartLock> holder(lock, xsink);
   
   int rc = timeout_ms >= 0 ? c->wait(lock, timeout_ms, xsink) : c->wait(lock, xsink);

   //printd(5, "Condition::wait() m=%s (%p) timeout=%d rc=%d\n", lock->getName(), lock, timeout, rc);
      
   if (rc && rc != ETIMEDOUT && !*xsink)
      xsink->raiseErrnoException("CONDITION-WAIT-ERROR", rc, "unknown system error code returned from Condition::wait(lock=%s, timeout=%d): rc=%d", lock->getName(), timeout_ms, rc);
   return rc;
}

// int Condition::wait_count(AbstractSmartLock lock){}
static int64 Condition_wait_count_C17AbstractSmartLock(QoreObject* self, Condition* c, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(lock, AbstractSmartLock, args, 0, CID_ABSTRACTSMARTLOCK, "Condition::wait_count()", "AbstractSmartLock", xsink);
   if (*xsink)
      return 0;
# 138 "QC_Condition.qpp"
ReferenceHolder<AbstractSmartLock> holder(lock, xsink);

   return c->wait_count(lock);
}

DLLLOCAL QoreClass* initConditionClass(QoreNamespace& ns) {
   QC_CONDITION = new QoreClass("Condition", QDOM_THREAD_CLASS);
   CID_CONDITION = QC_CONDITION->getID();

   // nothing Condition::broadcast(){}
   QC_CONDITION->addMethodExtended3("broadcast", (q_method_t)Condition_broadcast, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // Condition::constructor() {}
   QC_CONDITION->setConstructorExtended3(Condition_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // Condition::copy() {}
   QC_CONDITION->setCopy((q_copy_t)Condition_copy);

   // nothing Condition::signal(){}
   QC_CONDITION->addMethodExtended3("signal", (q_method_t)Condition_signal, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int Condition::wait(AbstractSmartLock lock, timeout timeout_ms = 0){}
   QC_CONDITION->addMethodExtended3("wait", (q_method_int64_t)Condition_wait_C17AbstractSmartLockVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, QC_ABSTRACTSMARTLOCK->getTypeInfo(), NULL, "lock", timeoutTypeInfo, zero(), "timeout_ms");

   // int Condition::wait_count(AbstractSmartLock lock){}
   QC_CONDITION->addMethodExtended3("wait_count", (q_method_int64_t)Condition_wait_count_C17AbstractSmartLock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, QC_ABSTRACTSMARTLOCK->getTypeInfo(), NULL, "lock");

   return QC_CONDITION;
}
