/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Counter.cpp

  Qore Programming Language
  
  Copyright 2003 - 2013 David Nichols
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_Counter.h>
/* Qore class Qore::Thread::Counter */

qore_classid_t CID_COUNTER;
QoreClass* QC_COUNTER;

// Counter::constructor(softint c = 0) {}
static void Counter_constructor_vi(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   int64 c = HARD_QORE_INT(args, 0);
# 45 "QC_Counter.qpp"
if (c < 0) {
      xsink->raiseException("COUNTER-ERROR", "Counter::constructor() called with initial value "QLLD", however value must be non-negative", c);
      return;
   }
   self->setPrivate(CID_COUNTER, new Counter(c));
}

// Counter::copy() {}
static void Counter_copy(QoreObject* self, QoreObject* old, Counter* c, ExceptionSink* xsink) {
# 75 "QC_Counter.qpp"
self->setPrivate(CID_COUNTER, new Counter(c->getCount()));
}

// nothing Counter::dec(){}
static AbstractQoreNode* Counter_dec(QoreObject* self, Counter* c, const QoreListNode* args, ExceptionSink* xsink) {
# 100 "QC_Counter.qpp"
c->dec(xsink);
   return 0;
}

// Counter::destructor() {}
static void Counter_destructor(QoreObject* self, Counter* c, ExceptionSink* xsink) {
# 63 "QC_Counter.qpp"
c->destructor(xsink);
   c->deref(xsink);
}

// int Counter::getCount(){}
static int64 Counter_getCount(QoreObject* self, Counter* c, const QoreListNode* args, ExceptionSink* xsink) {
# 142 "QC_Counter.qpp"
return c->getCount();
}

// int Counter::getWaiting(){}
static int64 Counter_getWaiting(QoreObject* self, Counter* c, const QoreListNode* args, ExceptionSink* xsink) {
# 154 "QC_Counter.qpp"
return c->getWaiting();
}

// nothing Counter::inc(){}
static AbstractQoreNode* Counter_inc(QoreObject* self, Counter* c, const QoreListNode* args, ExceptionSink* xsink) {
# 86 "QC_Counter.qpp"
c->inc();
   return 0;
}

// nothing Counter::waitForZero(){}
static AbstractQoreNode* Counter_waitForZero(QoreObject* self, Counter* c, const QoreListNode* args, ExceptionSink* xsink) {
# 113 "QC_Counter.qpp"
c->waitForZero(xsink);
   return 0;
}

// int Counter::waitForZero(timeout timeout_ms){}
static int64 Counter_waitForZero_Vt(QoreObject* self, Counter* c, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 130 "QC_Counter.qpp"
return c->waitForZero(xsink, timeout_ms);
}

DLLLOCAL QoreClass* initCounterClass(QoreNamespace& ns) {
   QC_COUNTER = new QoreClass("Counter", QDOM_THREAD_CLASS);
   CID_COUNTER = QC_COUNTER->getID();

   // Counter::constructor(softint c = 0) {}
   QC_COUNTER->setConstructorExtended3(Counter_constructor_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softBigIntTypeInfo, zero(), "c");

   // Counter::copy() {}
   QC_COUNTER->setCopy((q_copy_t)Counter_copy);

   // nothing Counter::dec(){}
   QC_COUNTER->addMethodExtended3("dec", (q_method_t)Counter_dec, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // Counter::destructor() {}
   QC_COUNTER->setDestructor((q_destructor_t)Counter_destructor);

   // int Counter::getCount(){}
   QC_COUNTER->addMethodExtended3("getCount", (q_method_int64_t)Counter_getCount, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int Counter::getWaiting(){}
   QC_COUNTER->addMethodExtended3("getWaiting", (q_method_int64_t)Counter_getWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // nothing Counter::inc(){}
   QC_COUNTER->addMethodExtended3("inc", (q_method_t)Counter_inc, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing Counter::waitForZero(){}
   QC_COUNTER->addMethodExtended3("waitForZero", (q_method_t)Counter_waitForZero, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int Counter::waitForZero(timeout timeout_ms){}
   QC_COUNTER->addMethodExtended3("waitForZero", (q_method_int64_t)Counter_waitForZero_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

   return QC_COUNTER;
}
