/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Dir.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_Dir.h>
#include <qore/intern/QC_File.h>

#include <sys/types.h>
#include <errno.h>
/* Qore class Qore::Dir */

qore_classid_t CID_DIR;
QoreClass* QC_DIR;

// bool Dir::chdir(string path){}
static bool Dir_chdir_Vs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
# 90 "QC_Dir.qpp"
return !d->chdir(path->getBuffer(), xsink);
}

// nothing Dir::chgrp(int gid){}
static AbstractQoreNode* Dir_chgrp_Vi(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   int64 gid = HARD_QORE_INT(args, 0);
# 263 "QC_Dir.qpp"
#ifdef HAVE_CHOWN
   d->chown((uid_t)-1, (gid_t)gid, xsink);
   return 0;
#else
   return missing_method_error("Dir::chgrp", "UNIX_FILEMGT", xsink);
#endif
   return 0;
}

// nothing Dir::chgrp(string groupname){}
static AbstractQoreNode* Dir_chgrp_Vs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* groupname = HARD_QORE_STRING(args, 0);
# 295 "QC_Dir.qpp"
#ifdef HAVE_CHOWN
   gid_t gid;

   // Try getting GID for name
   int rc = q_gname2gid(groupname->getBuffer(), gid);
   if (rc)
      xsink->raiseException("DIR-CHGRP-PARAMETER-ERROR", "no groupid found for group '%s'", groupname->getBuffer());
   else
      d->chown((uid_t)-1, gid, xsink);
   return 0;
#else
   return missing_method_error("Dir::chgrp", "UNIX_FILEMGT", xsink);
#endif
   return 0;
}

// nothing Dir::chmod(softint mode){}
static AbstractQoreNode* Dir_chmod_vi(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   int64 mode = HARD_QORE_INT(args, 0);
# 167 "QC_Dir.qpp"
d->chmod((int)mode, xsink);
   return 0;
}

// nothing Dir::chown(int uid){}
static AbstractQoreNode* Dir_chown_Vi(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   int64 uid = HARD_QORE_INT(args, 0);
# 193 "QC_Dir.qpp"
#ifdef HAVE_CHOWN
    d->chown((uid_t)uid, (gid_t)-1, xsink);
   return 0;
#else
   return missing_method_error("Dir::chown", "UNIX_FILEMGT", xsink);
#endif
   return 0;
}

// nothing Dir::chown(string username){}
static AbstractQoreNode* Dir_chown_Vs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* username = HARD_QORE_STRING(args, 0);
# 225 "QC_Dir.qpp"
#ifdef HAVE_CHOWN
   uid_t uid;

   // Try getting UID for username
   int rc = q_uname2uid(username->getBuffer(), uid);
   if (rc)
      xsink->raiseException("DIR-CHOWN-PARAMETER-ERROR", "no userid found for user '%s'", username->getBuffer());
   else
      d->chown(uid, (gid_t)-1, xsink);
   return 0;
#else
   return missing_method_error("Dir::chown", "UNIX_FILEMGT", xsink);
#endif
   return 0;
}

// Dir::constructor(*string encoding) {}
static void Dir_constructor_Ns(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 0));
# 51 "QC_Dir.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   SimpleRefHolder<Dir> d(new Dir(xsink, qe));
   if (*xsink)
      return;
   self->setPrivate(CID_DIR, d.release());
}

// Dir::copy() {}
static void Dir_copy(QoreObject* self, QoreObject* old, Dir* d, ExceptionSink* xsink) {
# 66 "QC_Dir.qpp"
SimpleRefHolder<Dir> nd(new Dir(xsink, *d));
   if (*xsink)
      return;
  
   self->setPrivate(CID_DIR, nd.release());
}

// int Dir::create(softint mode = 0777){}
static int64 Dir_create_vi(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   int64 mode = HARD_QORE_INT(args, 0);
# 144 "QC_Dir.qpp"
return d->create((int)mode, xsink); // throws exception
}

// bool Dir::exists(){}
static bool Dir_exists(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
# 117 "QC_Dir.qpp"
return !d->checkPath();
}

// hash Dir::hstat(){}
static AbstractQoreNode* Dir_hstat(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
# 704 "QC_Dir.qpp"
return d->hstat(xsink);
}

// list Dir::list(bool full = False){}
static AbstractQoreNode* Dir_list_Vb(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   bool full = HARD_QORE_BOOL(args, 0);
# 389 "QC_Dir.qpp"
return d->list(xsink, -1, 0, 0, full);
}

// list Dir::list(string regex, softint regex_options = 0, softbool full = False){}
static AbstractQoreNode* Dir_list_Vsvivb(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* regex = HARD_QORE_STRING(args, 0);
   int64 regex_options = HARD_QORE_INT(args, 1);
   bool full = HARD_QORE_BOOL(args, 2);
# 422 "QC_Dir.qpp"
return d->list(xsink, -1, regex, (int)regex_options, full);
}

// list Dir::listDirs(bool full = False){}
static AbstractQoreNode* Dir_listDirs_Vb(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   bool full = HARD_QORE_BOOL(args, 0);
# 514 "QC_Dir.qpp"
return d->list(xsink, S_IFDIR, 0, 0, full);
}

// list Dir::listDirs(string regex, softint regex_options = 0, softbool full = False){}
static AbstractQoreNode* Dir_listDirs_Vsvivb(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* regex = HARD_QORE_STRING(args, 0);
   int64 regex_options = HARD_QORE_INT(args, 1);
   bool full = HARD_QORE_BOOL(args, 2);
# 547 "QC_Dir.qpp"
return d->list(xsink, S_IFDIR, regex, (int)regex_options, full);
}

// list Dir::listFiles(bool full = False){}
static AbstractQoreNode* Dir_listFiles_Vb(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   bool full = HARD_QORE_BOOL(args, 0);
# 452 "QC_Dir.qpp"
return d->list(xsink, S_IFMT^S_IFDIR, 0, 0, full);
}

// list Dir::listFiles(string regex, softint regex_options = 0, softbool full = False){}
static AbstractQoreNode* Dir_listFiles_Vsvivb(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* regex = HARD_QORE_STRING(args, 0);
   int64 regex_options = HARD_QORE_INT(args, 1);
   bool full = HARD_QORE_BOOL(args, 2);
# 485 "QC_Dir.qpp"
return d->list(xsink, S_IFMT^S_IFDIR, regex, (int)regex_options, full);
}

// nothing Dir::mkdir(string subdir, softint mode = 0777){}
static AbstractQoreNode* Dir_mkdir_Vsvi(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* subdir = HARD_QORE_STRING(args, 0);
   int64 mode = HARD_QORE_INT(args, 1);
# 331 "QC_Dir.qpp"
// check if there is a path delimiter
   if (strchr(subdir->getBuffer(), QORE_DIR_SEP))
      xsink->raiseException("DIR-MKDIR-PARAMETER-ERROR", "only single, direct subdirectories are allowed");
   else
      d->mkdir(xsink, subdir->getBuffer(), (int)mode);
   return 0;
}

// Dir Dir::openDir(string subdir, *string encoding){}
static QoreObject* Dir_openDir_VsNs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* subdir = HARD_QORE_STRING(args, 0);
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 603 "QC_Dir.qpp"
// check if there is a path delimiter in
   if (strchr(subdir->getBuffer(), QORE_DIR_SEP)) {
      xsink->raiseException("DIR-OPENDIR-PARAMETER-ERROR", "only direct directory names without path (i.e. without '%c' characters) are allowed", QORE_DIR_SEP);
      return 0;
   }

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : d->getEncoding();
   ReferenceHolder<Dir> dc(new Dir(xsink, qe, d->getPath(subdir->getBuffer()).c_str()), xsink);   
   // create the qoreObject and set the Dir object as private data of the class tagged with the CID_DIR class ID
   return new QoreObject(QC_DIR, getProgram(), dc.release());
}

// File Dir::openFile(string filename, int flags = O_RDONLY, int mode = 0666, *string encoding){}
static QoreObject* Dir_openFile_VsViViNs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* filename = HARD_QORE_STRING(args, 0);
   int64 flags = HARD_QORE_INT(args, 1);
   int64 mode = HARD_QORE_INT(args, 2);
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 3));
# 569 "QC_Dir.qpp"
if (strchr(filename->getBuffer(), QORE_DIR_SEP)) {
      xsink->raiseException("DIR-OPENFILE-PARAMETER-ERROR", "only filenames without path (i.e. without '%c' characters) are allowed", QORE_DIR_SEP);
      return 0;
   }
  
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

   // open the file with exception
   ReferenceHolder<File> f(new File(qe), xsink);
   std::string path = d->getPath(filename->getBuffer());

   if (f->open2(xsink, path.c_str(), flags, mode, qe)) {
      assert(*xsink);
      return 0;
   }

   // create the QoreObject and set the File object as private data of the class tagged with the CID_FILE class ID
   return new QoreObject(QC_FILE, getProgram(), f.release());
}

// *string Dir::path(){}
static AbstractQoreNode* Dir_path(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
# 104 "QC_Dir.qpp"
return d->dirname();
}

// bool Dir::removeFile(string file){}
static bool Dir_removeFile_Vs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* file = HARD_QORE_STRING(args, 0);
# 639 "QC_Dir.qpp"
// check if there is a path delimiter in
   if (strchr(file->getBuffer(), QORE_DIR_SEP)) {
      xsink->raiseException("DIR-REMOVEFILE-PARAMETER-ERROR", "only filenames without path (i.e. without '%c' characters) are allowed", QORE_DIR_SEP);
      return 0;
   }

   std::string path = d->getPath(file->getBuffer());
   errno = 0; // clear errno flag
   if (unlink(path.c_str()) && errno != ENOENT) {
      xsink->raiseErrnoException("DIR-REMOVEFILE-FAILURE", errno, "error removing file '%s'", file->getBuffer());
      return 0;
   }

   // if an errno was set it must be ENOENT at this point.
   // so we return that no file is removed
   return errno ? false : true;
}

// nothing Dir::rmdir(string subdir){}
static AbstractQoreNode* Dir_rmdir_Vs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* subdir = HARD_QORE_STRING(args, 0);
# 358 "QC_Dir.qpp"
// check if there is a path delimiter
   if (strchr(subdir->getBuffer(), QORE_DIR_SEP))
      xsink->raiseException("DIR-RMDIR-PARAMETER-ERROR", "only direct subdirectories are allowed");
   else
      d->rmdir(subdir->getBuffer(), xsink);
   return 0;
}

// list Dir::stat(){}
static AbstractQoreNode* Dir_stat(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
# 679 "QC_Dir.qpp"
return d->stat(xsink);
}

// hash Dir::statvfs(){}
static AbstractQoreNode* Dir_statvfs(QoreObject* self, Dir* d, const QoreListNode* args, ExceptionSink* xsink) {
# 730 "QC_Dir.qpp"
#ifdef HAVE_SYS_STATVFS_H
   return d->statvfs(xsink);
#else
   return missing_method_error("Dir::statvfs", "STATVFS", xsink);
#endif
   return 0;
}

DLLLOCAL QoreClass* initDirClass(QoreNamespace& ns) {
   QC_DIR = new QoreClass("Dir", QDOM_FILESYSTEM);
   CID_DIR = QC_DIR->getID();

   // bool Dir::chdir(string path){}
   QC_DIR->addMethodExtended3("chdir", (q_method_bool_t)Dir_chdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "path");

   // nothing Dir::chgrp(int gid){}
   QC_DIR->addMethodExtended3("chgrp", (q_method_t)Dir_chgrp_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "gid");

   // nothing Dir::chgrp(string groupname){}
   QC_DIR->addMethodExtended3("chgrp", (q_method_t)Dir_chgrp_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "groupname");

   // nothing Dir::chmod(softint mode){}
   QC_DIR->addMethodExtended3("chmod", (q_method_t)Dir_chmod_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "mode");

   // nothing Dir::chown(int uid){}
   QC_DIR->addMethodExtended3("chown", (q_method_t)Dir_chown_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "uid");

   // nothing Dir::chown(string username){}
   QC_DIR->addMethodExtended3("chown", (q_method_t)Dir_chown_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "username");

   // Dir::constructor(*string encoding) {}
   QC_DIR->setConstructorExtended3(Dir_constructor_Ns, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, stringOrNothingTypeInfo, NULL, "encoding");

   // Dir::copy() {}
   QC_DIR->setCopy((q_copy_t)Dir_copy);

   // int Dir::create(softint mode = 0777){}
   QC_DIR->addMethodExtended3("create", (q_method_int64_t)Dir_create_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(0777), "mode");

   // bool Dir::exists(){}
   QC_DIR->addMethodExtended3("exists", (q_method_bool_t)Dir_exists, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // hash Dir::hstat(){}
   QC_DIR->addMethodExtended3("hstat", (q_method_t)Dir_hstat, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   // list Dir::list(bool full = False){}
   QC_DIR->addMethodExtended3("list", (q_method_t)Dir_list_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, boolTypeInfo, &False, "full");

   // list Dir::list(string regex, softint regex_options = 0, softbool full = False){}
   QC_DIR->addMethodExtended3("list", (q_method_t)Dir_list_Vsvivb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "regex", softBigIntTypeInfo, zero(), "regex_options", softBoolTypeInfo, &False, "full");

   // list Dir::listDirs(bool full = False){}
   QC_DIR->addMethodExtended3("listDirs", (q_method_t)Dir_listDirs_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, boolTypeInfo, &False, "full");

   // list Dir::listDirs(string regex, softint regex_options = 0, softbool full = False){}
   QC_DIR->addMethodExtended3("listDirs", (q_method_t)Dir_listDirs_Vsvivb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "regex", softBigIntTypeInfo, zero(), "regex_options", softBoolTypeInfo, &False, "full");

   // list Dir::listFiles(bool full = False){}
   QC_DIR->addMethodExtended3("listFiles", (q_method_t)Dir_listFiles_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, boolTypeInfo, &False, "full");

   // list Dir::listFiles(string regex, softint regex_options = 0, softbool full = False){}
   QC_DIR->addMethodExtended3("listFiles", (q_method_t)Dir_listFiles_Vsvivb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "regex", softBigIntTypeInfo, zero(), "regex_options", softBoolTypeInfo, &False, "full");

   // nothing Dir::mkdir(string subdir, softint mode = 0777){}
   QC_DIR->addMethodExtended3("mkdir", (q_method_t)Dir_mkdir_Vsvi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "subdir", softBigIntTypeInfo, new QoreBigIntNode(0777), "mode");

   // Dir Dir::openDir(string subdir, *string encoding){}
   QC_DIR->addMethodExtended3("openDir", (q_method_t)Dir_openDir_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_DIR->getTypeInfo(), 2, stringTypeInfo, NULL, "subdir", stringOrNothingTypeInfo, NULL, "encoding");

   // File Dir::openFile(string filename, int flags = O_RDONLY, int mode = 0666, *string encoding){}
   QC_DIR->addMethodExtended3("openFile", (q_method_t)Dir_openFile_VsViViNs, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_FILE->getTypeInfo(), 4, stringTypeInfo, NULL, "filename", bigIntTypeInfo, new QoreBigIntNode(O_RDONLY), "flags", bigIntTypeInfo, new QoreBigIntNode(0666), "mode", stringOrNothingTypeInfo, NULL, "encoding");

   // *string Dir::path(){}
   QC_DIR->addMethodExtended3("path", (q_method_t)Dir_path, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // bool Dir::removeFile(string file){}
   QC_DIR->addMethodExtended3("removeFile", (q_method_bool_t)Dir_removeFile_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "file");

   // nothing Dir::rmdir(string subdir){}
   QC_DIR->addMethodExtended3("rmdir", (q_method_t)Dir_rmdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "subdir");

   // list Dir::stat(){}
   QC_DIR->addMethodExtended3("stat", (q_method_t)Dir_stat, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);

   // hash Dir::statvfs(){}
   QC_DIR->addMethodExtended3("statvfs", (q_method_t)Dir_statvfs, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   return QC_DIR;
}
