/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_FtpClient.qpp

  Qore Programming Language

  Copyright 2003 - 2014 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/ssl_constants.h>

#include <qore/intern/QC_FtpClient.h>
#include <qore/intern/QC_Queue.h>
/* Qore class Qore::FtpClient */

qore_classid_t CID_FTPCLIENT;
QoreClass* QC_FTPCLIENT;

// nothing FtpClient::clearStats(){}
static AbstractQoreNode* FtpClient_clearStats(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 998 "QC_FtpClient.qpp"
f->clearStats();
   return 0;
}

// nothing FtpClient::clearWarningQueue(){}
static AbstractQoreNode* FtpClient_clearWarningQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 920 "QC_FtpClient.qpp"
f->clearWarningQueue(xsink);
   return 0;
}

// nothing FtpClient::connect(){}
static AbstractQoreNode* FtpClient_connect(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 146 "QC_FtpClient.qpp"
f->connect(xsink);
   return 0;
}

// FtpClient::constructor() {}
static void FtpClient_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 79 "QC_FtpClient.qpp"
self->setPrivate(CID_FTPCLIENT, new QoreFtpClientClass);
}

// FtpClient::constructor(string url) {}
static void FtpClient_constructor_Vs(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* url = HARD_QORE_STRING(args, 0);
# 96 "QC_FtpClient.qpp"
ReferenceHolder<QoreFtpClientClass> f(new QoreFtpClientClass(url, xsink), xsink);
   if (*xsink)
      return;

   self->setPrivate(CID_FTPCLIENT, f.release());
}

// FtpClient::copy() {}
static void FtpClient_copy(QoreObject* self, QoreObject* old, QoreFtpClientClass* f, ExceptionSink* xsink) {
# 108 "QC_FtpClient.qpp"
xsink->raiseException("FTPCLIENT-COPY-ERROR", "FtpClient objects cannot be copied");
}

// nothing FtpClient::cwd(string path){}
static AbstractQoreNode* FtpClient_cwd_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
# 290 "QC_FtpClient.qpp"
f->cwd(path->getBuffer(), xsink);
   return 0;
}

// nothing FtpClient::del(string remote_path){}
static AbstractQoreNode* FtpClient_del_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 0);
# 472 "QC_FtpClient.qpp"
f->del(remote_path->getBuffer(), xsink);
   return 0;
}

// FtpClient::destructor() {}
static void FtpClient_destructor(QoreObject* self, QoreFtpClientClass* f, ExceptionSink* xsink) {
# 122 "QC_FtpClient.qpp"
// have to clear callbacks before destroying
   f->cleanup(xsink);
   f->deref(xsink);
}

// nothing FtpClient::disconnect(){}
static AbstractQoreNode* FtpClient_disconnect(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 159 "QC_FtpClient.qpp"
f->disconnect();
   return 0;
}

// nothing FtpClient::get(string remote_path, *string local_path){}
static AbstractQoreNode* FtpClient_get_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 0);
   const QoreStringNode* local_path = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 360 "QC_FtpClient.qpp"
f->get(remote_path->getBuffer(), local_path ? local_path->getBuffer() : 0, xsink);
   return 0;
}

// binary FtpClient::getAsBinary(string remote_path){}
static AbstractQoreNode* FtpClient_getAsBinary_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 0);
# 452 "QC_FtpClient.qpp"
return f->getAsBinary(remote_path->getBuffer(), xsink);
}

// string FtpClient::getAsString(string remote_path){}
static AbstractQoreNode* FtpClient_getAsString_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 0);
# 427 "QC_FtpClient.qpp"
return f->getAsString(remote_path->getBuffer(), xsink);
}

// *string FtpClient::getHostName(){}
static AbstractQoreNode* FtpClient_getHostName(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 624 "QC_FtpClient.qpp"
const char *h = f->getHostName();
   return h ? new QoreStringNode(h) : 0;
}

// *string FtpClient::getPassword(){}
static AbstractQoreNode* FtpClient_getPassword(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 610 "QC_FtpClient.qpp"
const char *p = f->getPassword();
   return p ? new QoreStringNode(p) : 0;
}

// int FtpClient::getPort(){}
static int64 FtpClient_getPort(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 638 "QC_FtpClient.qpp"
return f->getPort();
}

// *string FtpClient::getSSLCipherName(){}
static AbstractQoreNode* FtpClient_getSSLCipherName(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 734 "QC_FtpClient.qpp"
const char *str = f->getSSLCipherName();
   return str ? new QoreStringNode(str) : 0;
}

// *string FtpClient::getSSLCipherVersion(){}
static AbstractQoreNode* FtpClient_getSSLCipherVersion(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 748 "QC_FtpClient.qpp"
const char *str = f->getSSLCipherVersion();
   return str ? new QoreStringNode(str) : 0;
}

// string FtpClient::getURL(){}
static AbstractQoreNode* FtpClient_getURL(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 651 "QC_FtpClient.qpp"
return f->getURL();
}

// hash FtpClient::getUsageInfo(){}
static AbstractQoreNode* FtpClient_getUsageInfo(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 984 "QC_FtpClient.qpp"
return f->getUsageInfo();
}

// *string FtpClient::getUserName(){}
static AbstractQoreNode* FtpClient_getUserName(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 596 "QC_FtpClient.qpp"
const char *u = f->getUserName();
   return u ? new QoreStringNode(u) : 0;
}

// bool FtpClient::isDataSecure(){}
static bool FtpClient_isDataSecure(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 721 "QC_FtpClient.qpp"
return f->isDataSecure();
}

// bool FtpClient::isSecure(){}
static bool FtpClient_isSecure(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 707 "QC_FtpClient.qpp"
return get_bool_node(f->isSecure());
}

// *string FtpClient::list(){}
static AbstractQoreNode* FtpClient_list(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 181 "QC_FtpClient.qpp"
return f->list(0, true, xsink);
}

// *string FtpClient::list(string path){}
static AbstractQoreNode* FtpClient_list_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
# 206 "QC_FtpClient.qpp"
return f->list(path->getBuffer(), true, xsink);
}

// nothing FtpClient::mkdir(string remote_path){}
static AbstractQoreNode* FtpClient_mkdir_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 0);
# 492 "QC_FtpClient.qpp"
f->mkdir(remote_path->getBuffer(), xsink);
   return 0;
}

// *string FtpClient::nlst(){}
static AbstractQoreNode* FtpClient_nlst(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 228 "QC_FtpClient.qpp"
return f->list(0, false, xsink);
}

// *string FtpClient::nlst(string path){}
static AbstractQoreNode* FtpClient_nlst_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
# 252 "QC_FtpClient.qpp"
return f->list(path->getBuffer(), false, xsink);
}

// nothing FtpClient::put(string local_path, *string remote_path){}
static AbstractQoreNode* FtpClient_put_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* local_path = HARD_QORE_STRING(args, 0);
   const QoreStringNode* remote_path = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 315 "QC_FtpClient.qpp"
f->put(local_path->getBuffer(), remote_path ? remote_path->getBuffer() : 0, xsink);
   return 0;
}

// nothing FtpClient::putData(string data, string remote_path){}
static AbstractQoreNode* FtpClient_putData_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* data = HARD_QORE_STRING(args, 0);
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 1);
# 381 "QC_FtpClient.qpp"
f->putData(data->getBuffer(), data->size(), remote_path->getBuffer(), xsink);
   return 0;
}

// nothing FtpClient::putData(binary data, string remote_path){}
static AbstractQoreNode* FtpClient_putData_VyVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const BinaryNode* data = HARD_QORE_BINARY(args, 0);
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 1);
# 402 "QC_FtpClient.qpp"
f->putData(data->getPtr(), data->size(), remote_path->getBuffer(), xsink);
   return 0;
}

// string FtpClient::pwd(){}
static AbstractQoreNode* FtpClient_pwd(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 272 "QC_FtpClient.qpp"
return f->pwd(xsink);
}

// nothing FtpClient::rename(string from, string to){}
static AbstractQoreNode* FtpClient_rename_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* from = HARD_QORE_STRING(args, 0);
   const QoreStringNode* to = HARD_QORE_STRING(args, 1);
# 336 "QC_FtpClient.qpp"
f->rename(from->getBuffer(), to->getBuffer(), xsink);
   return 0;
}

// nothing FtpClient::rmdir(string remote_path){}
static AbstractQoreNode* FtpClient_rmdir_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* remote_path = HARD_QORE_STRING(args, 0);
# 512 "QC_FtpClient.qpp"
f->rmdir(remote_path->getBuffer(), xsink);
   return 0;
}

// nothing FtpClient::setControlEventQueue(){}
static AbstractQoreNode* FtpClient_setControlEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 884 "QC_FtpClient.qpp"
f->setControlEventQueue(0, xsink);
   return 0;
}

// nothing FtpClient::setControlEventQueue(Qore::Thread::Queue queue){}
static AbstractQoreNode* FtpClient_setControlEventQueue_C5Queue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setControlEventQueue()", "Qore::Thread::Queue", xsink);
   if (*xsink)
      return 0;
# 900 "QC_FtpClient.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setControlEventQueue(q.release(), xsink);
   return 0;
}

// nothing FtpClient::setDataEventQueue(){}
static AbstractQoreNode* FtpClient_setDataEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 851 "QC_FtpClient.qpp"
f->setDataEventQueue(0, xsink);
   return 0;
}

// nothing FtpClient::setDataEventQueue(Qore::Thread::Queue queue){}
static AbstractQoreNode* FtpClient_setDataEventQueue_C5Queue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setDataEventQueue()", "Qore::Thread::Queue", xsink);
   if (*xsink)
      return 0;
# 867 "QC_FtpClient.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setDataEventQueue(q.release(), xsink);
   return 0;
}

// nothing FtpClient::setEventQueue(){}
static AbstractQoreNode* FtpClient_setEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 818 "QC_FtpClient.qpp"
f->setEventQueue(0, xsink);
   return 0;
}

// nothing FtpClient::setEventQueue(Qore::Thread::Queue queue){}
static AbstractQoreNode* FtpClient_setEventQueue_C5Queue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setEventQueue()", "Qore::Thread::Queue", xsink);
   if (*xsink)
      return 0;
# 834 "QC_FtpClient.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setEventQueue(q.release(), xsink);
   return 0;
}

// nothing FtpClient::setHostName(string host){}
static AbstractQoreNode* FtpClient_setHostName_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* host = HARD_QORE_STRING(args, 0);
# 549 "QC_FtpClient.qpp"
f->setHostName(host->getBuffer());
   return 0;
}

// nothing FtpClient::setInsecure(){}
static AbstractQoreNode* FtpClient_setInsecure(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 679 "QC_FtpClient.qpp"
if (f->setInsecure())
      xsink->raiseException("SET-INSECURE-ERROR", "this method cannot be called while the control connection is established");
   return 0;
}

// nothing FtpClient::setInsecureData(){}
static AbstractQoreNode* FtpClient_setInsecureData(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 694 "QC_FtpClient.qpp"
if (f->setInsecureData())
      xsink->raiseException("SET-INSECUREDATA-ERROR", "this method cannot be called while the control connection is established");
   return 0;
}

// nothing FtpClient::setModeAuto(){}
static AbstractQoreNode* FtpClient_setModeAuto(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 774 "QC_FtpClient.qpp"
f->setModeAuto();
   return 0;
}

// nothing FtpClient::setModeEPSV(){}
static AbstractQoreNode* FtpClient_setModeEPSV(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 785 "QC_FtpClient.qpp"
f->setModeEPSV();
   return 0;
}

// nothing FtpClient::setModePASV(){}
static AbstractQoreNode* FtpClient_setModePASV(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 796 "QC_FtpClient.qpp"
f->setModePASV();
   return 0;
}

// nothing FtpClient::setModePORT(){}
static AbstractQoreNode* FtpClient_setModePORT(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 807 "QC_FtpClient.qpp"
f->setModePORT();
   return 0;
}

// nothing FtpClient::setPassword(string pass){}
static AbstractQoreNode* FtpClient_setPassword_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* pass = HARD_QORE_STRING(args, 0);
# 536 "QC_FtpClient.qpp"
f->setPassword(pass->getBuffer());
   return 0;
}

// nothing FtpClient::setPort(int port){}
static AbstractQoreNode* FtpClient_setPort_Vi(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   int64 port = HARD_QORE_INT(args, 0);
# 563 "QC_FtpClient.qpp"
if (port <= 0)
      xsink->raiseException("FTPCLIENT-SETPORT-PARAMETER-ERROR", "expecting positive port number as first parameter of FtpClient::setPort(softint $port); got %d", port);
   else
      f->setPort(port);
   return 0;
}

// nothing FtpClient::setSecure(bool secure = True){}
static AbstractQoreNode* FtpClient_setSecure_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   bool secure = HARD_QORE_BOOL(args, 0);
# 665 "QC_FtpClient.qpp"
if (secure ? f->setSecure() : f->setInsecure())
      xsink->raiseException("SET-SECURE-ERROR", "this method cannot be called while the control connection is established");
   return 0;
}

// nothing FtpClient::setURL(string url){}
static AbstractQoreNode* FtpClient_setURL_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* url = HARD_QORE_STRING(args, 0);
# 583 "QC_FtpClient.qpp"
f->setURL(url, xsink);
   return 0;
}

// nothing FtpClient::setUserName(string user){}
static AbstractQoreNode* FtpClient_setUserName_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* user = HARD_QORE_STRING(args, 0);
# 524 "QC_FtpClient.qpp"
f->setUserName(user->getBuffer());
   return 0;
}

// nothing FtpClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, any arg, timeout min_ms = 1s){}
static AbstractQoreNode* FtpClient_setWarningQueue_ViViC5QueueVaVt(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
   int64 warning_ms = HARD_QORE_INT(args, 0);
   int64 warning_bs = HARD_QORE_INT(args, 1);
   HARD_QORE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "FtpClient::setWarningQueue()", "Queue", xsink);
   if (*xsink)
      return 0;
   const AbstractQoreNode* arg = get_param(args, 3);
   int64 min_ms = HARD_QORE_INT(args, 4);
# 955 "QC_FtpClient.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg ? arg->refSelf() : 0, min_ms);
   return 0;
}

// *string FtpClient::verifyPeerCertificate(){}
static AbstractQoreNode* FtpClient_verifyPeerCertificate(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, ExceptionSink* xsink) {
# 762 "QC_FtpClient.qpp"
const char *c = getSSLCVCode(f->verifyPeerCertificate());
   return c ? new QoreStringNode(c) : 0;
}

DLLLOCAL QoreClass* initFtpClientClass(QoreNamespace& ns) {
   QC_FTPCLIENT = new QoreClass("FtpClient", QDOM_NETWORK);
   CID_FTPCLIENT = QC_FTPCLIENT->getID();

   // nothing FtpClient::clearStats(){}
   QC_FTPCLIENT->addMethodExtended3("clearStats", (q_method_t)FtpClient_clearStats, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::clearWarningQueue(){}
   QC_FTPCLIENT->addMethodExtended3("clearWarningQueue", (q_method_t)FtpClient_clearWarningQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::connect(){}
   QC_FTPCLIENT->addMethodExtended3("connect", (q_method_t)FtpClient_connect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // FtpClient::constructor() {}
   QC_FTPCLIENT->setConstructorExtended3(FtpClient_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // FtpClient::constructor(string url) {}
   QC_FTPCLIENT->setConstructorExtended3(FtpClient_constructor_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, NULL, "url");

   // FtpClient::copy() {}
   QC_FTPCLIENT->setCopy((q_copy_t)FtpClient_copy);

   // nothing FtpClient::cwd(string path){}
   QC_FTPCLIENT->addMethodExtended3("cwd", (q_method_t)FtpClient_cwd_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "path");

   // nothing FtpClient::del(string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("del", (q_method_t)FtpClient_del_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

   // FtpClient::destructor() {}
   QC_FTPCLIENT->setDestructor((q_destructor_t)FtpClient_destructor);

   // nothing FtpClient::disconnect(){}
   QC_FTPCLIENT->addMethodExtended3("disconnect", (q_method_t)FtpClient_disconnect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::get(string remote_path, *string local_path){}
   QC_FTPCLIENT->addMethodExtended3("get", (q_method_t)FtpClient_get_VsNs, false, QC_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, NULL, "remote_path", stringOrNothingTypeInfo, NULL, "local_path");

   // binary FtpClient::getAsBinary(string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("getAsBinary", (q_method_t)FtpClient_getAsBinary_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

   // string FtpClient::getAsString(string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("getAsString", (q_method_t)FtpClient_getAsString_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

   // *string FtpClient::getHostName(){}
   QC_FTPCLIENT->addMethodExtended3("getHostName", (q_method_t)FtpClient_getHostName, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // *string FtpClient::getPassword(){}
   QC_FTPCLIENT->addMethodExtended3("getPassword", (q_method_t)FtpClient_getPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // int FtpClient::getPort(){}
   QC_FTPCLIENT->addMethodExtended3("getPort", (q_method_int64_t)FtpClient_getPort, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // *string FtpClient::getSSLCipherName(){}
   QC_FTPCLIENT->addMethodExtended3("getSSLCipherName", (q_method_t)FtpClient_getSSLCipherName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // *string FtpClient::getSSLCipherVersion(){}
   QC_FTPCLIENT->addMethodExtended3("getSSLCipherVersion", (q_method_t)FtpClient_getSSLCipherVersion, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // string FtpClient::getURL(){}
   QC_FTPCLIENT->addMethodExtended3("getURL", (q_method_t)FtpClient_getURL, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

   // hash FtpClient::getUsageInfo(){}
   QC_FTPCLIENT->addMethodExtended3("getUsageInfo", (q_method_t)FtpClient_getUsageInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

   // *string FtpClient::getUserName(){}
   QC_FTPCLIENT->addMethodExtended3("getUserName", (q_method_t)FtpClient_getUserName, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // bool FtpClient::isDataSecure(){}
   QC_FTPCLIENT->addMethodExtended3("isDataSecure", (q_method_bool_t)FtpClient_isDataSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool FtpClient::isSecure(){}
   QC_FTPCLIENT->addMethodExtended3("isSecure", (q_method_bool_t)FtpClient_isSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // *string FtpClient::list(){}
   QC_FTPCLIENT->addMethodExtended3("list", (q_method_t)FtpClient_list, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // *string FtpClient::list(string path){}
   QC_FTPCLIENT->addMethodExtended3("list", (q_method_t)FtpClient_list_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "path");

   // nothing FtpClient::mkdir(string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("mkdir", (q_method_t)FtpClient_mkdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

   // *string FtpClient::nlst(){}
   QC_FTPCLIENT->addMethodExtended3("nlst", (q_method_t)FtpClient_nlst, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // *string FtpClient::nlst(string path){}
   QC_FTPCLIENT->addMethodExtended3("nlst", (q_method_t)FtpClient_nlst_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "path");

   // nothing FtpClient::put(string local_path, *string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("put", (q_method_t)FtpClient_put_VsNs, false, QC_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, NULL, "local_path", stringOrNothingTypeInfo, NULL, "remote_path");

   // nothing FtpClient::putData(string data, string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("putData", (q_method_t)FtpClient_putData_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "data", stringTypeInfo, NULL, "remote_path");

   // nothing FtpClient::putData(binary data, string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("putData", (q_method_t)FtpClient_putData_VyVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, NULL, "data", stringTypeInfo, NULL, "remote_path");

   // string FtpClient::pwd(){}
   QC_FTPCLIENT->addMethodExtended3("pwd", (q_method_t)FtpClient_pwd, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

   // nothing FtpClient::rename(string from, string to){}
   QC_FTPCLIENT->addMethodExtended3("rename", (q_method_t)FtpClient_rename_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "from", stringTypeInfo, NULL, "to");

   // nothing FtpClient::rmdir(string remote_path){}
   QC_FTPCLIENT->addMethodExtended3("rmdir", (q_method_t)FtpClient_rmdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

   // nothing FtpClient::setControlEventQueue(){}
   QC_FTPCLIENT->addMethodExtended3("setControlEventQueue", (q_method_t)FtpClient_setControlEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setControlEventQueue(Qore::Thread::Queue queue){}
   QC_FTPCLIENT->addMethodExtended3("setControlEventQueue", (q_method_t)FtpClient_setControlEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

   // nothing FtpClient::setDataEventQueue(){}
   QC_FTPCLIENT->addMethodExtended3("setDataEventQueue", (q_method_t)FtpClient_setDataEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setDataEventQueue(Qore::Thread::Queue queue){}
   QC_FTPCLIENT->addMethodExtended3("setDataEventQueue", (q_method_t)FtpClient_setDataEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

   // nothing FtpClient::setEventQueue(){}
   QC_FTPCLIENT->addMethodExtended3("setEventQueue", (q_method_t)FtpClient_setEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setEventQueue(Qore::Thread::Queue queue){}
   QC_FTPCLIENT->addMethodExtended3("setEventQueue", (q_method_t)FtpClient_setEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

   // nothing FtpClient::setHostName(string host){}
   QC_FTPCLIENT->addMethodExtended3("setHostName", (q_method_t)FtpClient_setHostName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "host");

   // nothing FtpClient::setInsecure(){}
   QC_FTPCLIENT->addMethodExtended3("setInsecure", (q_method_t)FtpClient_setInsecure, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setInsecureData(){}
   QC_FTPCLIENT->addMethodExtended3("setInsecureData", (q_method_t)FtpClient_setInsecureData, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setModeAuto(){}
   QC_FTPCLIENT->addMethodExtended3("setModeAuto", (q_method_t)FtpClient_setModeAuto, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setModeEPSV(){}
   QC_FTPCLIENT->addMethodExtended3("setModeEPSV", (q_method_t)FtpClient_setModeEPSV, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setModePASV(){}
   QC_FTPCLIENT->addMethodExtended3("setModePASV", (q_method_t)FtpClient_setModePASV, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setModePORT(){}
   QC_FTPCLIENT->addMethodExtended3("setModePORT", (q_method_t)FtpClient_setModePORT, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing FtpClient::setPassword(string pass){}
   QC_FTPCLIENT->addMethodExtended3("setPassword", (q_method_t)FtpClient_setPassword_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "pass");

   // nothing FtpClient::setPort(int port){}
   QC_FTPCLIENT->addMethodExtended3("setPort", (q_method_t)FtpClient_setPort_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "port");

   // nothing FtpClient::setSecure(bool secure = True){}
   QC_FTPCLIENT->addMethodExtended3("setSecure", (q_method_t)FtpClient_setSecure_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, &True, "secure");

   // nothing FtpClient::setURL(string url){}
   QC_FTPCLIENT->addMethodExtended3("setURL", (q_method_t)FtpClient_setURL_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "url");

   // nothing FtpClient::setUserName(string user){}
   QC_FTPCLIENT->addMethodExtended3("setUserName", (q_method_t)FtpClient_setUserName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "user");

   // nothing FtpClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, any arg, timeout min_ms = 1s){}
   QC_FTPCLIENT->addMethodExtended3("setWarningQueue", (q_method_t)FtpClient_setWarningQueue_ViViC5QueueVaVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, NULL, "warning_ms", bigIntTypeInfo, NULL, "warning_bs", QC_QUEUE->getTypeInfo(), NULL, "queue", anyTypeInfo, NULL, "arg", timeoutTypeInfo, DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0), "min_ms");

   // *string FtpClient::verifyPeerCertificate(){}
   QC_FTPCLIENT->addMethodExtended3("verifyPeerCertificate", (q_method_t)FtpClient_verifyPeerCertificate, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   return QC_FTPCLIENT;
}
