/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Gate.qpp

  Qore Programming Language
  
  Copyright 2003 - 2013 David Nichols
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_Gate.h>
/* Qore class Qore::Thread::Gate */

qore_classid_t CID_GATE;
QoreClass* QC_GATE;

// Gate::constructor() {}
static void Gate_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 55 "QC_Gate.qpp"
self->setPrivate(CID_GATE, new QoreGate);
}

// Gate::copy() {}
static void Gate_copy(QoreObject* self, QoreObject* old, QoreGate* g, ExceptionSink* xsink) {
# 81 "QC_Gate.qpp"
self->setPrivate(CID_GATE, new QoreGate());
}

// Gate::destructor() {}
static void Gate_destructor(QoreObject* self, QoreGate* g, ExceptionSink* xsink) {
# 69 "QC_Gate.qpp"
g->destructor(xsink);
   g->deref(xsink);
}

// int Gate::enter(timeout timeout_ms){}
static int64 Gate_enter_Vt(QoreObject* self, QoreGate* g, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 100 "QC_Gate.qpp"
return g->grab(xsink, timeout_ms);
}

// nothing Gate::enter(){}
static AbstractQoreNode* Gate_enter(QoreObject* self, QoreGate* g, const QoreListNode* args, ExceptionSink* xsink) {
# 114 "QC_Gate.qpp"
g->grab(xsink);
   return 0;
}

// int Gate::exit(){}
static int64 Gate_exit(QoreObject* self, QoreGate* g, const QoreListNode* args, ExceptionSink* xsink) {
# 129 "QC_Gate.qpp"
return g->release(xsink);
}

// int Gate::numInside(){}
static int64 Gate_numInside(QoreObject* self, QoreGate* g, const QoreListNode* args, ExceptionSink* xsink) {
# 159 "QC_Gate.qpp"
return g->get_count();
}

// int Gate::numWaiting(){}
static int64 Gate_numWaiting(QoreObject* self, QoreGate* g, const QoreListNode* args, ExceptionSink* xsink) {
# 170 "QC_Gate.qpp"
return g->get_waiting();
}

// int Gate::tryEnter(){}
static int64 Gate_tryEnter(QoreObject* self, QoreGate* g, const QoreListNode* args, ExceptionSink* xsink) {
# 147 "QC_Gate.qpp"
return g->tryGrab();
}

DLLLOCAL QoreClass* initGateClass(QoreNamespace& ns) {
   QC_GATE = new QoreClass("Gate", QDOM_THREAD_CLASS);
   CID_GATE = QC_GATE->getID();

   // Gate::constructor() {}
   QC_GATE->setConstructorExtended3(Gate_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // Gate::copy() {}
   QC_GATE->setCopy((q_copy_t)Gate_copy);

   // Gate::destructor() {}
   QC_GATE->setDestructor((q_destructor_t)Gate_destructor);

   // int Gate::enter(timeout timeout_ms){}
   QC_GATE->addMethodExtended3("enter", (q_method_int64_t)Gate_enter_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

   // nothing Gate::enter(){}
   QC_GATE->addMethodExtended3("enter", (q_method_t)Gate_enter, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int Gate::exit(){}
   QC_GATE->addMethodExtended3("exit", (q_method_int64_t)Gate_exit, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // int Gate::numInside(){}
   QC_GATE->addMethodExtended3("numInside", (q_method_int64_t)Gate_numInside, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int Gate::numWaiting(){}
   QC_GATE->addMethodExtended3("numWaiting", (q_method_int64_t)Gate_numWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int Gate::tryEnter(){}
   QC_GATE->addMethodExtended3("tryEnter", (q_method_int64_t)Gate_tryEnter, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   return QC_GATE;
}
