/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_HTTPClient.qpp

  Qore Programming Language

  Copyright (C) 2006 - 2014 Qore Technologies

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/QoreHttpClientObject.h>
#include <qore/intern/QC_HTTPClient.h>
#include <qore/intern/QC_Queue.h>
#include <qore/intern/ssl_constants.h>
#include <qore/minitest.hpp>

#ifdef DEBUG_TESTS
#  include "tests/QC_HTTPClient_tests.cpp"
#endif
/* Qore class Qore::HTTPClient */

qore_classid_t CID_HTTPCLIENT;
QoreClass* QC_HTTPCLIENT;

// nothing HTTPClient::clearProxyURL(){}
static AbstractQoreNode* HTTPClient_clearProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 786 "QC_HTTPClient.qpp"
client->clearProxyURL();
   return 0;
}

// nothing HTTPClient::clearProxyUserPassword(){}
static AbstractQoreNode* HTTPClient_clearProxyUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1027 "QC_HTTPClient.qpp"
client->clearProxyUserPassword();
   return 0;
}

// nothing HTTPClient::clearStats(){}
static AbstractQoreNode* HTTPClient_clearStats(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1157 "QC_HTTPClient.qpp"
client->clearStats();
   return 0;
}

// nothing HTTPClient::clearUserPassword(){}
static AbstractQoreNode* HTTPClient_clearUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 984 "QC_HTTPClient.qpp"
client->clearUserPassword();
   return 0;
}

// nothing HTTPClient::clearWarningQueue(){}
static AbstractQoreNode* HTTPClient_clearWarningQueue(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1079 "QC_HTTPClient.qpp"
client->clearWarningQueue(xsink);
   return 0;
}

// nothing HTTPClient::connect(){}
static AbstractQoreNode* HTTPClient_connect(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 231 "QC_HTTPClient.qpp"
client->connect(xsink);
   return 0;
}

// HTTPClient::constructor(hash opts) {}
static void HTTPClient_constructor_Vh(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* opts = HARD_QORE_HASH(args, 0);
# 121 "QC_HTTPClient.qpp"
ReferenceHolder<QoreHttpClientObject> client(new QoreHttpClientObject, xsink);
   if (client->setOptions(opts, xsink))
      return;

   self->setPrivate(CID_HTTPCLIENT, client.release());
}

// HTTPClient::constructor() {}
static void HTTPClient_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 136 "QC_HTTPClient.qpp"
self->setPrivate(CID_HTTPCLIENT, new QoreHttpClientObject);
}

// HTTPClient::copy() {}
static void HTTPClient_copy(QoreObject* self, QoreObject* old, QoreHttpClientObject* client, ExceptionSink* xsink) {
# 144 "QC_HTTPClient.qpp"
xsink->raiseException("HTTPCLIENT-COPY-ERROR", "copying HTTPClient objects is not yet supported");
}

// HTTPClient::destructor() {}
static void HTTPClient_destructor(QoreObject* self, QoreHttpClientObject* client, ExceptionSink* xsink) {
# 155 "QC_HTTPClient.qpp"
// have to clear callbacks before destroying
   client->cleanup(xsink);
   client->deref(xsink);
}

// nothing HTTPClient::disconnect(){}
static AbstractQoreNode* HTTPClient_disconnect(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 242 "QC_HTTPClient.qpp"
client->disconnect();
   return 0;
}

// *string HTTPClient::get(string path, *hash headers, *reference info){}
static AbstractQoreNode* HTTPClient_get_VsNhNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 1));
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 2));
# 533 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<AbstractQoreNode> rv(client->get(path->getBuffer(), headers, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// int HTTPClient::getConnectTimeout(){}
static int64 HTTPClient_getConnectTimeout(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 894 "QC_HTTPClient.qpp"
return client->getConnectTimeout();
}

// *string HTTPClient::getConnectionPath(){}
static AbstractQoreNode* HTTPClient_getConnectionPath(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1064 "QC_HTTPClient.qpp"
const char* p = client->getConnectionPath();
   return p ? new QoreStringNode(p) : 0;
}

// string HTTPClient::getDefaultPath(){}
static AbstractQoreNode* HTTPClient_getDefaultPath(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1051 "QC_HTTPClient.qpp"
const char* dp = client->getDefaultPath();
   return dp ? new QoreStringNode(dp) : 0;
}

// string HTTPClient::getEncoding(){}
static AbstractQoreNode* HTTPClient_getEncoding(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 701 "QC_HTTPClient.qpp"
return new QoreStringNode(client->getEncoding()->getCode());
}

// string HTTPClient::getHTTPVersion(){}
static AbstractQoreNode* HTTPClient_getHTTPVersion(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 183 "QC_HTTPClient.qpp"
return new QoreStringNode(client->getHTTPVersion());
}

// int HTTPClient::getMaxRedirects(){}
static int64 HTTPClient_getMaxRedirects(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 837 "QC_HTTPClient.qpp"
return client->getMaxRedirects();
}

// bool HTTPClient::getNoDelay(){}
static bool HTTPClient_getNoDelay(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 929 "QC_HTTPClient.qpp"
return client->getNoDelay();
}

// *string HTTPClient::getProxyURL(){}
static AbstractQoreNode* HTTPClient_getProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 775 "QC_HTTPClient.qpp"
return client->getProxyURL();
}

// int HTTPClient::getTimeout(){}
static int64 HTTPClient_getTimeout(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 677 "QC_HTTPClient.qpp"
return client->getTimeout();
}

// *string HTTPClient::getURL(){}
static AbstractQoreNode* HTTPClient_getURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 734 "QC_HTTPClient.qpp"
return client->getURL();
}

// hash HTTPClient::getUsageInfo(){}
static AbstractQoreNode* HTTPClient_getUsageInfo(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1143 "QC_HTTPClient.qpp"
return client->getUsageInfo();
}

// hash HTTPClient::head(string path, *hash headers, *reference info){}
static AbstractQoreNode* HTTPClient_head_VsNhNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 1));
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 2));
# 568 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->head(path->getBuffer(), headers, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// bool HTTPClient::isConnected(){}
static bool HTTPClient_isConnected(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 941 "QC_HTTPClient.qpp"
return client->isConnected();
}

// bool HTTPClient::isProxySecure(){}
static bool HTTPClient_isProxySecure(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 811 "QC_HTTPClient.qpp"
return client->isProxySecure();
}

// bool HTTPClient::isSecure(){}
static bool HTTPClient_isSecure(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 212 "QC_HTTPClient.qpp"
return client->isSecure();
}

// *string HTTPClient::post(string path, string body, *hash headers, *reference info){}
static AbstractQoreNode* HTTPClient_post_VsVsNhNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const QoreStringNode* body = HARD_QORE_STRING(args, 1);
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 2));
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 3));
# 607 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<AbstractQoreNode> rv(client->post(path->getBuffer(), headers, body->getBuffer(), body->size(), *ohrh, xsink), xsink);
#ifdef DEBUG
   assert(!rv || rv->getType() == NT_STRING);
#endif
   return *xsink ? 0 : rv.release();
}

// *string HTTPClient::post(string path, *binary body, *hash headers, *reference info){}
static AbstractQoreNode* HTTPClient_post_VsNyNhNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const BinaryNode* body = reinterpret_cast<const BinaryNode*>(get_param(args, 1));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 2));
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 3));
# 648 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<AbstractQoreNode> rv(client->post(path->getBuffer(), headers, body ? body->getPtr() : 0, body ? body->size() : 0, *ohrh, xsink), xsink);
#ifdef DEBUG
   assert(!rv || rv->getType() == NT_STRING);
#endif
   return *xsink ? 0 : rv.release();
}

// hash HTTPClient::send(string body, string method, *string path, *hash headers, softbool getbody = False, *reference info){}
static AbstractQoreNode* HTTPClient_send_VsVsNsNhvbNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* body = HARD_QORE_STRING(args, 0);
   const QoreStringNode* method = HARD_QORE_STRING(args, 1);
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 2));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 3));
   bool getbody = HARD_QORE_BOOL(args, 4);
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 5));
# 277 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->send(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, body->getBuffer(), body->strlen(), getbody, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// hash HTTPClient::send(*binary body, string method, *string path, *hash headers, softbool getbody = False, *reference info){}
static AbstractQoreNode* HTTPClient_send_NyVsNsNhvbNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const BinaryNode* body = reinterpret_cast<const BinaryNode*>(get_param(args, 0));
   const QoreStringNode* method = HARD_QORE_STRING(args, 1);
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 2));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 3));
   bool getbody = HARD_QORE_BOOL(args, 4);
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 5));
# 313 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->send(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, body ? body->getPtr() : 0, body ? body->size() : 0, getbody, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// nothing HTTPClient::sendWithCallbacks(code scb, code rcb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
static AbstractQoreNode* HTTPClient_sendWithCallbacks_VcVcVsNsNhVtvbNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const ResolvedCallReferenceNode* scb = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 0));
   const ResolvedCallReferenceNode* rcb = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
   const QoreStringNode* method = HARD_QORE_STRING(args, 2);
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 3));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 4));
   int64 timeout_ms = HARD_QORE_INT(args, 5);
   bool getbody = HARD_QORE_BOOL(args, 6);
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 7));
# 495 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   client->sendWithCallbacks(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, scb, getbody, *ohrh, timeout_ms, rcb, self, xsink);
   return 0;
}

// nothing HTTPClient::sendWithRecvCallback(code rcb, string body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
static AbstractQoreNode* HTTPClient_sendWithRecvCallback_VcVsVsNsNhVtvbNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const ResolvedCallReferenceNode* rcb = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 0));
   const QoreStringNode* body = HARD_QORE_STRING(args, 1);
   const QoreStringNode* method = HARD_QORE_STRING(args, 2);
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 3));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 4));
   int64 timeout_ms = HARD_QORE_INT(args, 5);
   bool getbody = HARD_QORE_BOOL(args, 6);
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 7));
# 401 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   client->sendWithRecvCallback(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, body->getBuffer(), body->size(), getbody, *ohrh, timeout_ms, rcb, self, xsink);
   return 0;
}

// nothing HTTPClient::sendWithRecvCallback(code rcb, *binary body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
static AbstractQoreNode* HTTPClient_sendWithRecvCallback_VcNyVsNsNhVtvbNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const ResolvedCallReferenceNode* rcb = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 0));
   const BinaryNode* body = reinterpret_cast<const BinaryNode*>(get_param(args, 1));
   const QoreStringNode* method = HARD_QORE_STRING(args, 2);
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 3));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 4));
   int64 timeout_ms = HARD_QORE_INT(args, 5);
   bool getbody = HARD_QORE_BOOL(args, 6);
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 7));
# 446 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   client->sendWithRecvCallback(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, body ? body->getPtr() : 0, body ? body->size() : 0, getbody, *ohrh, timeout_ms, rcb, self, xsink);
   return 0;
}

// hash HTTPClient::sendWithSendCallback(code scb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
static AbstractQoreNode* HTTPClient_sendWithSendCallback_VcVsNsNhVtvbNr(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const ResolvedCallReferenceNode* scb = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 0));
   const QoreStringNode* method = HARD_QORE_STRING(args, 1);
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 2));
   const QoreHashNode* headers = reinterpret_cast<const QoreHashNode*>(get_param(args, 3));
   int64 timeout_ms = HARD_QORE_INT(args, 4);
   bool getbody = HARD_QORE_BOOL(args, 5);
   const ReferenceNode* info = reinterpret_cast<const ReferenceNode*>(get_param(args, 6));
# 355 "QC_HTTPClient.qpp"
OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->sendWithSendCallback(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, scb, getbody, *ohrh, timeout_ms, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// nothing HTTPClient::setConnectTimeout(timeout timeout_ms = -1){}
static AbstractQoreNode* HTTPClient_setConnectTimeout_Vt(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 882 "QC_HTTPClient.qpp"
client->setConnectTimeout((int)timeout_ms);
   return 0;
}

// nothing HTTPClient::setDefaultPath(*string path){}
static AbstractQoreNode* HTTPClient_setDefaultPath_Ns(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = reinterpret_cast<const QoreStringNode*>(get_param(args, 0));
# 1039 "QC_HTTPClient.qpp"
client->setDefaultPath(path ? path->getBuffer() : 0);
   return 0;
}

// nothing HTTPClient::setEncoding(string encoding){}
static AbstractQoreNode* HTTPClient_setEncoding_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* encoding = HARD_QORE_STRING(args, 0);
# 689 "QC_HTTPClient.qpp"
client->setEncoding(QEM.findCreate(encoding));
   return 0;
}

// nothing HTTPClient::setEventQueue(){}
static AbstractQoreNode* HTTPClient_setEventQueue(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 848 "QC_HTTPClient.qpp"
client->setEventQueue(0, xsink);
   return 0;
}

// nothing HTTPClient::setEventQueue(Qore::Thread::Queue queue){}
static AbstractQoreNode* HTTPClient_setEventQueue_C5Queue(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "HTTPClient::setEventQueue()", "Qore::Thread::Queue", xsink);
   if (*xsink)
      return 0;
# 864 "QC_HTTPClient.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the HttpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }
   // pass reference from QoreObject::getReferencedPrivateData() to function
   client->setEventQueue(q.release(), xsink);
   return 0;
}

// nothing HTTPClient::setHTTPVersion(string ver){}
static AbstractQoreNode* HTTPClient_setHTTPVersion_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* ver = HARD_QORE_STRING(args, 0);
# 171 "QC_HTTPClient.qpp"
client->setHTTPVersion(ver->getBuffer(), xsink);
   return 0;
}

// nothing HTTPClient::setMaxRedirects(softint mr = 0){}
static AbstractQoreNode* HTTPClient_setMaxRedirects_vi(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   int64 mr = HARD_QORE_INT(args, 0);
# 825 "QC_HTTPClient.qpp"
client->setMaxRedirects((int)mr);
   return 0;
}

// int HTTPClient::setNoDelay(softbool b = True){}
static int64 HTTPClient_setNoDelay_vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   bool b = HARD_QORE_BOOL(args, 0);
# 916 "QC_HTTPClient.qpp"
return client->setNoDelay(b);
}

// nothing HTTPClient::setPersistent(){}
static AbstractQoreNode* HTTPClient_setPersistent(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1174 "QC_HTTPClient.qpp"
client->setPersistent(xsink);
   return 0;
}

// nothing HTTPClient::setProxySecure(softbool b = True){}
static AbstractQoreNode* HTTPClient_setProxySecure_vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   bool b = HARD_QORE_BOOL(args, 0);
# 799 "QC_HTTPClient.qpp"
client->setProxySecure(b);
   return 0;
}

// nothing HTTPClient::setProxyURL(){}
static AbstractQoreNode* HTTPClient_setProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 746 "QC_HTTPClient.qpp"
client->clearProxyURL();
   return 0;
}

// nothing HTTPClient::setProxyURL(string url){}
static AbstractQoreNode* HTTPClient_setProxyURL_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* url = HARD_QORE_STRING(args, 0);
# 763 "QC_HTTPClient.qpp"
client->setProxyURL(url->getBuffer(), xsink);
   return 0;
}

// nothing HTTPClient::setProxyUserPassword(string user, string pass){}
static AbstractQoreNode* HTTPClient_setProxyUserPassword_VsVs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* user = HARD_QORE_STRING(args, 0);
   const QoreStringNode* pass = HARD_QORE_STRING(args, 1);
# 999 "QC_HTTPClient.qpp"
client->setProxyUserPassword(user->getBuffer(), pass->getBuffer());
   return 0;
}

// nothing HTTPClient::setProxyUserPassword(){}
static AbstractQoreNode* HTTPClient_setProxyUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 1015 "QC_HTTPClient.qpp"
client->clearProxyUserPassword();
   return 0;
}

// nothing HTTPClient::setSecure(softbool secure = True){}
static AbstractQoreNode* HTTPClient_setSecure_vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   bool secure = HARD_QORE_BOOL(args, 0);
# 199 "QC_HTTPClient.qpp"
client->setSecure(secure);
   return 0;
}

// nothing HTTPClient::setTimeout(timeout timeout_ms = 0){}
static AbstractQoreNode* HTTPClient_setTimeout_Vt(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 665 "QC_HTTPClient.qpp"
client->setTimeout((int)timeout_ms);
   return 0;
}

// nothing HTTPClient::setURL(string url){}
static AbstractQoreNode* HTTPClient_setURL_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* url = HARD_QORE_STRING(args, 0);
# 722 "QC_HTTPClient.qpp"
client->setURL(url->getBuffer(), xsink);
   return 0;
}

// nothing HTTPClient::setUserPassword(string user, string pass){}
static AbstractQoreNode* HTTPClient_setUserPassword_VsVs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* user = HARD_QORE_STRING(args, 0);
   const QoreStringNode* pass = HARD_QORE_STRING(args, 1);
# 956 "QC_HTTPClient.qpp"
client->setUserPassword(user->getBuffer(), pass->getBuffer());
   return 0;
}

// nothing HTTPClient::setUserPassword(){}
static AbstractQoreNode* HTTPClient_setUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
# 972 "QC_HTTPClient.qpp"
client->clearUserPassword();
   return 0;
}

// nothing HTTPClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, any arg, timeout min_ms = 1s){}
static AbstractQoreNode* HTTPClient_setWarningQueue_ViViC5QueueVaVt(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, ExceptionSink* xsink) {
   int64 warning_ms = HARD_QORE_INT(args, 0);
   int64 warning_bs = HARD_QORE_INT(args, 1);
   HARD_QORE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "HTTPClient::setWarningQueue()", "Queue", xsink);
   if (*xsink)
      return 0;
   const AbstractQoreNode* arg = get_param(args, 3);
   int64 min_ms = HARD_QORE_INT(args, 4);
# 1114 "QC_HTTPClient.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }
   // pass reference from QoreObject::getReferencedPrivateData() to function
   client->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg ? arg->refSelf() : 0, min_ms);
   return 0;
}

DLLLOCAL QoreClass* initHTTPClientClass(QoreNamespace& ns) {
   QC_HTTPCLIENT = new QoreClass("HTTPClient", QDOM_NETWORK);
   CID_HTTPCLIENT = QC_HTTPCLIENT->getID();

   // set parent class
   assert(QC_SOCKET);
   QC_HTTPCLIENT->addBuiltinVirtualBaseClass(QC_SOCKET);

   // nothing HTTPClient::clearProxyURL(){}
   QC_HTTPCLIENT->addMethodExtended3("clearProxyURL", (q_method_t)HTTPClient_clearProxyURL, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::clearProxyUserPassword(){}
   QC_HTTPCLIENT->addMethodExtended3("clearProxyUserPassword", (q_method_t)HTTPClient_clearProxyUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::clearStats(){}
   QC_HTTPCLIENT->addMethodExtended3("clearStats", (q_method_t)HTTPClient_clearStats, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::clearUserPassword(){}
   QC_HTTPCLIENT->addMethodExtended3("clearUserPassword", (q_method_t)HTTPClient_clearUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::clearWarningQueue(){}
   QC_HTTPCLIENT->addMethodExtended3("clearWarningQueue", (q_method_t)HTTPClient_clearWarningQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::connect(){}
   QC_HTTPCLIENT->addMethodExtended3("connect", (q_method_t)HTTPClient_connect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // HTTPClient::constructor(hash opts) {}
   QC_HTTPCLIENT->setConstructorExtended3(HTTPClient_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "opts");

   // HTTPClient::constructor() {}
   QC_HTTPCLIENT->setConstructorExtended3(HTTPClient_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // HTTPClient::copy() {}
   QC_HTTPCLIENT->setCopy((q_copy_t)HTTPClient_copy);

   // HTTPClient::destructor() {}
   QC_HTTPCLIENT->setDestructor((q_destructor_t)HTTPClient_destructor);

   // nothing HTTPClient::disconnect(){}
   QC_HTTPCLIENT->addMethodExtended3("disconnect", (q_method_t)HTTPClient_disconnect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // *string HTTPClient::get(string path, *hash headers, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("get", (q_method_t)HTTPClient_get_VsNhNr, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 3, stringTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", anyTypeInfo, NULL, "info");

   // int HTTPClient::getConnectTimeout(){}
   QC_HTTPCLIENT->addMethodExtended3("getConnectTimeout", (q_method_int64_t)HTTPClient_getConnectTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // *string HTTPClient::getConnectionPath(){}
   QC_HTTPCLIENT->addMethodExtended3("getConnectionPath", (q_method_t)HTTPClient_getConnectionPath, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // string HTTPClient::getDefaultPath(){}
   QC_HTTPCLIENT->addMethodExtended3("getDefaultPath", (q_method_t)HTTPClient_getDefaultPath, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // string HTTPClient::getEncoding(){}
   QC_HTTPCLIENT->addMethodExtended3("getEncoding", (q_method_t)HTTPClient_getEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // string HTTPClient::getHTTPVersion(){}
   QC_HTTPCLIENT->addMethodExtended3("getHTTPVersion", (q_method_t)HTTPClient_getHTTPVersion, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // int HTTPClient::getMaxRedirects(){}
   QC_HTTPCLIENT->addMethodExtended3("getMaxRedirects", (q_method_int64_t)HTTPClient_getMaxRedirects, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool HTTPClient::getNoDelay(){}
   QC_HTTPCLIENT->addMethodExtended3("getNoDelay", (q_method_bool_t)HTTPClient_getNoDelay, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // *string HTTPClient::getProxyURL(){}
   QC_HTTPCLIENT->addMethodExtended3("getProxyURL", (q_method_t)HTTPClient_getProxyURL, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // int HTTPClient::getTimeout(){}
   QC_HTTPCLIENT->addMethodExtended3("getTimeout", (q_method_int64_t)HTTPClient_getTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // *string HTTPClient::getURL(){}
   QC_HTTPCLIENT->addMethodExtended3("getURL", (q_method_t)HTTPClient_getURL, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // hash HTTPClient::getUsageInfo(){}
   QC_HTTPCLIENT->addMethodExtended3("getUsageInfo", (q_method_t)HTTPClient_getUsageInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

   // hash HTTPClient::head(string path, *hash headers, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("head", (q_method_t)HTTPClient_head_VsNhNr, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 3, stringTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", anyTypeInfo, NULL, "info");

   // bool HTTPClient::isConnected(){}
   QC_HTTPCLIENT->addMethodExtended3("isConnected", (q_method_bool_t)HTTPClient_isConnected, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HTTPClient::isProxySecure(){}
   QC_HTTPCLIENT->addMethodExtended3("isProxySecure", (q_method_bool_t)HTTPClient_isProxySecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HTTPClient::isSecure(){}
   QC_HTTPCLIENT->addMethodExtended3("isSecure", (q_method_bool_t)HTTPClient_isSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // *string HTTPClient::post(string path, string body, *hash headers, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("post", (q_method_t)HTTPClient_post_VsVsNhNr, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, stringTypeInfo, NULL, "path", stringTypeInfo, NULL, "body", hashOrNothingTypeInfo, NULL, "headers", anyTypeInfo, NULL, "info");

   // *string HTTPClient::post(string path, *binary body, *hash headers, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("post", (q_method_t)HTTPClient_post_VsNyNhNr, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, stringTypeInfo, NULL, "path", binaryOrNothingTypeInfo, NULL, "body", hashOrNothingTypeInfo, NULL, "headers", anyTypeInfo, NULL, "info");

   // hash HTTPClient::send(string body, string method, *string path, *hash headers, softbool getbody = False, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("send", (q_method_t)HTTPClient_send_VsVsNsNhvbNr, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 6, stringTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", softBoolTypeInfo, &False, "getbody", anyTypeInfo, NULL, "info");

   // hash HTTPClient::send(*binary body, string method, *string path, *hash headers, softbool getbody = False, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("send", (q_method_t)HTTPClient_send_NyVsNsNhvbNr, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 6, binaryOrNothingTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", softBoolTypeInfo, &False, "getbody", anyTypeInfo, NULL, "info");

   // nothing HTTPClient::sendWithCallbacks(code scb, code rcb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("sendWithCallbacks", (q_method_t)HTTPClient_sendWithCallbacks_VcVcVsNsNhVtvbNr, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, NULL, "scb", codeTypeInfo, NULL, "rcb", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", anyTypeInfo, NULL, "info");

   // nothing HTTPClient::sendWithRecvCallback(code rcb, string body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("sendWithRecvCallback", (q_method_t)HTTPClient_sendWithRecvCallback_VcVsVsNsNhVtvbNr, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, NULL, "rcb", stringTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", anyTypeInfo, NULL, "info");

   // nothing HTTPClient::sendWithRecvCallback(code rcb, *binary body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("sendWithRecvCallback", (q_method_t)HTTPClient_sendWithRecvCallback_VcNyVsNsNhVtvbNr, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, NULL, "rcb", binaryOrNothingTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", anyTypeInfo, NULL, "info");

   // hash HTTPClient::sendWithSendCallback(code scb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference info){}
   QC_HTTPCLIENT->addMethodExtended3("sendWithSendCallback", (q_method_t)HTTPClient_sendWithSendCallback_VcVsNsNhVtvbNr, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 7, codeTypeInfo, NULL, "scb", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", anyTypeInfo, NULL, "info");

   // nothing HTTPClient::setConnectTimeout(timeout timeout_ms = -1){}
   QC_HTTPCLIENT->addMethodExtended3("setConnectTimeout", (q_method_t)HTTPClient_setConnectTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

   // nothing HTTPClient::setDefaultPath(*string path){}
   QC_HTTPCLIENT->addMethodExtended3("setDefaultPath", (q_method_t)HTTPClient_setDefaultPath_Ns, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, NULL, "path");

   // nothing HTTPClient::setEncoding(string encoding){}
   QC_HTTPCLIENT->addMethodExtended3("setEncoding", (q_method_t)HTTPClient_setEncoding_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "encoding");

   // nothing HTTPClient::setEventQueue(){}
   QC_HTTPCLIENT->addMethodExtended3("setEventQueue", (q_method_t)HTTPClient_setEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::setEventQueue(Qore::Thread::Queue queue){}
   QC_HTTPCLIENT->addMethodExtended3("setEventQueue", (q_method_t)HTTPClient_setEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

   // nothing HTTPClient::setHTTPVersion(string ver){}
   QC_HTTPCLIENT->addMethodExtended3("setHTTPVersion", (q_method_t)HTTPClient_setHTTPVersion_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "ver");

   // nothing HTTPClient::setMaxRedirects(softint mr = 0){}
   QC_HTTPCLIENT->addMethodExtended3("setMaxRedirects", (q_method_t)HTTPClient_setMaxRedirects_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, zero(), "mr");

   // int HTTPClient::setNoDelay(softbool b = True){}
   QC_HTTPCLIENT->addMethodExtended3("setNoDelay", (q_method_int64_t)HTTPClient_setNoDelay_vb, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBoolTypeInfo, &True, "b");

   // nothing HTTPClient::setPersistent(){}
   QC_HTTPCLIENT->addMethodExtended3("setPersistent", (q_method_t)HTTPClient_setPersistent, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::setProxySecure(softbool b = True){}
   QC_HTTPCLIENT->addMethodExtended3("setProxySecure", (q_method_t)HTTPClient_setProxySecure_vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBoolTypeInfo, &True, "b");

   // nothing HTTPClient::setProxyURL(){}
   QC_HTTPCLIENT->addMethodExtended3("setProxyURL", (q_method_t)HTTPClient_setProxyURL, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::setProxyURL(string url){}
   QC_HTTPCLIENT->addMethodExtended3("setProxyURL", (q_method_t)HTTPClient_setProxyURL_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "url");

   // nothing HTTPClient::setProxyUserPassword(string user, string pass){}
   QC_HTTPCLIENT->addMethodExtended3("setProxyUserPassword", (q_method_t)HTTPClient_setProxyUserPassword_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "user", stringTypeInfo, NULL, "pass");

   // nothing HTTPClient::setProxyUserPassword(){}
   QC_HTTPCLIENT->addMethodExtended3("setProxyUserPassword", (q_method_t)HTTPClient_setProxyUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::setSecure(softbool secure = True){}
   QC_HTTPCLIENT->addMethodExtended3("setSecure", (q_method_t)HTTPClient_setSecure_vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBoolTypeInfo, &True, "secure");

   // nothing HTTPClient::setTimeout(timeout timeout_ms = 0){}
   QC_HTTPCLIENT->addMethodExtended3("setTimeout", (q_method_t)HTTPClient_setTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

   // nothing HTTPClient::setURL(string url){}
   QC_HTTPCLIENT->addMethodExtended3("setURL", (q_method_t)HTTPClient_setURL_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "url");

   // nothing HTTPClient::setUserPassword(string user, string pass){}
   QC_HTTPCLIENT->addMethodExtended3("setUserPassword", (q_method_t)HTTPClient_setUserPassword_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "user", stringTypeInfo, NULL, "pass");

   // nothing HTTPClient::setUserPassword(){}
   QC_HTTPCLIENT->addMethodExtended3("setUserPassword", (q_method_t)HTTPClient_setUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing HTTPClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, any arg, timeout min_ms = 1s){}
   QC_HTTPCLIENT->addMethodExtended3("setWarningQueue", (q_method_t)HTTPClient_setWarningQueue_ViViC5QueueVaVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, NULL, "warning_ms", bigIntTypeInfo, NULL, "warning_bs", QC_QUEUE->getTypeInfo(), NULL, "queue", anyTypeInfo, NULL, "arg", timeoutTypeInfo, DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0), "min_ms");

   return QC_HTTPCLIENT;
}
