/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashIterator.qpp HashIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreHashIterator.h>
/* Qore class Qore::HashIterator */

qore_classid_t CID_HASHITERATOR;
QoreClass* QC_HASHITERATOR;

// HashIterator::constructor(hash h) {}
static void HashIterator_constructor_Vh(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* h = HARD_QORE_HASH(args, 0);
# 62 "QC_HashIterator.qpp"
self->setPrivate(CID_HASHITERATOR, new QoreHashIterator(h));
}

// HashIterator::constructor() {}
static void HashIterator_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 73 "QC_HashIterator.qpp"
self->setPrivate(CID_HASHITERATOR, new QoreHashIterator);
}

// HashIterator::copy() {}
static void HashIterator_copy(QoreObject* self, QoreObject* old, QoreHashIterator* i, ExceptionSink* xsink) {
# 83 "QC_HashIterator.qpp"
self->setPrivate(CID_HASHITERATOR, new QoreHashIterator(*i));
}

// bool HashIterator::empty(){}
static bool HashIterator_empty(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 138 "QC_HashIterator.qpp"
return i->empty();
}

// bool HashIterator::first(){}
static bool HashIterator_first(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 153 "QC_HashIterator.qpp"
return i->first();
}

// string HashIterator::getKey(){}
static AbstractQoreNode* HashIterator_getKey(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 248 "QC_HashIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getKey(xsink);
}

// any HashIterator::getKeyValue(){}
static AbstractQoreNode* HashIterator_getKeyValue(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 208 "QC_HashIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getReferencedKeyValue(xsink);
}

// any HashIterator::getValue(){}
static AbstractQoreNode* HashIterator_getValue(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 187 "QC_HashIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getReferencedValue(xsink);
}

// hash HashIterator::getValuePair(){}
static AbstractQoreNode* HashIterator_getValuePair(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 229 "QC_HashIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getReferencedValuePair(xsink);
}

// bool HashIterator::last(){}
static bool HashIterator_last(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 168 "QC_HashIterator.qpp"
return i->last();
}

// bool HashIterator::next(){}
static bool HashIterator_next(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 102 "QC_HashIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

// bool HashIterator::prev(){}
static bool HashIterator_prev(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 123 "QC_HashIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing HashIterator::reset(){}
static AbstractQoreNode* HashIterator_reset(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 277 "QC_HashIterator.qpp"
if (i->check(xsink))
       return 0;
   i->reset();
   return 0;
}

// bool HashIterator::valid(){}
static bool HashIterator_valid(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 263 "QC_HashIterator.qpp"
return i->valid();
}

DLLLOCAL QoreClass* initHashIteratorClass(QoreNamespace& ns) {
   QC_HASHITERATOR = new QoreClass("HashIterator", QDOM_DEFAULT);
   CID_HASHITERATOR = QC_HASHITERATOR->getID();

   // set parent class
   assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
   QC_HASHITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

   // HashIterator::constructor(hash h) {}
   QC_HASHITERATOR->setConstructorExtended3(HashIterator_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "h");

   // HashIterator::constructor() {}
   QC_HASHITERATOR->setConstructorExtended3(HashIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // HashIterator::copy() {}
   QC_HASHITERATOR->setCopy((q_copy_t)HashIterator_copy);

   // bool HashIterator::empty(){}
   QC_HASHITERATOR->addMethodExtended3("empty", (q_method_bool_t)HashIterator_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HashIterator::first(){}
   QC_HASHITERATOR->addMethodExtended3("first", (q_method_bool_t)HashIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // string HashIterator::getKey(){}
   QC_HASHITERATOR->addMethodExtended3("getKey", (q_method_t)HashIterator_getKey, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   // any HashIterator::getKeyValue(){}
   QC_HASHITERATOR->addMethodExtended3("getKeyValue", (q_method_t)HashIterator_getKeyValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo);

   // any HashIterator::getValue(){}
   QC_HASHITERATOR->addMethodExtended3("getValue", (q_method_t)HashIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo);

   // hash HashIterator::getValuePair(){}
   QC_HASHITERATOR->addMethodExtended3("getValuePair", (q_method_t)HashIterator_getValuePair, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

   // bool HashIterator::last(){}
   QC_HASHITERATOR->addMethodExtended3("last", (q_method_bool_t)HashIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HashIterator::next(){}
   QC_HASHITERATOR->addMethodExtended3("next", (q_method_bool_t)HashIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool HashIterator::prev(){}
   QC_HASHITERATOR->addMethodExtended3("prev", (q_method_bool_t)HashIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing HashIterator::reset(){}
   QC_HASHITERATOR->addMethodExtended3("reset", (q_method_t)HashIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool HashIterator::valid(){}
   QC_HASHITERATOR->addMethodExtended3("valid", (q_method_bool_t)HashIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_HASHITERATOR;
}
