/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashKeyIterator.qpp HashKeyIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreHashIterator.h>
/* Qore class Qore::HashKeyIterator */

qore_classid_t CID_HASHKEYITERATOR;
QoreClass* QC_HASHKEYITERATOR;

// HashKeyIterator::constructor(hash h) {}
static void HashKeyIterator_constructor_Vh(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* h = HARD_QORE_HASH(args, 0);
# 66 "QC_HashKeyIterator.qpp"
self->setPrivate(CID_HASHKEYITERATOR, new QoreHashIterator(h));
}

// HashKeyIterator::constructor() {}
static void HashKeyIterator_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 77 "QC_HashKeyIterator.qpp"
self->setPrivate(CID_HASHKEYITERATOR, new QoreHashIterator);
}

// HashKeyIterator::copy() {}
static void HashKeyIterator_copy(QoreObject* self, QoreObject* old, QoreHashIterator* i, ExceptionSink* xsink) {
# 87 "QC_HashKeyIterator.qpp"
self->setPrivate(CID_HASHKEYITERATOR, new QoreHashIterator(*i));
}

// string HashKeyIterator::getValue(){}
static AbstractQoreNode* HashKeyIterator_getValue(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 103 "QC_HashKeyIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getKey(xsink);
}

DLLLOCAL QoreClass* initHashKeyIteratorClass(QoreNamespace& ns) {
   QC_HASHKEYITERATOR = new QoreClass("HashKeyIterator", QDOM_DEFAULT);
   CID_HASHKEYITERATOR = QC_HASHKEYITERATOR->getID();

   // set parent class
   assert(QC_HASHITERATOR);
   QC_HASHKEYITERATOR->addBuiltinVirtualBaseClass(QC_HASHITERATOR);

   // HashKeyIterator::constructor(hash h) {}
   QC_HASHKEYITERATOR->setConstructorExtended3(HashKeyIterator_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "h");

   // HashKeyIterator::constructor() {}
   QC_HASHKEYITERATOR->setConstructorExtended3(HashKeyIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // HashKeyIterator::copy() {}
   QC_HASHKEYITERATOR->setCopy((q_copy_t)HashKeyIterator_copy);

   // string HashKeyIterator::getValue(){}
   QC_HASHKEYITERATOR->addMethodExtended3("getValue", (q_method_t)HashKeyIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

   return QC_HASHKEYITERATOR;
}
