/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashListIterator.qpp HashListIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreHashListIterator.h>
/* Qore class Qore::HashListIterator */

qore_classid_t CID_HASHLISTITERATOR;
QoreClass* QC_HASHLISTITERATOR;

// HashListIterator::constructor(hash h) {}
static void HashListIterator_constructor_Vh(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* h = HARD_QORE_HASH(args, 0);
# 66 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator(h));
}

// HashListIterator::constructor() {}
static void HashListIterator_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 77 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator);
}

// HashListIterator::copy() {}
static void HashListIterator_copy(QoreObject* self, QoreObject* old, QoreHashListIterator* i, ExceptionSink* xsink) {
# 87 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator(*i));
}

// bool HashListIterator::empty(){}
static bool HashListIterator_empty(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 142 "QC_HashListIterator.qpp"
return i->empty();
}

// bool HashListIterator::first(){}
static bool HashListIterator_first(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 157 "QC_HashListIterator.qpp"
return i->first();
}

// any HashListIterator::getKeyValue(string key){}
static AbstractQoreNode* HashListIterator_getKeyValue_Vs(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 311 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return 0;

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// hash HashListIterator::getRow(){}
static AbstractQoreNode* HashListIterator_getRow(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 215 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getRow(xsink);
}

// hash HashListIterator::getValue(){}
static AbstractQoreNode* HashListIterator_getValue(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 193 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getRow(xsink);
}

// int HashListIterator::index(){}
static int64 HashListIterator_index(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 231 "QC_HashListIterator.qpp"
return i->index();
}

// bool HashListIterator::last(){}
static bool HashListIterator_last(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 172 "QC_HashListIterator.qpp"
return i->last();
}

// int HashListIterator::max(){}
static int64 HashListIterator_max(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 245 "QC_HashListIterator.qpp"
return i->max();
}

// any HashListIterator::memberGate(string key){}
static AbstractQoreNode* HashListIterator_memberGate_Vs(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 286 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return 0;

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool HashListIterator::next(){}
static bool HashListIterator_next(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 106 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

// bool HashListIterator::prev(){}
static bool HashListIterator_prev(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 127 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing HashListIterator::reset(){}
static AbstractQoreNode* HashListIterator_reset(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 341 "QC_HashListIterator.qpp"
if (i->check(xsink))
       return 0;
   i->reset();
   return 0;
}

// bool HashListIterator::set(int pos){}
static bool HashListIterator_set_Vi(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   int64 pos = HARD_QORE_INT(args, 0);
# 262 "QC_HashListIterator.qpp"
if (i->check(xsink))
      return false;
   return !i->set(pos);
}

// bool HashListIterator::valid(){}
static bool HashListIterator_valid(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 327 "QC_HashListIterator.qpp"
return i->valid();
}

DLLLOCAL QoreClass* initHashListIteratorClass(QoreNamespace& ns) {
   QC_HASHLISTITERATOR = new QoreClass("HashListIterator", QDOM_DEFAULT);
   CID_HASHLISTITERATOR = QC_HASHLISTITERATOR->getID();

   // set parent class
   assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
   QC_HASHLISTITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

   // HashListIterator::constructor(hash h) {}
   QC_HASHLISTITERATOR->setConstructorExtended3(HashListIterator_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "h");

   // HashListIterator::constructor() {}
   QC_HASHLISTITERATOR->setConstructorExtended3(HashListIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // HashListIterator::copy() {}
   QC_HASHLISTITERATOR->setCopy((q_copy_t)HashListIterator_copy);

   // bool HashListIterator::empty(){}
   QC_HASHLISTITERATOR->addMethodExtended3("empty", (q_method_bool_t)HashListIterator_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HashListIterator::first(){}
   QC_HASHLISTITERATOR->addMethodExtended3("first", (q_method_bool_t)HashListIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // any HashListIterator::getKeyValue(string key){}
   QC_HASHLISTITERATOR->addMethodExtended3("getKeyValue", (q_method_t)HashListIterator_getKeyValue_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // hash HashListIterator::getRow(){}
   QC_HASHLISTITERATOR->addMethodExtended3("getRow", (q_method_t)HashListIterator_getRow, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

   // hash HashListIterator::getValue(){}
   QC_HASHLISTITERATOR->addMethodExtended3("getValue", (q_method_t)HashListIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

   // int HashListIterator::index(){}
   QC_HASHLISTITERATOR->addMethodExtended3("index", (q_method_int64_t)HashListIterator_index, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool HashListIterator::last(){}
   QC_HASHLISTITERATOR->addMethodExtended3("last", (q_method_bool_t)HashListIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int HashListIterator::max(){}
   QC_HASHLISTITERATOR->addMethodExtended3("max", (q_method_int64_t)HashListIterator_max, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // any HashListIterator::memberGate(string key){}
   QC_HASHLISTITERATOR->addMethodExtended3("memberGate", (q_method_t)HashListIterator_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // bool HashListIterator::next(){}
   QC_HASHLISTITERATOR->addMethodExtended3("next", (q_method_bool_t)HashListIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool HashListIterator::prev(){}
   QC_HASHLISTITERATOR->addMethodExtended3("prev", (q_method_bool_t)HashListIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing HashListIterator::reset(){}
   QC_HASHLISTITERATOR->addMethodExtended3("reset", (q_method_t)HashListIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool HashListIterator::set(int pos){}
   QC_HASHLISTITERATOR->addMethodExtended3("set", (q_method_bool_t)HashListIterator_set_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, NULL, "pos");

   // bool HashListIterator::valid(){}
   QC_HASHLISTITERATOR->addMethodExtended3("valid", (q_method_bool_t)HashListIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_HASHLISTITERATOR;
}
