/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashListIterator.qpp HashListIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreHashListIterator.h>

// internal reverse iterator class implementation only for the getName() function - the iterators remain
// forwards and are used in the reverse sense by the Qore language class implementation below
class QoreHashListReverseIterator : public QoreHashListIterator {
public:
   DLLLOCAL QoreHashListReverseIterator(const QoreHashNode* n_h) : QoreHashListIterator(n_h) {
   }

   DLLLOCAL QoreHashListReverseIterator() {
   }

   DLLLOCAL QoreHashListReverseIterator(const QoreHashListReverseIterator& old) : QoreHashListIterator(old) {
   }

   DLLLOCAL virtual const char* getName() const {
      return "HashListReverseIterator";
   }
};
/* Qore class Qore::HashListReverseIterator */

qore_classid_t CID_HASHLISTREVERSEITERATOR;
QoreClass* QC_HASHLISTREVERSEITERATOR;

// HashListReverseIterator::constructor(hash h) {}
static void HashListReverseIterator_constructor_Vh(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* h = HARD_QORE_HASH(args, 0);
# 90 "QC_HashListReverseIterator.qpp"
self->setPrivate(CID_HASHLISTREVERSEITERATOR, new QoreHashListReverseIterator(h));
}

// HashListReverseIterator::constructor() {}
static void HashListReverseIterator_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 101 "QC_HashListReverseIterator.qpp"
self->setPrivate(CID_HASHLISTREVERSEITERATOR, new QoreHashListReverseIterator);
}

// HashListReverseIterator::copy() {}
static void HashListReverseIterator_copy(QoreObject* self, QoreObject* old, QoreHashListReverseIterator* i, ExceptionSink* xsink) {
# 111 "QC_HashListReverseIterator.qpp"
self->setPrivate(CID_HASHLISTREVERSEITERATOR, new QoreHashListReverseIterator(*i));
}

// bool HashListReverseIterator::first(){}
static bool HashListReverseIterator_first(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 168 "QC_HashListReverseIterator.qpp"
return i->last();
}

// bool HashListReverseIterator::last(){}
static bool HashListReverseIterator_last(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 183 "QC_HashListReverseIterator.qpp"
return i->first();
}

// any HashListReverseIterator::memberGate(string key){}
static AbstractQoreNode* HashListReverseIterator_memberGate_Vs(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 205 "QC_HashListReverseIterator.qpp"
if (i->check(xsink))
      return 0;

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool HashListReverseIterator::next(){}
static bool HashListReverseIterator_next(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 130 "QC_HashListReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// bool HashListReverseIterator::prev(){}
static bool HashListReverseIterator_prev(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 151 "QC_HashListReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

DLLLOCAL QoreClass* initHashListReverseIteratorClass(QoreNamespace& ns) {
   QC_HASHLISTREVERSEITERATOR = new QoreClass("HashListReverseIterator", QDOM_DEFAULT);
   CID_HASHLISTREVERSEITERATOR = QC_HASHLISTREVERSEITERATOR->getID();

   // set parent class
   assert(QC_HASHLISTITERATOR);
   QC_HASHLISTREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_HASHLISTITERATOR);

   // HashListReverseIterator::constructor(hash h) {}
   QC_HASHLISTREVERSEITERATOR->setConstructorExtended3(HashListReverseIterator_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "h");

   // HashListReverseIterator::constructor() {}
   QC_HASHLISTREVERSEITERATOR->setConstructorExtended3(HashListReverseIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // HashListReverseIterator::copy() {}
   QC_HASHLISTREVERSEITERATOR->setCopy((q_copy_t)HashListReverseIterator_copy);

   // bool HashListReverseIterator::first(){}
   QC_HASHLISTREVERSEITERATOR->addMethodExtended3("first", (q_method_bool_t)HashListReverseIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HashListReverseIterator::last(){}
   QC_HASHLISTREVERSEITERATOR->addMethodExtended3("last", (q_method_bool_t)HashListReverseIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // any HashListReverseIterator::memberGate(string key){}
   QC_HASHLISTREVERSEITERATOR->addMethodExtended3("memberGate", (q_method_t)HashListReverseIterator_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // bool HashListReverseIterator::next(){}
   QC_HASHLISTREVERSEITERATOR->addMethodExtended3("next", (q_method_bool_t)HashListReverseIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool HashListReverseIterator::prev(){}
   QC_HASHLISTREVERSEITERATOR->addMethodExtended3("prev", (q_method_bool_t)HashListReverseIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   return QC_HASHLISTREVERSEITERATOR;
}
