/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashReverseIterator.qpp HashReverseIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreHashIterator.h>
/* Qore class Qore::HashReverseIterator */

qore_classid_t CID_HASHREVERSEITERATOR;
QoreClass* QC_HASHREVERSEITERATOR;

// HashReverseIterator::constructor(hash h) {}
static void HashReverseIterator_constructor_Vh(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreHashNode* h = HARD_QORE_HASH(args, 0);
# 67 "QC_HashReverseIterator.qpp"
self->setPrivate(CID_HASHREVERSEITERATOR, new QoreHashReverseIterator(h));
}

// HashReverseIterator::constructor() {}
static void HashReverseIterator_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 78 "QC_HashReverseIterator.qpp"
self->setPrivate(CID_HASHREVERSEITERATOR, new QoreHashReverseIterator);
}

// HashReverseIterator::copy() {}
static void HashReverseIterator_copy(QoreObject* self, QoreObject* old, QoreHashReverseIterator* i, ExceptionSink* xsink) {
# 88 "QC_HashReverseIterator.qpp"
self->setPrivate(CID_HASHREVERSEITERATOR, new QoreHashReverseIterator(*i));
}

// bool HashReverseIterator::first(){}
static bool HashReverseIterator_first(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 151 "QC_HashReverseIterator.qpp"
return i->last();
}

// bool HashReverseIterator::last(){}
static bool HashReverseIterator_last(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 166 "QC_HashReverseIterator.qpp"
return i->first();
}

// bool HashReverseIterator::next(){}
static bool HashReverseIterator_next(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 110 "QC_HashReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// bool HashReverseIterator::prev(){}
static bool HashReverseIterator_prev(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 134 "QC_HashReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

DLLLOCAL QoreClass* initHashReverseIteratorClass(QoreNamespace& ns) {
   QC_HASHREVERSEITERATOR = new QoreClass("HashReverseIterator", QDOM_DEFAULT);
   CID_HASHREVERSEITERATOR = QC_HASHREVERSEITERATOR->getID();

   // set parent class
   assert(QC_HASHITERATOR);
   QC_HASHREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_HASHITERATOR);

   // HashReverseIterator::constructor(hash h) {}
   QC_HASHREVERSEITERATOR->setConstructorExtended3(HashReverseIterator_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "h");

   // HashReverseIterator::constructor() {}
   QC_HASHREVERSEITERATOR->setConstructorExtended3(HashReverseIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // HashReverseIterator::copy() {}
   QC_HASHREVERSEITERATOR->setCopy((q_copy_t)HashReverseIterator_copy);

   // bool HashReverseIterator::first(){}
   QC_HASHREVERSEITERATOR->addMethodExtended3("first", (q_method_bool_t)HashReverseIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HashReverseIterator::last(){}
   QC_HASHREVERSEITERATOR->addMethodExtended3("last", (q_method_bool_t)HashReverseIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool HashReverseIterator::next(){}
   QC_HASHREVERSEITERATOR->addMethodExtended3("next", (q_method_bool_t)HashReverseIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool HashReverseIterator::prev(){}
   QC_HASHREVERSEITERATOR->addMethodExtended3("prev", (q_method_bool_t)HashReverseIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   return QC_HASHREVERSEITERATOR;
}
