/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListHashIterator.qpp ListHashIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreListHashIterator.h>
/* Qore class Qore::ListHashIterator */

qore_classid_t CID_LISTHASHITERATOR;
QoreClass* QC_LISTHASHITERATOR;

// ListHashIterator::constructor(softlist l) {}
static void ListHashIterator_constructor_vl(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 66 "QC_ListHashIterator.qpp"
self->setPrivate(CID_LISTHASHITERATOR, new QoreListHashIterator(l));
}

// ListHashIterator::copy() {}
static void ListHashIterator_copy(QoreObject* self, QoreObject* old, QoreListHashIterator* i, ExceptionSink* xsink) {
# 76 "QC_ListHashIterator.qpp"
self->setPrivate(CID_LISTHASHITERATOR, new QoreListHashIterator(*i));
}

// bool ListHashIterator::empty(){}
static bool ListHashIterator_empty(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 131 "QC_ListHashIterator.qpp"
return i->empty();
}

// bool ListHashIterator::first(){}
static bool ListHashIterator_first(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 146 "QC_ListHashIterator.qpp"
return i->first();
}

// any ListHashIterator::getKeyValue(string key){}
static AbstractQoreNode* ListHashIterator_getKeyValue_Vs(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 296 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return 0;

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// hash ListHashIterator::getRow(){}
static AbstractQoreNode* ListHashIterator_getRow(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 202 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getRow(xsink);
}

// hash ListHashIterator::getValue(){}
static AbstractQoreNode* ListHashIterator_getValue(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 181 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getRow(xsink);
}

// int ListHashIterator::index(){}
static int64 ListHashIterator_index(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 218 "QC_ListHashIterator.qpp"
return i->index();
}

// bool ListHashIterator::last(){}
static bool ListHashIterator_last(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 161 "QC_ListHashIterator.qpp"
return i->last();
}

// int ListHashIterator::max(){}
static int64 ListHashIterator_max(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 232 "QC_ListHashIterator.qpp"
return i->max();
}

// any ListHashIterator::memberGate(string key){}
static AbstractQoreNode* ListHashIterator_memberGate_Vs(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 272 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return 0;

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool ListHashIterator::next(){}
static bool ListHashIterator_next(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 95 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

// bool ListHashIterator::prev(){}
static bool ListHashIterator_prev(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 116 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing ListHashIterator::reset(){}
static AbstractQoreNode* ListHashIterator_reset(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 326 "QC_ListHashIterator.qpp"
if (i->check(xsink))
       return 0;
   i->reset();
   return 0;
}

// bool ListHashIterator::set(int pos){}
static bool ListHashIterator_set_Vi(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   int64 pos = HARD_QORE_INT(args, 0);
# 249 "QC_ListHashIterator.qpp"
if (i->check(xsink))
      return false;
   return !i->set(pos);
}

// bool ListHashIterator::valid(){}
static bool ListHashIterator_valid(QoreObject* self, QoreListHashIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 312 "QC_ListHashIterator.qpp"
return i->valid();
}

DLLLOCAL QoreClass* initListHashIteratorClass(QoreNamespace& ns) {
   QC_LISTHASHITERATOR = new QoreClass("ListHashIterator", QDOM_DEFAULT);
   CID_LISTHASHITERATOR = QC_LISTHASHITERATOR->getID();

   // set parent class
   assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
   QC_LISTHASHITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

   // ListHashIterator::constructor(softlist l) {}
   QC_LISTHASHITERATOR->setConstructorExtended3(ListHashIterator_constructor_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softListTypeInfo, NULL, "l");

   // ListHashIterator::copy() {}
   QC_LISTHASHITERATOR->setCopy((q_copy_t)ListHashIterator_copy);

   // bool ListHashIterator::empty(){}
   QC_LISTHASHITERATOR->addMethodExtended3("empty", (q_method_bool_t)ListHashIterator_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ListHashIterator::first(){}
   QC_LISTHASHITERATOR->addMethodExtended3("first", (q_method_bool_t)ListHashIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // any ListHashIterator::getKeyValue(string key){}
   QC_LISTHASHITERATOR->addMethodExtended3("getKeyValue", (q_method_t)ListHashIterator_getKeyValue_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // hash ListHashIterator::getRow(){}
   QC_LISTHASHITERATOR->addMethodExtended3("getRow", (q_method_t)ListHashIterator_getRow, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

   // hash ListHashIterator::getValue(){}
   QC_LISTHASHITERATOR->addMethodExtended3("getValue", (q_method_t)ListHashIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

   // int ListHashIterator::index(){}
   QC_LISTHASHITERATOR->addMethodExtended3("index", (q_method_int64_t)ListHashIterator_index, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool ListHashIterator::last(){}
   QC_LISTHASHITERATOR->addMethodExtended3("last", (q_method_bool_t)ListHashIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int ListHashIterator::max(){}
   QC_LISTHASHITERATOR->addMethodExtended3("max", (q_method_int64_t)ListHashIterator_max, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // any ListHashIterator::memberGate(string key){}
   QC_LISTHASHITERATOR->addMethodExtended3("memberGate", (q_method_t)ListHashIterator_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // bool ListHashIterator::next(){}
   QC_LISTHASHITERATOR->addMethodExtended3("next", (q_method_bool_t)ListHashIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool ListHashIterator::prev(){}
   QC_LISTHASHITERATOR->addMethodExtended3("prev", (q_method_bool_t)ListHashIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing ListHashIterator::reset(){}
   QC_LISTHASHITERATOR->addMethodExtended3("reset", (q_method_t)ListHashIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool ListHashIterator::set(int pos){}
   QC_LISTHASHITERATOR->addMethodExtended3("set", (q_method_bool_t)ListHashIterator_set_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, NULL, "pos");

   // bool ListHashIterator::valid(){}
   QC_LISTHASHITERATOR->addMethodExtended3("valid", (q_method_bool_t)ListHashIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_LISTHASHITERATOR;
}
