/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListHashIterator.qpp ListHashIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreListHashIterator.h>

// internal reverse iterator class implementation only for the getName() function - the iterators remain
// forwards and are used in the reverse sense by the Qore language class implementation below
class QoreListHashReverseIterator : public QoreListHashIterator {
public:
   DLLLOCAL QoreListHashReverseIterator(const QoreListNode* n_l) : QoreListHashIterator(n_l) {
   }

   DLLLOCAL QoreListHashReverseIterator(const QoreListHashReverseIterator& old) : QoreListHashIterator(old) {
   }

   DLLLOCAL virtual const char* getName() const {
      return "ListHashReverseIterator";
   }
};
/* Qore class Qore::ListHashReverseIterator */

qore_classid_t CID_LISTHASHREVERSEITERATOR;
QoreClass* QC_LISTHASHREVERSEITERATOR;

// ListHashReverseIterator::constructor(softlist l) {}
static void ListHashReverseIterator_constructor_vl(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 86 "QC_ListHashReverseIterator.qpp"
self->setPrivate(CID_LISTHASHREVERSEITERATOR, new QoreListHashReverseIterator(l));
}

// ListHashReverseIterator::copy() {}
static void ListHashReverseIterator_copy(QoreObject* self, QoreObject* old, QoreListHashReverseIterator* i, ExceptionSink* xsink) {
# 96 "QC_ListHashReverseIterator.qpp"
self->setPrivate(CID_LISTHASHREVERSEITERATOR, new QoreListHashReverseIterator(*i));
}

// bool ListHashReverseIterator::first(){}
static bool ListHashReverseIterator_first(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 153 "QC_ListHashReverseIterator.qpp"
return i->last();
}

// bool ListHashReverseIterator::last(){}
static bool ListHashReverseIterator_last(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 168 "QC_ListHashReverseIterator.qpp"
return i->first();
}

// any ListHashReverseIterator::memberGate(string key){}
static AbstractQoreNode* ListHashReverseIterator_memberGate_Vs(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 189 "QC_ListHashReverseIterator.qpp"
if (i->check(xsink))
      return 0;

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool ListHashReverseIterator::next(){}
static bool ListHashReverseIterator_next(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 115 "QC_ListHashReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// bool ListHashReverseIterator::prev(){}
static bool ListHashReverseIterator_prev(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 136 "QC_ListHashReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

DLLLOCAL QoreClass* initListHashReverseIteratorClass(QoreNamespace& ns) {
   QC_LISTHASHREVERSEITERATOR = new QoreClass("ListHashReverseIterator", QDOM_DEFAULT);
   CID_LISTHASHREVERSEITERATOR = QC_LISTHASHREVERSEITERATOR->getID();

   // set parent class
   assert(QC_LISTHASHITERATOR);
   QC_LISTHASHREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_LISTHASHITERATOR);

   // ListHashReverseIterator::constructor(softlist l) {}
   QC_LISTHASHREVERSEITERATOR->setConstructorExtended3(ListHashReverseIterator_constructor_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softListTypeInfo, NULL, "l");

   // ListHashReverseIterator::copy() {}
   QC_LISTHASHREVERSEITERATOR->setCopy((q_copy_t)ListHashReverseIterator_copy);

   // bool ListHashReverseIterator::first(){}
   QC_LISTHASHREVERSEITERATOR->addMethodExtended3("first", (q_method_bool_t)ListHashReverseIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ListHashReverseIterator::last(){}
   QC_LISTHASHREVERSEITERATOR->addMethodExtended3("last", (q_method_bool_t)ListHashReverseIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // any ListHashReverseIterator::memberGate(string key){}
   QC_LISTHASHREVERSEITERATOR->addMethodExtended3("memberGate", (q_method_t)ListHashReverseIterator_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // bool ListHashReverseIterator::next(){}
   QC_LISTHASHREVERSEITERATOR->addMethodExtended3("next", (q_method_bool_t)ListHashReverseIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool ListHashReverseIterator::prev(){}
   QC_LISTHASHREVERSEITERATOR->addMethodExtended3("prev", (q_method_bool_t)ListHashReverseIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   return QC_LISTHASHREVERSEITERATOR;
}
