/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListIterator.qpp ListIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreListIterator.h>
/* Qore class Qore::ListIterator */

qore_classid_t CID_LISTITERATOR;
QoreClass* QC_LISTITERATOR;

// ListIterator::constructor(softlist l) {}
static void ListIterator_constructor_vl(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 64 "QC_ListIterator.qpp"
self->setPrivate(CID_LISTITERATOR, new QoreListIterator(l));
}

// ListIterator::copy() {}
static void ListIterator_copy(QoreObject* self, QoreObject* old, QoreListIterator* i, ExceptionSink* xsink) {
# 74 "QC_ListIterator.qpp"
self->setPrivate(CID_LISTITERATOR, new QoreListIterator(*i));
}

// bool ListIterator::empty(){}
static bool ListIterator_empty(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 129 "QC_ListIterator.qpp"
return i->empty();
}

// bool ListIterator::first(){}
static bool ListIterator_first(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 144 "QC_ListIterator.qpp"
return i->first();
}

// any ListIterator::getValue(){}
static AbstractQoreNode* ListIterator_getValue(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 176 "QC_ListIterator.qpp"
if (i->check(xsink))
      return 0;
   return i->getReferencedValue(xsink);
}

// int ListIterator::index(){}
static int64 ListIterator_index(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 192 "QC_ListIterator.qpp"
return i->index();
}

// bool ListIterator::last(){}
static bool ListIterator_last(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 159 "QC_ListIterator.qpp"
return i->last();
}

// int ListIterator::max(){}
static int64 ListIterator_max(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 206 "QC_ListIterator.qpp"
return i->max();
}

// bool ListIterator::next(){}
static bool ListIterator_next(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 93 "QC_ListIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

// bool ListIterator::prev(){}
static bool ListIterator_prev(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 114 "QC_ListIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing ListIterator::reset(){}
static AbstractQoreNode* ListIterator_reset(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 252 "QC_ListIterator.qpp"
if (i->check(xsink))
       return 0;
   i->reset();
   return 0;
}

// bool ListIterator::set(int pos){}
static bool ListIterator_set_Vi(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
   int64 pos = HARD_QORE_INT(args, 0);
# 223 "QC_ListIterator.qpp"
if (i->check(xsink))
      return false;
   return !i->set(pos);
}

// bool ListIterator::valid(){}
static bool ListIterator_valid(QoreObject* self, QoreListIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 238 "QC_ListIterator.qpp"
return i->valid();
}

DLLLOCAL QoreClass* initListIteratorClass(QoreNamespace& ns) {
   QC_LISTITERATOR = new QoreClass("ListIterator", QDOM_DEFAULT);
   CID_LISTITERATOR = QC_LISTITERATOR->getID();

   // set parent class
   assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
   QC_LISTITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

   // ListIterator::constructor(softlist l) {}
   QC_LISTITERATOR->setConstructorExtended3(ListIterator_constructor_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softListTypeInfo, NULL, "l");

   // ListIterator::copy() {}
   QC_LISTITERATOR->setCopy((q_copy_t)ListIterator_copy);

   // bool ListIterator::empty(){}
   QC_LISTITERATOR->addMethodExtended3("empty", (q_method_bool_t)ListIterator_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ListIterator::first(){}
   QC_LISTITERATOR->addMethodExtended3("first", (q_method_bool_t)ListIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // any ListIterator::getValue(){}
   QC_LISTITERATOR->addMethodExtended3("getValue", (q_method_t)ListIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo);

   // int ListIterator::index(){}
   QC_LISTITERATOR->addMethodExtended3("index", (q_method_int64_t)ListIterator_index, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool ListIterator::last(){}
   QC_LISTITERATOR->addMethodExtended3("last", (q_method_bool_t)ListIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int ListIterator::max(){}
   QC_LISTITERATOR->addMethodExtended3("max", (q_method_int64_t)ListIterator_max, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool ListIterator::next(){}
   QC_LISTITERATOR->addMethodExtended3("next", (q_method_bool_t)ListIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool ListIterator::prev(){}
   QC_LISTITERATOR->addMethodExtended3("prev", (q_method_bool_t)ListIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing ListIterator::reset(){}
   QC_LISTITERATOR->addMethodExtended3("reset", (q_method_t)ListIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool ListIterator::set(int pos){}
   QC_LISTITERATOR->addMethodExtended3("set", (q_method_bool_t)ListIterator_set_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, NULL, "pos");

   // bool ListIterator::valid(){}
   QC_LISTITERATOR->addMethodExtended3("valid", (q_method_bool_t)ListIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_LISTITERATOR;
}
