/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListReverseIterator.qpp ListReverseIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreListIterator.h>

// internal reverse iterator class implementation only for the getName() function - the iterators remain
// forwards and are used in the reverse sense by the Qore language class implementation below
class QoreListReverseIterator : public QoreListIterator {
public:
   DLLLOCAL QoreListReverseIterator(const QoreListNode* l) : QoreListIterator(l) {
   }

   DLLLOCAL QoreListReverseIterator(const QoreListReverseIterator& old) : QoreListIterator(old) {
   }

   DLLLOCAL virtual const char* getName() const {
      return "ListReverseIterator";
   }
};
/* Qore class Qore::ListReverseIterator */

qore_classid_t CID_LISTREVERSEITERATOR;
QoreClass* QC_LISTREVERSEITERATOR;

// ListReverseIterator::constructor(softlist l) {}
static void ListReverseIterator_constructor_vl(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 83 "QC_ListReverseIterator.qpp"
self->setPrivate(CID_LISTREVERSEITERATOR, new QoreListReverseIterator(l));
}

// ListReverseIterator::copy() {}
static void ListReverseIterator_copy(QoreObject* self, QoreObject* old, QoreListReverseIterator* i, ExceptionSink* xsink) {
# 93 "QC_ListReverseIterator.qpp"
self->setPrivate(CID_LISTREVERSEITERATOR, new QoreListReverseIterator(*i));
}

// bool ListReverseIterator::first(){}
static bool ListReverseIterator_first(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 156 "QC_ListReverseIterator.qpp"
return i->last();
}

// bool ListReverseIterator::last(){}
static bool ListReverseIterator_last(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 171 "QC_ListReverseIterator.qpp"
return i->first();
}

// bool ListReverseIterator::next(){}
static bool ListReverseIterator_next(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 115 "QC_ListReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// bool ListReverseIterator::prev(){}
static bool ListReverseIterator_prev(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 139 "QC_ListReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

DLLLOCAL QoreClass* initListReverseIteratorClass(QoreNamespace& ns) {
   QC_LISTREVERSEITERATOR = new QoreClass("ListReverseIterator", QDOM_DEFAULT);
   CID_LISTREVERSEITERATOR = QC_LISTREVERSEITERATOR->getID();

   // set parent class
   assert(QC_LISTITERATOR);
   QC_LISTREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_LISTITERATOR);

   // ListReverseIterator::constructor(softlist l) {}
   QC_LISTREVERSEITERATOR->setConstructorExtended3(ListReverseIterator_constructor_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softListTypeInfo, NULL, "l");

   // ListReverseIterator::copy() {}
   QC_LISTREVERSEITERATOR->setCopy((q_copy_t)ListReverseIterator_copy);

   // bool ListReverseIterator::first(){}
   QC_LISTREVERSEITERATOR->addMethodExtended3("first", (q_method_bool_t)ListReverseIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ListReverseIterator::last(){}
   QC_LISTREVERSEITERATOR->addMethodExtended3("last", (q_method_bool_t)ListReverseIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ListReverseIterator::next(){}
   QC_LISTREVERSEITERATOR->addMethodExtended3("next", (q_method_bool_t)ListReverseIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool ListReverseIterator::prev(){}
   QC_LISTREVERSEITERATOR->addMethodExtended3("prev", (q_method_bool_t)ListReverseIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   return QC_LISTREVERSEITERATOR;
}
