/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Mutex.qpp

  Qore Programming Language
  
  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_Mutex.h>
#include <qore/intern/QC_AbstractSmartLock.h>
/* Qore class Qore::Thread::Mutex */

qore_classid_t CID_MUTEX;
QoreClass* QC_MUTEX;

// Mutex::constructor() {}
static void Mutex_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 58 "QC_Mutex.qpp"
self->setPrivate(CID_MUTEX, new SmartMutex);
}

// Mutex::copy() {}
static void Mutex_copy(QoreObject* self, QoreObject* old, SmartMutex* m, ExceptionSink* xsink) {
# 83 "QC_Mutex.qpp"
self->setPrivate(CID_MUTEX, new SmartMutex);
}

// Mutex::destructor() {}
static void Mutex_destructor(QoreObject* self, SmartMutex* m, ExceptionSink* xsink) {
# 72 "QC_Mutex.qpp"
m->destructor(xsink);
   m->deref(xsink);
}

// nothing Mutex::lock(){}
static AbstractQoreNode* Mutex_lock(QoreObject* self, SmartMutex* m, const QoreListNode* args, ExceptionSink* xsink) {
# 98 "QC_Mutex.qpp"
m->grab(xsink);
   return 0;
}

// int Mutex::lock(timeout timeout_ms){}
static int64 Mutex_lock_Vt(QoreObject* self, SmartMutex* m, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 120 "QC_Mutex.qpp"
return m->grab(xsink, timeout_ms);
}

// int Mutex::trylock(){}
static int64 Mutex_trylock(QoreObject* self, SmartMutex* m, const QoreListNode* args, ExceptionSink* xsink) {
# 136 "QC_Mutex.qpp"
return m->tryGrab();
}

// nothing Mutex::unlock(){}
static AbstractQoreNode* Mutex_unlock(QoreObject* self, SmartMutex* m, const QoreListNode* args, ExceptionSink* xsink) {
# 149 "QC_Mutex.qpp"
m->release(xsink);
   return 0;
}

DLLLOCAL QoreClass* initMutexClass(QoreNamespace& ns) {
   QC_MUTEX = new QoreClass("Mutex", QDOM_THREAD_CLASS);
   CID_MUTEX = QC_MUTEX->getID();

   // set parent class
   assert(QC_ABSTRACTSMARTLOCK);
   QC_MUTEX->addBuiltinVirtualBaseClass(QC_ABSTRACTSMARTLOCK);

   // Mutex::constructor() {}
   QC_MUTEX->setConstructorExtended3(Mutex_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // Mutex::copy() {}
   QC_MUTEX->setCopy((q_copy_t)Mutex_copy);

   // Mutex::destructor() {}
   QC_MUTEX->setDestructor((q_destructor_t)Mutex_destructor);

   // nothing Mutex::lock(){}
   QC_MUTEX->addMethodExtended3("lock", (q_method_t)Mutex_lock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int Mutex::lock(timeout timeout_ms){}
   QC_MUTEX->addMethodExtended3("lock", (q_method_int64_t)Mutex_lock_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

   // int Mutex::trylock(){}
   QC_MUTEX->addMethodExtended3("trylock", (q_method_int64_t)Mutex_trylock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // nothing Mutex::unlock(){}
   QC_MUTEX->addMethodExtended3("unlock", (q_method_t)Mutex_unlock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   return QC_MUTEX;
}
