/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashReverseIterator.qpp HashReverseIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QoreObjectIterator.h>
/* Qore class Qore::ObjectReverseIterator */

qore_classid_t CID_OBJECTREVERSEITERATOR;
QoreClass* QC_OBJECTREVERSEITERATOR;

// ObjectReverseIterator::constructor(object o) {}
static void ObjectReverseIterator_constructor_Vo(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   QoreObject* o = HARD_QORE_OBJECT(args, 0);
# 76 "QC_ObjectReverseIterator.qpp"
self->setPrivate(CID_OBJECTREVERSEITERATOR, new QoreObjectReverseIterator(o));
}

// ObjectReverseIterator::constructor() {}
static void ObjectReverseIterator_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 87 "QC_ObjectReverseIterator.qpp"
self->setPrivate(CID_OBJECTREVERSEITERATOR, new QoreObjectReverseIterator);
}

// ObjectReverseIterator::copy() {}
static void ObjectReverseIterator_copy(QoreObject* self, QoreObject* old, QoreObjectReverseIterator* i, ExceptionSink* xsink) {
# 97 "QC_ObjectReverseIterator.qpp"
self->setPrivate(CID_OBJECTREVERSEITERATOR, new QoreObjectReverseIterator(*i));
}

// bool ObjectReverseIterator::first(){}
static bool ObjectReverseIterator_first(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 160 "QC_ObjectReverseIterator.qpp"
return i->last();
}

// bool ObjectReverseIterator::last(){}
static bool ObjectReverseIterator_last(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 175 "QC_ObjectReverseIterator.qpp"
return i->first();
}

// bool ObjectReverseIterator::next(){}
static bool ObjectReverseIterator_next(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 119 "QC_ObjectReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->prev();
}

// bool ObjectReverseIterator::prev(){}
static bool ObjectReverseIterator_prev(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 143 "QC_ObjectReverseIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

DLLLOCAL QoreClass* initObjectReverseIteratorClass(QoreNamespace& ns) {
   QC_OBJECTREVERSEITERATOR = new QoreClass("ObjectReverseIterator", QDOM_DEFAULT);
   CID_OBJECTREVERSEITERATOR = QC_OBJECTREVERSEITERATOR->getID();

   // set parent class
   assert(QC_OBJECTITERATOR);
   QC_OBJECTREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_OBJECTITERATOR);

   // ObjectReverseIterator::constructor(object o) {}
   QC_OBJECTREVERSEITERATOR->setConstructorExtended3(ObjectReverseIterator_constructor_Vo, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, objectTypeInfo, NULL, "o");

   // ObjectReverseIterator::constructor() {}
   QC_OBJECTREVERSEITERATOR->setConstructorExtended3(ObjectReverseIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // ObjectReverseIterator::copy() {}
   QC_OBJECTREVERSEITERATOR->setCopy((q_copy_t)ObjectReverseIterator_copy);

   // bool ObjectReverseIterator::first(){}
   QC_OBJECTREVERSEITERATOR->addMethodExtended3("first", (q_method_bool_t)ObjectReverseIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ObjectReverseIterator::last(){}
   QC_OBJECTREVERSEITERATOR->addMethodExtended3("last", (q_method_bool_t)ObjectReverseIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ObjectReverseIterator::next(){}
   QC_OBJECTREVERSEITERATOR->addMethodExtended3("next", (q_method_bool_t)ObjectReverseIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // bool ObjectReverseIterator::prev(){}
   QC_OBJECTREVERSEITERATOR->addMethodExtended3("prev", (q_method_bool_t)ObjectReverseIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   return QC_OBJECTREVERSEITERATOR;
}
