/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_RWLock.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_RWLock.h>
#include <qore/intern/QC_AbstractSmartLock.h>
/* Qore class Qore::Thread::RWLock */

qore_classid_t CID_RWLOCK;
QoreClass* QC_RWLOCK;

// RWLock::constructor() {}
static void RWLock_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 63 "QC_RWLock.qpp"
self->setPrivate(CID_RWLOCK, new RWLock);
}

// RWLock::copy() {}
static void RWLock_copy(QoreObject* self, QoreObject* old, RWLock* rwl, ExceptionSink* xsink) {
# 89 "QC_RWLock.qpp"
self->setPrivate(CID_RWLOCK, new RWLock);
}

// RWLock::destructor() {}
static void RWLock_destructor(QoreObject* self, RWLock* rwl, ExceptionSink* xsink) {
# 77 "QC_RWLock.qpp"
rwl->destructor(xsink);
   rwl->deref(xsink);
}

// int RWLock::getReadWaiting(){}
static int64 RWLock_getReadWaiting(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 237 "QC_RWLock.qpp"
return rwl->getReadWaiting();
}

// int RWLock::getWriteWaiting(){}
static int64 RWLock_getWriteWaiting(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 249 "QC_RWLock.qpp"
return rwl->getWriteWaiting();
}

// bool RWLock::lockOwner(){}
static bool RWLock_lockOwner(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 262 "QC_RWLock.qpp"
return rwl->lockOwner();
}

// int RWLock::numReaders(){}
static int64 RWLock_numReaders(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 225 "QC_RWLock.qpp"
return rwl->numReaders();
}

// nothing RWLock::readLock(){}
static AbstractQoreNode* RWLock_readLock(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 103 "QC_RWLock.qpp"
rwl->readLock(xsink);
   return 0;
}

// int RWLock::readLock(timeout timeout_ms){}
static int64 RWLock_readLock_Vt(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 123 "QC_RWLock.qpp"
return rwl->readLock(xsink, timeout_ms);
}

// bool RWLock::readLockOwner(){}
static bool RWLock_readLockOwner(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 275 "QC_RWLock.qpp"
return rwl->readLockOwner();
}

// nothing RWLock::readUnlock(){}
static AbstractQoreNode* RWLock_readUnlock(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 136 "QC_RWLock.qpp"
rwl->readUnlock(xsink);
   return 0;
}

// int RWLock::tryReadLock(){}
static int64 RWLock_tryReadLock(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 198 "QC_RWLock.qpp"
return rwl->tryReadLock();
}

// int RWLock::tryWriteLock(){}
static int64 RWLock_tryWriteLock(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 212 "QC_RWLock.qpp"
return rwl->tryGrab();
}

// int RWLock::writeLock(timeout timeout_ms){}
static int64 RWLock_writeLock_Vt(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 156 "QC_RWLock.qpp"
return rwl->grab(xsink, timeout_ms);
}

// nothing RWLock::writeLock(){}
static AbstractQoreNode* RWLock_writeLock(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 170 "QC_RWLock.qpp"
rwl->grab(xsink);
   return 0;
}

// bool RWLock::writeLockOwner(){}
static bool RWLock_writeLockOwner(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 288 "QC_RWLock.qpp"
return rwl->writeLockOwner();
}

// nothing RWLock::writeUnlock(){}
static AbstractQoreNode* RWLock_writeUnlock(QoreObject* self, RWLock* rwl, const QoreListNode* args, ExceptionSink* xsink) {
# 183 "QC_RWLock.qpp"
rwl->release(xsink);
   return 0;
}

DLLLOCAL QoreClass* initRWLockClass(QoreNamespace& ns) {
   QC_RWLOCK = new QoreClass("RWLock", QDOM_THREAD_CLASS);
   CID_RWLOCK = QC_RWLOCK->getID();

   // set parent class
   assert(QC_ABSTRACTSMARTLOCK);
   QC_RWLOCK->addBuiltinVirtualBaseClass(QC_ABSTRACTSMARTLOCK);

   // RWLock::constructor() {}
   QC_RWLOCK->setConstructorExtended3(RWLock_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // RWLock::copy() {}
   QC_RWLOCK->setCopy((q_copy_t)RWLock_copy);

   // RWLock::destructor() {}
   QC_RWLOCK->setDestructor((q_destructor_t)RWLock_destructor);

   // int RWLock::getReadWaiting(){}
   QC_RWLOCK->addMethodExtended3("getReadWaiting", (q_method_int64_t)RWLock_getReadWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int RWLock::getWriteWaiting(){}
   QC_RWLOCK->addMethodExtended3("getWriteWaiting", (q_method_int64_t)RWLock_getWriteWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // bool RWLock::lockOwner(){}
   QC_RWLOCK->addMethodExtended3("lockOwner", (q_method_bool_t)RWLock_lockOwner, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int RWLock::numReaders(){}
   QC_RWLOCK->addMethodExtended3("numReaders", (q_method_int64_t)RWLock_numReaders, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // nothing RWLock::readLock(){}
   QC_RWLOCK->addMethodExtended3("readLock", (q_method_t)RWLock_readLock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int RWLock::readLock(timeout timeout_ms){}
   QC_RWLOCK->addMethodExtended3("readLock", (q_method_int64_t)RWLock_readLock_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

   // bool RWLock::readLockOwner(){}
   QC_RWLOCK->addMethodExtended3("readLockOwner", (q_method_bool_t)RWLock_readLockOwner, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // nothing RWLock::readUnlock(){}
   QC_RWLOCK->addMethodExtended3("readUnlock", (q_method_t)RWLock_readUnlock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int RWLock::tryReadLock(){}
   QC_RWLOCK->addMethodExtended3("tryReadLock", (q_method_int64_t)RWLock_tryReadLock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // int RWLock::tryWriteLock(){}
   QC_RWLOCK->addMethodExtended3("tryWriteLock", (q_method_int64_t)RWLock_tryWriteLock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // int RWLock::writeLock(timeout timeout_ms){}
   QC_RWLOCK->addMethodExtended3("writeLock", (q_method_int64_t)RWLock_writeLock_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

   // nothing RWLock::writeLock(){}
   QC_RWLOCK->addMethodExtended3("writeLock", (q_method_t)RWLock_writeLock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool RWLock::writeLockOwner(){}
   QC_RWLOCK->addMethodExtended3("writeLockOwner", (q_method_bool_t)RWLock_writeLockOwner, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // nothing RWLock::writeUnlock(){}
   QC_RWLOCK->addMethodExtended3("writeUnlock", (q_method_t)RWLock_writeUnlock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   return QC_RWLOCK;
}
