/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_RangeIterator.qpp RangeIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 Qore Technologies s.r.o.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/RangeIterator.h>
/* Qore class Qore::RangeIterator */

qore_classid_t CID_RANGEITERATOR;
QoreClass* QC_RANGEITERATOR;

// RangeIterator::constructor(int start, int stop, int step = 1) {}
static void RangeIterator_constructor_ViViVi(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
   int64 stop = HARD_QORE_INT(args, 1);
   int64 step = HARD_QORE_INT(args, 2);
# 67 "QC_RangeIterator.qpp"
self->setPrivate(CID_RANGEITERATOR, new RangeIterator(start, stop, step, xsink));
}

// RangeIterator::copy() {}
static void RangeIterator_copy(QoreObject* self, QoreObject* old, RangeIterator* i, ExceptionSink* xsink) {
# 77 "QC_RangeIterator.qpp"
self->setPrivate(CID_RANGEITERATOR, new RangeIterator(*i));
}

// any RangeIterator::getValue(){}
static AbstractQoreNode* RangeIterator_getValue(QoreObject* self, RangeIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 114 "QC_RangeIterator.qpp"
if (i->check(xsink))
       return 0;
   return i->getValue(xsink);
}

// bool RangeIterator::next(){}
static bool RangeIterator_next(QoreObject* self, RangeIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 95 "QC_RangeIterator.qpp"
if (i->check(xsink))
       return false;
   return i->next();
}

// nothing RangeIterator::reset(){}
static AbstractQoreNode* RangeIterator_reset(QoreObject* self, RangeIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 146 "QC_RangeIterator.qpp"
if (i->check(xsink))
       return 0;
   i->reset();
   return 0;
}

// bool RangeIterator::valid(){}
static bool RangeIterator_valid(QoreObject* self, RangeIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 161 "QC_RangeIterator.qpp"
return i->valid();
}

DLLLOCAL QoreClass* initRangeIteratorClass(QoreNamespace& ns) {
   QC_RANGEITERATOR = new QoreClass("RangeIterator", QDOM_DEFAULT);
   CID_RANGEITERATOR = QC_RANGEITERATOR->getID();

   // set parent class
   assert(QC_ABSTRACTITERATOR);
   QC_RANGEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

   // RangeIterator::constructor(int start, int stop, int step = 1) {}
   QC_RANGEITERATOR->setConstructorExtended3(RangeIterator_constructor_ViViVi, false, QC_NO_FLAGS, QDOM_DEFAULT, 3, bigIntTypeInfo, NULL, "start", bigIntTypeInfo, NULL, "stop", bigIntTypeInfo, new QoreBigIntNode(1), "step");

   // RangeIterator::copy() {}
   QC_RANGEITERATOR->setCopy((q_copy_t)RangeIterator_copy);

   // any RangeIterator::getValue(){}
   QC_RANGEITERATOR->addMethodExtended3("getValue", (q_method_t)RangeIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo);

   // bool RangeIterator::next(){}
   QC_RANGEITERATOR->addMethodExtended3("next", (q_method_bool_t)RangeIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing RangeIterator::reset(){}
   QC_RANGEITERATOR->addMethodExtended3("reset", (q_method_t)RangeIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool RangeIterator::valid(){}
   QC_RANGEITERATOR->addMethodExtended3("valid", (q_method_bool_t)RangeIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_RANGEITERATOR;
}
# 165 "QC_RangeIterator.qpp"
# 213 "QC_RangeIterator.qpp"
// RangeIterator xrange(int start, int stop, int step = 1) {}
static QoreObject* f_xrange_ViViVi(const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
   int64 stop = HARD_QORE_INT(args, 1);
   int64 step = HARD_QORE_INT(args, 2);
# 209 "QC_RangeIterator.qpp"
   return new QoreObject(QC_RANGEITERATOR, 0, new RangeIterator(start, stop, step, xsink));
}

// RangeIterator xrange(int stop) {}
static QoreObject* f_xrange_Vi(const QoreListNode* args, ExceptionSink* xsink) {
   int64 stop = HARD_QORE_INT(args, 0);
# 233 "QC_RangeIterator.qpp"
   return new QoreObject(QC_RANGEITERATOR, 0, new RangeIterator(0, stop, 1, xsink));
}


DLLLOCAL void init_RangeIterator_functions(QoreNamespace& ns) {
   // RangeIterator xrange(int start, int stop, int step = 1) {}
   ns.addBuiltinVariant("xrange", (q_func_t)f_xrange_ViViVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, QC_RANGEITERATOR->getTypeInfo(), 3, bigIntTypeInfo, NULL, "start", bigIntTypeInfo, NULL, "stop", bigIntTypeInfo, new QoreBigIntNode(1), "step");
   // RangeIterator xrange(int stop) {}
   ns.addBuiltinVariant("xrange", (q_func_t)f_xrange_Vi, QC_CONSTANT, QDOM_DEFAULT, QC_RANGEITERATOR->getTypeInfo(), 1, bigIntTypeInfo, NULL, "stop");

}
