/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_ReadOnlyFile.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_File.h>
#include <qore/intern/QC_Queue.h>
/* Qore class Qore::ReadOnlyFile */

qore_classid_t CID_READONLYFILE;
QoreClass* QC_READONLYFILE;

// int ReadOnlyFile::close(){}
static int64 ReadOnlyFile_close(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 276 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::close", xsink))
      return 0;

   return f->close();
}

// ReadOnlyFile::constructor(string path, *string encoding) {}
static void ReadOnlyFile_constructor_VsNs(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 191 "QC_ReadOnlyFile.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

   SimpleRefHolder<File> f(new File(qe));
   if (f->isTty() && runtime_check_parse_option(PO_NO_TERMINAL_IO)) {
      xsink->raiseException("ILLEGAL-EXPRESSION", "ReadOnlyFile::constructor() cannot be called with a TTY target when 'no-terminal-io' is set");
      return;
   }

   if (f->open(path->getBuffer(), O_RDONLY, 0, qe)) {
      xsink->raiseErrnoException("READONLYFILE-OPEN-ERROR", errno, "cannot open '%s'", path->getBuffer());
      return;
   }

   self->setPrivate(CID_READONLYFILE, f.release());
}

// ReadOnlyFile::copy() {}
static void ReadOnlyFile_copy(QoreObject* self, QoreObject* old, File* f, ExceptionSink* xsink) {
# 222 "QC_ReadOnlyFile.qpp"
self->setPrivate(CID_READONLYFILE, new File(f->getEncoding()));
}

// string ReadOnlyFile::getEncoding(){}
static AbstractQoreNode* ReadOnlyFile_getEncoding(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 715 "QC_ReadOnlyFile.qpp"
return new QoreStringNode(f->getEncoding()->getCode());
}

// *string ReadOnlyFile::getFileName(){}
static AbstractQoreNode* ReadOnlyFile_getFileName(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 960 "QC_ReadOnlyFile.qpp"
return f->getFileName();
}

// int ReadOnlyFile::getPos(){}
static int64 ReadOnlyFile_getPos(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 752 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::getPos", xsink))
      return 0;

   return f->getPos();
}

// *string ReadOnlyFile::getchar(){}
static AbstractQoreNode* ReadOnlyFile_getchar(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 773 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::getchar", xsink))
      return 0;

   return f->getchar(xsink);
}

// hash ReadOnlyFile::hstat(){}
static AbstractQoreNode* ReadOnlyFile_hstat(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 889 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::hstat", xsink))
      return 0;

   return f->hstat(xsink);
}

// bool ReadOnlyFile::isDataAvailable(timeout timeout_ms = 0){}
static bool ReadOnlyFile_isDataAvailable_Vt(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   int64 timeout_ms = HARD_QORE_INT(args, 0);
# 795 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::isDataAvailable", xsink))
      return 0;

   return f->isDataAvailable(timeout_ms, xsink);
}

// bool ReadOnlyFile::isOpen(){}
static bool ReadOnlyFile_isOpen(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 932 "QC_ReadOnlyFile.qpp"
return f->isOpen();
}

// bool ReadOnlyFile::isTty(){}
static bool ReadOnlyFile_isTty(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 946 "QC_ReadOnlyFile.qpp"
return f->isTty();
}

// nothing ReadOnlyFile::open(string path, *string encoding){}
static AbstractQoreNode* ReadOnlyFile_open_VsNs(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 253 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::open", xsink))
      return 0;

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : f->getEncoding();
   if (f->open(path->getBuffer(), O_RDONLY, 0, qe))
      xsink->raiseErrnoException("READONLYFILE-OPEN-ERROR", errno, "cannot open '%s'", path->getBuffer());
   return 0;
}

// *string ReadOnlyFile::read(softint size, timeout timeout_ms = -1){}
static AbstractQoreNode* ReadOnlyFile_read_viVt(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   int64 size = HARD_QORE_INT(args, 0);
   int64 timeout_ms = HARD_QORE_INT(args, 1);
# 310 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::read", xsink))
      return 0;

   if (!size) {
      xsink->raiseException("READONLYFILE-READ-PARAMETER-ERROR", "expecting non-zero size as first parameter of ReadOnlyFile::read()");
      return 0;
   }

   return f->read((qore_offset_t)size, timeout_ms, xsink);
}

// *binary ReadOnlyFile::readBinary(softint size, timeout timeout_ms = -1){}
static AbstractQoreNode* ReadOnlyFile_readBinary_viVt(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   int64 size = HARD_QORE_INT(args, 0);
   int64 timeout_ms = HARD_QORE_INT(args, 1);
# 641 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readBinary", xsink))
      return 0;

   if (!size) {
      xsink->raiseException("READONLYFILE-READ-BINARY-PARAMETER-ERROR", "expecting size as first parameter of File::readBinary()");
      return 0;
   }

   return f->readBinary((qore_offset_t)size, timeout_ms, xsink);
}

// *string ReadOnlyFile::readLine(bool incl_eol = True, *string eol){}
static AbstractQoreNode* ReadOnlyFile_readLine_VbNs(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   bool incl_eol = HARD_QORE_BOOL(args, 0);
   const QoreStringNode* eol = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 679 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readLine", xsink))
      return 0;

   if (eol && eol->empty())
      eol = 0;

   return eol ? f->readUntil(eol->getBuffer(), incl_eol, xsink) : f->readLine(incl_eol, xsink);
}

// *int ReadOnlyFile::readi1(){}
static AbstractQoreNode* ReadOnlyFile_readi1(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 458 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi1", xsink))
      return 0;

   char c;
   if (f->readi1(&c, xsink))
      return 0;
   return new QoreBigIntNode(c);
}

// *int ReadOnlyFile::readi2(){}
static AbstractQoreNode* ReadOnlyFile_readi2(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 482 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi2", xsink))
      return 0;

   short s;
   if (f->readi2(&s, xsink))
      return 0;
   return new QoreBigIntNode(s);
}

// *int ReadOnlyFile::readi2LSB(){}
static AbstractQoreNode* ReadOnlyFile_readi2LSB(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 555 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi2LSB", xsink))
      return 0;

   short s;
   if (f->readi2LSB(&s, xsink))
      return 0;

   return new QoreBigIntNode(s);
}

// *int ReadOnlyFile::readi4(){}
static AbstractQoreNode* ReadOnlyFile_readi4(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 506 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi4", xsink))
      return 0;

   int i;
   if (f->readi4(&i, xsink))
      return 0;
   return new QoreBigIntNode(i);
}

// *int ReadOnlyFile::readi4LSB(){}
static AbstractQoreNode* ReadOnlyFile_readi4LSB(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 580 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi4LSB", xsink))
      return 0;

   int i;
   if (f->readi4LSB(&i, xsink))
      return 0;
   return new QoreBigIntNode(i);
}

// *int ReadOnlyFile::readi8(){}
static AbstractQoreNode* ReadOnlyFile_readi8(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 530 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi8", xsink))
      return 0;

   int64 i;
   if (f->readi8(&i, xsink))
      return 0;

   return new QoreBigIntNode(i);
}

// *int ReadOnlyFile::readi8LSB(){}
static AbstractQoreNode* ReadOnlyFile_readi8LSB(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 604 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readi8LSB", xsink))
      return 0;

   int64 i;
   if (f->readi8LSB(&i, xsink))
      return 0;
   return new QoreBigIntNode(i);
}

// *int ReadOnlyFile::readu1(){}
static AbstractQoreNode* ReadOnlyFile_readu1(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 336 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readu1", xsink))
      return 0;

   unsigned char c;
   if (f->readu1(&c, xsink))
      return 0;
   return new QoreBigIntNode(c);
}

// *int ReadOnlyFile::readu2(){}
static AbstractQoreNode* ReadOnlyFile_readu2(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 360 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readu2", xsink))
      return 0;

   unsigned short s;
   if (f->readu2(&s, xsink))
      return 0;
   return new QoreBigIntNode(s);
}

// *int ReadOnlyFile::readu2LSB(){}
static AbstractQoreNode* ReadOnlyFile_readu2LSB(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 408 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readu2LSB", xsink))
      return 0;

   unsigned short s;
   if (f->readu2LSB(&s, xsink))
      return 0;

   return new QoreBigIntNode(s);
}

// *int ReadOnlyFile::readu4(){}
static AbstractQoreNode* ReadOnlyFile_readu4(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 384 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readu4", xsink))
      return 0;

   unsigned int i;
   if (f->readu4(&i, xsink))
      return 0;
   return new QoreBigIntNode(i);
}

// *int ReadOnlyFile::readu4LSB(){}
static AbstractQoreNode* ReadOnlyFile_readu4LSB(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 433 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::readu4LSB", xsink))
      return 0;

   unsigned int i;
   if (f->readu4LSB(&i, xsink))
      return 0;

   return new QoreBigIntNode(i);
}

// nothing ReadOnlyFile::setEncoding(*string encoding){}
static AbstractQoreNode* ReadOnlyFile_setEncoding_Ns(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 0));
# 699 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::setEncoding", xsink))
      return 0;

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   f->setEncoding(qe);
   return 0;
}

// nothing ReadOnlyFile::setEventQueue(Qore::Thread::Queue queue){}
static AbstractQoreNode* ReadOnlyFile_setEventQueue_C5Queue(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "ReadOnlyFile::setEventQueue()", "Qore::Thread::Queue", xsink);
   if (*xsink)
      return 0;
# 815 "QC_ReadOnlyFile.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   if (check_terminal_io(self, "ReadOnlyFile::setEventQueue", xsink))
      return 0;

   if (queue->getMax() != -1) {
      xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the File object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
      return 0;
   }

   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setEventQueue(q.release(), xsink);
   return 0;
}

// nothing ReadOnlyFile::setEventQueue(){}
static AbstractQoreNode* ReadOnlyFile_setEventQueue(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 840 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::setEventQueue", xsink))
      return 0;

   f->setEventQueue(0, xsink);
   return 0;
}

// int ReadOnlyFile::setPos(int pos = 0){}
static int64 ReadOnlyFile_setPos_Vi(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
   int64 pos = HARD_QORE_INT(args, 0);
# 733 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::setPos", xsink))
      return 0;

   return f->setPos(pos);
}

// list ReadOnlyFile::stat(){}
static AbstractQoreNode* ReadOnlyFile_stat(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 865 "QC_ReadOnlyFile.qpp"
if (check_terminal_io(self, "ReadOnlyFile::stat", xsink))
      return 0;

   return f->stat(xsink);
}

// hash ReadOnlyFile::statvfs(){}
static AbstractQoreNode* ReadOnlyFile_statvfs(QoreObject* self, File* f, const QoreListNode* args, ExceptionSink* xsink) {
# 913 "QC_ReadOnlyFile.qpp"
#ifdef HAVE_SYS_STATVFS_H
   if (check_terminal_io(self, "ReadOnlyFile::statvfs", xsink))
      return 0;

   return f->statvfs(xsink);
#else
   return missing_method_error("ReadOnlyFile::statvfs", "STATVFS", xsink);
#endif
   return 0;
}

// static binary ReadOnlyFile::readBinaryFile(string path){}
static AbstractQoreNode* static_ReadOnlyFile_readBinaryFile_Vs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
# 1000 "QC_ReadOnlyFile.qpp"
QoreFile qf;

   if (qf.open2(xsink, path->getBuffer()))
      return 0;

   return qf.readBinary(-1, xsink);
}

// static string ReadOnlyFile::readTextFile(string path, *string encoding){}
static AbstractQoreNode* static_ReadOnlyFile_readTextFile_VsNs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* path = HARD_QORE_STRING(args, 0);
   const QoreStringNode* encoding = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 977 "QC_ReadOnlyFile.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

   QoreFile qf;

   if (qf.open2(xsink, path->getBuffer(), O_RDONLY, 0777, qe))
      return 0;

   return qf.read(-1, xsink);
}

DLLLOCAL QoreClass* initReadOnlyFileClass(QoreNamespace& ns) {
   QC_READONLYFILE = new QoreClass("ReadOnlyFile", QDOM_FILESYSTEM);
   CID_READONLYFILE = QC_READONLYFILE->getID();

   // int ReadOnlyFile::close(){}
   QC_READONLYFILE->addMethodExtended3("close", (q_method_int64_t)ReadOnlyFile_close, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // ReadOnlyFile::constructor(string path, *string encoding) {}
   QC_READONLYFILE->setConstructorExtended3(ReadOnlyFile_constructor_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, NULL, "path", stringOrNothingTypeInfo, NULL, "encoding");

   // ReadOnlyFile::copy() {}
   QC_READONLYFILE->setCopy((q_copy_t)ReadOnlyFile_copy);

   // string ReadOnlyFile::getEncoding(){}
   QC_READONLYFILE->addMethodExtended3("getEncoding", (q_method_t)ReadOnlyFile_getEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // *string ReadOnlyFile::getFileName(){}
   QC_READONLYFILE->addMethodExtended3("getFileName", (q_method_t)ReadOnlyFile_getFileName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // int ReadOnlyFile::getPos(){}
   QC_READONLYFILE->addMethodExtended3("getPos", (q_method_int64_t)ReadOnlyFile_getPos, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

   // *string ReadOnlyFile::getchar(){}
   QC_READONLYFILE->addMethodExtended3("getchar", (q_method_t)ReadOnlyFile_getchar, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // hash ReadOnlyFile::hstat(){}
   QC_READONLYFILE->addMethodExtended3("hstat", (q_method_t)ReadOnlyFile_hstat, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   // bool ReadOnlyFile::isDataAvailable(timeout timeout_ms = 0){}
   QC_READONLYFILE->addMethodExtended3("isDataAvailable", (q_method_bool_t)ReadOnlyFile_isDataAvailable_Vt, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

   // bool ReadOnlyFile::isOpen(){}
   QC_READONLYFILE->addMethodExtended3("isOpen", (q_method_bool_t)ReadOnlyFile_isOpen, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // bool ReadOnlyFile::isTty(){}
   QC_READONLYFILE->addMethodExtended3("isTty", (q_method_bool_t)ReadOnlyFile_isTty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // nothing ReadOnlyFile::open(string path, *string encoding){}
   QC_READONLYFILE->addMethodExtended3("open", (q_method_t)ReadOnlyFile_open_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "path", stringOrNothingTypeInfo, NULL, "encoding");

   // *string ReadOnlyFile::read(softint size, timeout timeout_ms = -1){}
   QC_READONLYFILE->addMethodExtended3("read", (q_method_t)ReadOnlyFile_read_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, softBigIntTypeInfo, NULL, "size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

   // *binary ReadOnlyFile::readBinary(softint size, timeout timeout_ms = -1){}
   QC_READONLYFILE->addMethodExtended3("readBinary", (q_method_t)ReadOnlyFile_readBinary_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 2, softBigIntTypeInfo, NULL, "size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

   // *string ReadOnlyFile::readLine(bool incl_eol = True, *string eol){}
   QC_READONLYFILE->addMethodExtended3("readLine", (q_method_t)ReadOnlyFile_readLine_VbNs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, boolTypeInfo, &True, "incl_eol", stringOrNothingTypeInfo, NULL, "eol");

   // *int ReadOnlyFile::readi1(){}
   QC_READONLYFILE->addMethodExtended3("readi1", (q_method_t)ReadOnlyFile_readi1, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readi2(){}
   QC_READONLYFILE->addMethodExtended3("readi2", (q_method_t)ReadOnlyFile_readi2, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readi2LSB(){}
   QC_READONLYFILE->addMethodExtended3("readi2LSB", (q_method_t)ReadOnlyFile_readi2LSB, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readi4(){}
   QC_READONLYFILE->addMethodExtended3("readi4", (q_method_t)ReadOnlyFile_readi4, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readi4LSB(){}
   QC_READONLYFILE->addMethodExtended3("readi4LSB", (q_method_t)ReadOnlyFile_readi4LSB, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readi8(){}
   QC_READONLYFILE->addMethodExtended3("readi8", (q_method_t)ReadOnlyFile_readi8, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readi8LSB(){}
   QC_READONLYFILE->addMethodExtended3("readi8LSB", (q_method_t)ReadOnlyFile_readi8LSB, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readu1(){}
   QC_READONLYFILE->addMethodExtended3("readu1", (q_method_t)ReadOnlyFile_readu1, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readu2(){}
   QC_READONLYFILE->addMethodExtended3("readu2", (q_method_t)ReadOnlyFile_readu2, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readu2LSB(){}
   QC_READONLYFILE->addMethodExtended3("readu2LSB", (q_method_t)ReadOnlyFile_readu2LSB, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readu4(){}
   QC_READONLYFILE->addMethodExtended3("readu4", (q_method_t)ReadOnlyFile_readu4, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // *int ReadOnlyFile::readu4LSB(){}
   QC_READONLYFILE->addMethodExtended3("readu4LSB", (q_method_t)ReadOnlyFile_readu4LSB, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

   // nothing ReadOnlyFile::setEncoding(*string encoding){}
   QC_READONLYFILE->addMethodExtended3("setEncoding", (q_method_t)ReadOnlyFile_setEncoding_Ns, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, NULL, "encoding");

   // nothing ReadOnlyFile::setEventQueue(Qore::Thread::Queue queue){}
   QC_READONLYFILE->addMethodExtended3("setEventQueue", (q_method_t)ReadOnlyFile_setEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

   // nothing ReadOnlyFile::setEventQueue(){}
   QC_READONLYFILE->addMethodExtended3("setEventQueue", (q_method_t)ReadOnlyFile_setEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // int ReadOnlyFile::setPos(int pos = 0){}
   QC_READONLYFILE->addMethodExtended3("setPos", (q_method_int64_t)ReadOnlyFile_setPos_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, zero(), "pos");

   // list ReadOnlyFile::stat(){}
   QC_READONLYFILE->addMethodExtended3("stat", (q_method_t)ReadOnlyFile_stat, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);

   // hash ReadOnlyFile::statvfs(){}
   QC_READONLYFILE->addMethodExtended3("statvfs", (q_method_t)ReadOnlyFile_statvfs, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   // static binary ReadOnlyFile::readBinaryFile(string path){}
   QC_READONLYFILE->addStaticMethodExtended3("readBinaryFile", (q_func_t)static_ReadOnlyFile_readBinaryFile_Vs, false, QC_NO_FLAGS, QDOM_FILESYSTEM, binaryTypeInfo, 1, stringTypeInfo, NULL, "path");

   // static string ReadOnlyFile::readTextFile(string path, *string encoding){}
   QC_READONLYFILE->addStaticMethodExtended3("readTextFile", (q_func_t)static_ReadOnlyFile_readTextFile_VsNs, false, QC_NO_FLAGS, QDOM_FILESYSTEM, stringTypeInfo, 2, stringTypeInfo, NULL, "path", stringOrNothingTypeInfo, NULL, "encoding");
# 29 "QC_ReadOnlyFile.qpp"
# 36 "QC_ReadOnlyFile.qpp"
# 39 "QC_ReadOnlyFile.qpp"
# 42 "QC_ReadOnlyFile.qpp"
# 45 "QC_ReadOnlyFile.qpp"
#ifndef S_IFLNK
#define S_IFLNK 0
#endif
# 52 "QC_ReadOnlyFile.qpp"
#ifndef S_IFSOCK
#define S_IFSOCK 0
#endif
# 59 "QC_ReadOnlyFile.qpp"
#ifndef S_IFWHT
#define S_IFWHT S_IFREG
#endif
# 66 "QC_ReadOnlyFile.qpp"
# 69 "QC_ReadOnlyFile.qpp"
# 72 "QC_ReadOnlyFile.qpp"
# 75 "QC_ReadOnlyFile.qpp"
# 78 "QC_ReadOnlyFile.qpp"
#ifndef S_IRWXG
#define S_IRWXG 0
#endif
# 85 "QC_ReadOnlyFile.qpp"
#ifndef S_IRGRP
#define S_IRGRP 0
#endif
# 92 "QC_ReadOnlyFile.qpp"
#ifndef S_IWGRP
#define S_IWGRP 0
#endif
# 99 "QC_ReadOnlyFile.qpp"
#ifndef S_IXGRP
#define S_IXGRP 0
#endif
# 106 "QC_ReadOnlyFile.qpp"
#ifndef S_IRWXO
#define S_IRWXO 0
#endif
# 113 "QC_ReadOnlyFile.qpp"
#ifndef S_IROTH
#define S_IROTH 0
#endif
# 120 "QC_ReadOnlyFile.qpp"
#ifndef S_IWOTH
#define S_IWOTH 0
#endif
# 127 "QC_ReadOnlyFile.qpp"
#ifndef S_IXOTH
#define S_IXOTH 0
#endif
# 134 "QC_ReadOnlyFile.qpp"
#ifndef S_ISUID
#define S_ISUID 0
#endif
# 141 "QC_ReadOnlyFile.qpp"
#ifndef S_ISGID
#define S_ISGID 0
#endif
# 148 "QC_ReadOnlyFile.qpp"
#ifndef S_ISVTX
#define S_ISVTX 0
#endif
   ns.addConstant("S_IFBLK", new QoreBigIntNode(S_IFBLK));
   ns.addConstant("S_IFCHR", new QoreBigIntNode(S_IFCHR));
   ns.addConstant("S_IFDIR", new QoreBigIntNode(S_IFDIR));
   ns.addConstant("S_IFLNK", new QoreBigIntNode(S_IFLNK));
   ns.addConstant("S_IFMT", new QoreBigIntNode(S_IFMT));
   ns.addConstant("S_IFREG", new QoreBigIntNode(S_IFREG));
   ns.addConstant("S_IFSOCK", new QoreBigIntNode(S_IFSOCK));
   ns.addConstant("S_IFWHT", new QoreBigIntNode(S_IFWHT));
   ns.addConstant("S_IRGRP", new QoreBigIntNode(S_IRGRP));
   ns.addConstant("S_IROTH", new QoreBigIntNode(S_IROTH));
   ns.addConstant("S_IRUSR", new QoreBigIntNode(S_IRUSR));
   ns.addConstant("S_IRWXG", new QoreBigIntNode(S_IRWXG));
   ns.addConstant("S_IRWXO", new QoreBigIntNode(S_IRWXO));
   ns.addConstant("S_IRWXU", new QoreBigIntNode(S_IRWXU));
   ns.addConstant("S_ISGID", new QoreBigIntNode(S_ISGID));
   ns.addConstant("S_ISUID", new QoreBigIntNode(S_ISUID));
   ns.addConstant("S_ISVTX", new QoreBigIntNode(S_ISVTX));
   ns.addConstant("S_IWGRP", new QoreBigIntNode(S_IWGRP));
   ns.addConstant("S_IWOTH", new QoreBigIntNode(S_IWOTH));
   ns.addConstant("S_IWUSR", new QoreBigIntNode(S_IWUSR));
   ns.addConstant("S_IXGRP", new QoreBigIntNode(S_IXGRP));
   ns.addConstant("S_IXOTH", new QoreBigIntNode(S_IXOTH));
   ns.addConstant("S_IXUSR", new QoreBigIntNode(S_IXUSR));

   return QC_READONLYFILE;
}
