/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
 QC_SQLStatement.qpp
 
 Qore Programming Language
 
 Copyright 2003 - 2013 David Nichols
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <qore/Qore.h>
#include <qore/intern/QC_SQLStatement.h>
#include <qore/intern/QC_Datasource.h>
#include <qore/intern/QC_DatasourcePool.h>
/* Qore class Qore::SQL::SQLStatement */

qore_classid_t CID_SQLSTATEMENT;
QoreClass* QC_SQLSTATEMENT;

// bool SQLStatement::active(){}
static bool SQLStatement_active(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 724 "QC_SQLStatement.qpp"
return stmt->active();
}

// int SQLStatement::affectedRows(){}
static int64 SQLStatement_affectedRows(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 454 "QC_SQLStatement.qpp"
return stmt->affectedRows(xsink);
}

// nothing SQLStatement::beginTransaction(){}
static AbstractQoreNode* SQLStatement_beginTransaction(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 558 "QC_SQLStatement.qpp"
stmt->beginTransaction(xsink);
   return 0;
}

// nothing SQLStatement::bind(...){}
static AbstractQoreNode* SQLStatement_bind_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 226 "QC_SQLStatement.qpp"
stmt->bind(*args, xsink);
   return 0;
}

// nothing SQLStatement::bindArgs(softlist vargs){}
static AbstractQoreNode* SQLStatement_bindArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* vargs = HARD_QORE_LIST(args, 0);
# 257 "QC_SQLStatement.qpp"
stmt->bind(*vargs, xsink);
   return 0;
}

// nothing SQLStatement::bindPlaceholders(...){}
static AbstractQoreNode* SQLStatement_bindPlaceholders_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 286 "QC_SQLStatement.qpp"
stmt->bindPlaceholders(*args, xsink);
   return 0;
}

// nothing SQLStatement::bindPlaceholdersArgs(softlist vargs){}
static AbstractQoreNode* SQLStatement_bindPlaceholdersArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* vargs = HARD_QORE_LIST(args, 0);
# 316 "QC_SQLStatement.qpp"
stmt->bindPlaceholders(*vargs, xsink);
   return 0;
}

// nothing SQLStatement::bindValues(...){}
static AbstractQoreNode* SQLStatement_bindValues_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 346 "QC_SQLStatement.qpp"
stmt->bindValues(*args, xsink);
   return 0;
}

// nothing SQLStatement::bindValuesArgs(softlist vargs){}
static AbstractQoreNode* SQLStatement_bindValuesArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* vargs = HARD_QORE_LIST(args, 0);
# 375 "QC_SQLStatement.qpp"
stmt->bindValues(*vargs, xsink);
   return 0;
}

// nothing SQLStatement::close(){}
static AbstractQoreNode* SQLStatement_close(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 521 "QC_SQLStatement.qpp"
stmt->close(xsink);
   return 0;
}

// nothing SQLStatement::commit(){}
static AbstractQoreNode* SQLStatement_commit(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 534 "QC_SQLStatement.qpp"
stmt->commit(xsink);
   return 0;
}

// SQLStatement::constructor(Datasource ds) {}
static void SQLStatement_constructor_C10Datasource(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(ds, ManagedDatasource, args, 0, CID_DATASOURCE, "SQLStatement::constructor()", "Datasource", xsink);
   if (*xsink)
      return;
# 100 "QC_SQLStatement.qpp"
ReferenceHolder<ManagedDatasource> ds_holder(ds, xsink);

   if (!ds->getDriver()->hasStatementAPI()) {
      xsink->raiseException("SQLSTATEMENT-ERROR", "DBI driver '%s' does not support the prepared statement API", ds->getDriver()->getName());
      return;
   }

   ReferenceHolder<QoreSQLStatement> ss(new QoreSQLStatement, xsink);
   // FIXME: reuse reference from this call
   ss->init(ds->getReferencedHelper(*ss));
   self->setPrivate(CID_SQLSTATEMENT, ss.release());
}

// SQLStatement::constructor(DatasourcePool dsp) {}
static void SQLStatement_constructor_C14DatasourcePool(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(dsp, DatasourcePool, args, 0, CID_DATASOURCEPOOL, "SQLStatement::constructor()", "DatasourcePool", xsink);
   if (*xsink)
      return;
# 128 "QC_SQLStatement.qpp"
ReferenceHolder<DatasourcePool> dsp_holder(dsp, xsink);

   if (!dsp->getDriver()->hasStatementAPI()) {
      xsink->raiseException("SQLSTATEMENT-ERROR", "DBI driver '%s' does not support the prepared statement API", dsp->getDriver()->getName());
      return;
   }

   ReferenceHolder<QoreSQLStatement> ss(new QoreSQLStatement, xsink);
   // FIXME: reuse reference from this call
   ss->init(dsp->getReferencedHelper(*ss));
   self->setPrivate(CID_SQLSTATEMENT, ss.release());
}

// SQLStatement::copy() {}
static void SQLStatement_copy(QoreObject* self, QoreObject* old, QoreSQLStatement* stmt, ExceptionSink* xsink) {
# 158 "QC_SQLStatement.qpp"
xsink->raiseException("SQLSTATEMENT-COPY-ERROR", "SQLStatement objects cannot be copied");
   //self->setPrivate(CID_SQLSTATEMENT, new SQLStatement(*old));
}

// nothing SQLStatement::define(){}
static AbstractQoreNode* SQLStatement_define(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 510 "QC_SQLStatement.qpp"
stmt->define(xsink);
   return 0;
}

// hash SQLStatement::describe(){}
static AbstractQoreNode* SQLStatement_describe(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 699 "QC_SQLStatement.qpp"
return stmt->describe(xsink);
}

// SQLStatement::destructor() {}
static void SQLStatement_destructor(QoreObject* self, QoreSQLStatement* stmt, ExceptionSink* xsink) {
# 149 "QC_SQLStatement.qpp"
stmt->close(xsink);
   stmt->deref(xsink);
}

// nothing SQLStatement::exec(...){}
static AbstractQoreNode* SQLStatement_exec_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 406 "QC_SQLStatement.qpp"
stmt->exec(args, xsink);
   return 0;
}

// nothing SQLStatement::execArgs(softlist vargs){}
static AbstractQoreNode* SQLStatement_execArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* vargs = HARD_QORE_LIST(args, 0);
# 438 "QC_SQLStatement.qpp"
stmt->exec(vargs, xsink);
   return 0;
}

// hash SQLStatement::fetchColumns(softint rows = -1){}
static AbstractQoreNode* SQLStatement_fetchColumns_vi(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   int64 rows = HARD_QORE_INT(args, 0);
# 679 "QC_SQLStatement.qpp"
return stmt->fetchColumns((int)rows, xsink);
}

// *hash SQLStatement::fetchRow(){}
static AbstractQoreNode* SQLStatement_fetchRow(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 611 "QC_SQLStatement.qpp"
return stmt->fetchRow(xsink);
}

// list SQLStatement::fetchRows(softint rows = -1){}
static AbstractQoreNode* SQLStatement_fetchRows_vi(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   int64 rows = HARD_QORE_INT(args, 0);
# 657 "QC_SQLStatement.qpp"
return stmt->fetchRows((int)rows, xsink);
}

// hash SQLStatement::getOutput(){}
static AbstractQoreNode* SQLStatement_getOutput(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 470 "QC_SQLStatement.qpp"
return stmt->getOutput(xsink);
}

// hash SQLStatement::getOutputRows(){}
static AbstractQoreNode* SQLStatement_getOutputRows(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 486 "QC_SQLStatement.qpp"
return stmt->getOutputRows(xsink);
}

// *string SQLStatement::getSQL(){}
static AbstractQoreNode* SQLStatement_getSQL(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 711 "QC_SQLStatement.qpp"
return stmt->getSQL(xsink);
}

// *hash SQLStatement::getValue(){}
static AbstractQoreNode* SQLStatement_getValue(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 635 "QC_SQLStatement.qpp"
return stmt->fetchRow(xsink);
}

// any SQLStatement::memberGate(string key){}
static AbstractQoreNode* SQLStatement_memberGate_Vs(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 753 "QC_SQLStatement.qpp"
ReferenceHolder<QoreHashNode> h(stmt->fetchRow(xsink), xsink);
   if (*xsink)
      return 0;
   if (!h) {
      xsink->raiseException("SQLSTATEMENT-ITERATION-ERROR", "Cannot dereference statement iteration context to get current '%s' column value; make sure and call SQLStatement::next() before trying to dereference the current row being iterated", key->getBuffer());
      return 0;
   }

   bool exists = false;
   const AbstractQoreNode* rv = h->getKeyValueExistence(key, exists, xsink);
   if (*xsink)
      return 0;

   if (!exists) {
      xsink->raiseException("SQLSTATEMENT-COLUMN-ERROR", "The given column name ('%s') does not exist in the current row data", key->getBuffer());
      return 0;
   }

   return rv ? rv->refSelf() : 0;
}

// bool SQLStatement::next(){}
static bool SQLStatement_next(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 578 "QC_SQLStatement.qpp"
return stmt->next(xsink);
}

// nothing SQLStatement::prepare(string sql, ...){}
static AbstractQoreNode* SQLStatement_prepare_VsVV(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* sql = HARD_QORE_STRING(args, 0);
# 175 "QC_SQLStatement.qpp"
ReferenceHolder<QoreListNode> l(xsink);

   if (num_args(args) > 1)
      l = args->copyListFrom(1);

   stmt->prepare(*sql, *l, xsink);
   return 0;
}

// nothing SQLStatement::prepareRaw(string sql){}
static AbstractQoreNode* SQLStatement_prepareRaw_Vs(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* sql = HARD_QORE_STRING(args, 0);
# 196 "QC_SQLStatement.qpp"
stmt->prepareRaw(*sql, xsink);
   return 0;
}

// nothing SQLStatement::rollback(){}
static AbstractQoreNode* SQLStatement_rollback(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 547 "QC_SQLStatement.qpp"
stmt->rollback(xsink);
   return 0;
}

// bool SQLStatement::valid(){}
static bool SQLStatement_valid(QoreObject* self, QoreSQLStatement* stmt, const QoreListNode* args, ExceptionSink* xsink) {
# 591 "QC_SQLStatement.qpp"
return stmt->valid();
}

DLLLOCAL QoreClass* initSQLStatementClass(QoreNamespace& ns) {
   QC_SQLSTATEMENT = new QoreClass("SQLStatement", QDOM_DATABASE);
   CID_SQLSTATEMENT = QC_SQLSTATEMENT->getID();

   // set parent class
   assert(QC_ABSTRACTITERATOR);
   QC_SQLSTATEMENT->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

   // bool SQLStatement::active(){}
   QC_SQLSTATEMENT->addMethodExtended3("active", (q_method_bool_t)SQLStatement_active, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   // int SQLStatement::affectedRows(){}
   QC_SQLSTATEMENT->addMethodExtended3("affectedRows", (q_method_int64_t)SQLStatement_affectedRows, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // nothing SQLStatement::beginTransaction(){}
   QC_SQLSTATEMENT->addMethodExtended3("beginTransaction", (q_method_t)SQLStatement_beginTransaction, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing SQLStatement::bind(...){}
   QC_SQLSTATEMENT->addMethodExtended3("bind", (q_method_t)SQLStatement_bind_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing SQLStatement::bindArgs(softlist vargs){}
   QC_SQLSTATEMENT->addMethodExtended3("bindArgs", (q_method_t)SQLStatement_bindArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

   // nothing SQLStatement::bindPlaceholders(...){}
   QC_SQLSTATEMENT->addMethodExtended3("bindPlaceholders", (q_method_t)SQLStatement_bindPlaceholders_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing SQLStatement::bindPlaceholdersArgs(softlist vargs){}
   QC_SQLSTATEMENT->addMethodExtended3("bindPlaceholdersArgs", (q_method_t)SQLStatement_bindPlaceholdersArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

   // nothing SQLStatement::bindValues(...){}
   QC_SQLSTATEMENT->addMethodExtended3("bindValues", (q_method_t)SQLStatement_bindValues_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing SQLStatement::bindValuesArgs(softlist vargs){}
   QC_SQLSTATEMENT->addMethodExtended3("bindValuesArgs", (q_method_t)SQLStatement_bindValuesArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

   // nothing SQLStatement::close(){}
   QC_SQLSTATEMENT->addMethodExtended3("close", (q_method_t)SQLStatement_close, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing SQLStatement::commit(){}
   QC_SQLSTATEMENT->addMethodExtended3("commit", (q_method_t)SQLStatement_commit, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // SQLStatement::constructor(Datasource ds) {}
   QC_SQLSTATEMENT->setConstructorExtended3(SQLStatement_constructor_C10Datasource, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, QC_DATASOURCE->getTypeInfo(), NULL, "ds");

   // SQLStatement::constructor(DatasourcePool dsp) {}
   QC_SQLSTATEMENT->setConstructorExtended3(SQLStatement_constructor_C14DatasourcePool, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, QC_DATASOURCEPOOL->getTypeInfo(), NULL, "dsp");

   // SQLStatement::copy() {}
   QC_SQLSTATEMENT->setCopy((q_copy_t)SQLStatement_copy);

   // nothing SQLStatement::define(){}
   QC_SQLSTATEMENT->addMethodExtended3("define", (q_method_t)SQLStatement_define, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // hash SQLStatement::describe(){}
   QC_SQLSTATEMENT->addMethodExtended3("describe", (q_method_t)SQLStatement_describe, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   // SQLStatement::destructor() {}
   QC_SQLSTATEMENT->setDestructor((q_destructor_t)SQLStatement_destructor);

   // nothing SQLStatement::exec(...){}
   QC_SQLSTATEMENT->addMethodExtended3("exec", (q_method_t)SQLStatement_exec_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

   // nothing SQLStatement::execArgs(softlist vargs){}
   QC_SQLSTATEMENT->addMethodExtended3("execArgs", (q_method_t)SQLStatement_execArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

   // hash SQLStatement::fetchColumns(softint rows = -1){}
   QC_SQLSTATEMENT->addMethodExtended3("fetchColumns", (q_method_t)SQLStatement_fetchColumns_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "rows");

   // *hash SQLStatement::fetchRow(){}
   QC_SQLSTATEMENT->addMethodExtended3("fetchRow", (q_method_t)SQLStatement_fetchRow, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo);

   // list SQLStatement::fetchRows(softint rows = -1){}
   QC_SQLSTATEMENT->addMethodExtended3("fetchRows", (q_method_t)SQLStatement_fetchRows_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "rows");

   // hash SQLStatement::getOutput(){}
   QC_SQLSTATEMENT->addMethodExtended3("getOutput", (q_method_t)SQLStatement_getOutput, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   // hash SQLStatement::getOutputRows(){}
   QC_SQLSTATEMENT->addMethodExtended3("getOutputRows", (q_method_t)SQLStatement_getOutputRows, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

   // *string SQLStatement::getSQL(){}
   QC_SQLSTATEMENT->addMethodExtended3("getSQL", (q_method_t)SQLStatement_getSQL, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

   // *hash SQLStatement::getValue(){}
   QC_SQLSTATEMENT->addMethodExtended3("getValue", (q_method_t)SQLStatement_getValue, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo);

   // any SQLStatement::memberGate(string key){}
   QC_SQLSTATEMENT->addMethodExtended3("memberGate", (q_method_t)SQLStatement_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

   // bool SQLStatement::next(){}
   QC_SQLSTATEMENT->addMethodExtended3("next", (q_method_bool_t)SQLStatement_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing SQLStatement::prepare(string sql, ...){}
   QC_SQLSTATEMENT->addMethodExtended3("prepare", (q_method_t)SQLStatement_prepare_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // nothing SQLStatement::prepareRaw(string sql){}
   QC_SQLSTATEMENT->addMethodExtended3("prepareRaw", (q_method_t)SQLStatement_prepareRaw_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "sql");

   // nothing SQLStatement::rollback(){}
   QC_SQLSTATEMENT->addMethodExtended3("rollback", (q_method_t)SQLStatement_rollback, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool SQLStatement::valid(){}
   QC_SQLSTATEMENT->addMethodExtended3("valid", (q_method_bool_t)SQLStatement_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_SQLSTATEMENT;
}
