/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_SSLPrivateKey.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_SSLPrivateKey.h>

#include <openssl/ssl.h>
#include <openssl/pem.h>
/* Qore class Qore::SSLPrivateKey */

qore_classid_t CID_SSLPRIVATEKEY;
QoreClass* QC_SSLPRIVATEKEY;

// SSLPrivateKey::constructor(string pem, *string pass) {}
static void SSLPrivateKey_constructor_VsNs(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* pem = HARD_QORE_STRING(args, 0);
   const QoreStringNode* pass = reinterpret_cast<const QoreStringNode*>(get_param(args, 1));
# 49 "QC_SSLPrivateKey.qpp"
SimpleRefHolder<QoreSSLPrivateKey> qpk(new QoreSSLPrivateKey(pem, pass ? pass->getBuffer() : 0, xsink));

   if (!*xsink)
      self->setPrivate(CID_SSLPRIVATEKEY, qpk.release());
}

// SSLPrivateKey::constructor(binary der) {}
static void SSLPrivateKey_constructor_Vy(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const BinaryNode* der = HARD_QORE_BINARY(args, 0);
# 66 "QC_SSLPrivateKey.qpp"
SimpleRefHolder<QoreSSLPrivateKey> qc(new QoreSSLPrivateKey(der, xsink));
   if (!*xsink)
      self->setPrivate(CID_SSLPRIVATEKEY, qc.release());
}

// SSLPrivateKey::copy() {}
static void SSLPrivateKey_copy(QoreObject* self, QoreObject* old, QoreSSLPrivateKey* pk, ExceptionSink* xsink) {
# 76 "QC_SSLPrivateKey.qpp"
xsink->raiseException("SSLPRIVATEKEY-COPY-ERROR", "SSLPrivateKey objects cannot be copied");
}

// int SSLPrivateKey::getBitLength(){}
static int64 SSLPrivateKey_getBitLength(QoreObject* self, QoreSSLPrivateKey* pk, const QoreListNode* args, ExceptionSink* xsink) {
# 124 "QC_SSLPrivateKey.qpp"
return pk->getBitLength();
}

// hash SSLPrivateKey::getInfo(){}
static AbstractQoreNode* SSLPrivateKey_getInfo(QoreObject* self, QoreSSLPrivateKey* pk, const QoreListNode* args, ExceptionSink* xsink) {
# 91 "QC_SSLPrivateKey.qpp"
return pk->getInfo();
}

// string SSLPrivateKey::getPEM(){}
static AbstractQoreNode* SSLPrivateKey_getPEM(QoreObject* self, QoreSSLPrivateKey* pk, const QoreListNode* args, ExceptionSink* xsink) {
# 138 "QC_SSLPrivateKey.qpp"
return pk->getPEM(xsink);
}

// string SSLPrivateKey::getType(){}
static AbstractQoreNode* SSLPrivateKey_getType(QoreObject* self, QoreSSLPrivateKey* pk, const QoreListNode* args, ExceptionSink* xsink) {
# 103 "QC_SSLPrivateKey.qpp"
return new QoreStringNode(pk->getType());
}

// int SSLPrivateKey::getVersion(){}
static int64 SSLPrivateKey_getVersion(QoreObject* self, QoreSSLPrivateKey* pk, const QoreListNode* args, ExceptionSink* xsink) {
# 112 "QC_SSLPrivateKey.qpp"
return pk->getVersion();
}

DLLLOCAL QoreClass* initSSLPrivateKeyClass(QoreNamespace& ns) {
   QC_SSLPRIVATEKEY = new QoreClass("SSLPrivateKey", QDOM_DEFAULT);
   CID_SSLPRIVATEKEY = QC_SSLPRIVATEKEY->getID();

   // SSLPrivateKey::constructor(string pem, *string pass) {}
   QC_SSLPRIVATEKEY->setConstructorExtended3(SSLPrivateKey_constructor_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, NULL, "pem", stringOrNothingTypeInfo, NULL, "pass");

   // SSLPrivateKey::constructor(binary der) {}
   QC_SSLPRIVATEKEY->setConstructorExtended3(SSLPrivateKey_constructor_Vy, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, binaryTypeInfo, NULL, "der");

   // SSLPrivateKey::copy() {}
   QC_SSLPRIVATEKEY->setCopy((q_copy_t)SSLPrivateKey_copy);

   // int SSLPrivateKey::getBitLength(){}
   QC_SSLPRIVATEKEY->addMethodExtended3("getBitLength", (q_method_int64_t)SSLPrivateKey_getBitLength, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // hash SSLPrivateKey::getInfo(){}
   QC_SSLPRIVATEKEY->addMethodExtended3("getInfo", (q_method_t)SSLPrivateKey_getInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

   // string SSLPrivateKey::getPEM(){}
   QC_SSLPRIVATEKEY->addMethodExtended3("getPEM", (q_method_t)SSLPrivateKey_getPEM, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

   // string SSLPrivateKey::getType(){}
   QC_SSLPRIVATEKEY->addMethodExtended3("getType", (q_method_t)SSLPrivateKey_getType, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

   // int SSLPrivateKey::getVersion(){}
   QC_SSLPRIVATEKEY->addMethodExtended3("getVersion", (q_method_int64_t)SSLPrivateKey_getVersion, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   return QC_SSLPRIVATEKEY;
}
