/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Sequence.qpp
  
  Qore Programming Language
  
  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_Sequence.h>
/* Qore class Qore::Thread::Sequence */

qore_classid_t CID_SEQUENCE;
QoreClass* QC_SEQUENCE;

// Sequence::constructor() {}
static void Sequence_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 40 "QC_Sequence.qpp"
self->setPrivate(CID_SEQUENCE, new QoreSequence);
}

// Sequence::constructor(softint start) {}
static void Sequence_constructor_vi(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
# 51 "QC_Sequence.qpp"
self->setPrivate(CID_SEQUENCE, new QoreSequence((int)start));
}

// Sequence::copy() {}
static void Sequence_copy(QoreObject* self, QoreObject* old, QoreSequence* s, ExceptionSink* xsink) {
# 62 "QC_Sequence.qpp"
self->setPrivate(CID_SEQUENCE, new QoreSequence(s->getCurrent()));
}

// int Sequence::getCurrent(){}
static int64 Sequence_getCurrent(QoreObject* self, QoreSequence* s, const QoreListNode* args, ExceptionSink* xsink) {
# 84 "QC_Sequence.qpp"
return s->getCurrent();
}

// int Sequence::next(){}
static int64 Sequence_next(QoreObject* self, QoreSequence* s, const QoreListNode* args, ExceptionSink* xsink) {
# 73 "QC_Sequence.qpp"
return s->next();
}

DLLLOCAL QoreClass* initSequenceClass(QoreNamespace& ns) {
   QC_SEQUENCE = new QoreClass("Sequence", QDOM_DEFAULT);
   CID_SEQUENCE = QC_SEQUENCE->getID();

   // Sequence::constructor() {}
   QC_SEQUENCE->setConstructorExtended3(Sequence_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // Sequence::constructor(softint start) {}
   QC_SEQUENCE->setConstructorExtended3(Sequence_constructor_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softBigIntTypeInfo, NULL, "start");

   // Sequence::copy() {}
   QC_SEQUENCE->setCopy((q_copy_t)Sequence_copy);

   // int Sequence::getCurrent(){}
   QC_SEQUENCE->addMethodExtended3("getCurrent", (q_method_int64_t)Sequence_getCurrent, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

   // int Sequence::next(){}
   QC_SEQUENCE->addMethodExtended3("next", (q_method_int64_t)Sequence_next, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   return QC_SEQUENCE;
}
