/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file SingleValueIterator.qpp SingleValueIterator class definition */
/*
  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/SingleValueIterator.h>
/* Qore class Qore::SingleValueIterator */

qore_classid_t CID_SINGLEVALUEITERATOR;
QoreClass* QC_SINGLEVALUEITERATOR;

// SingleValueIterator::constructor(any v) {}
static void SingleValueIterator_constructor_Va(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* v = get_param(args, 0);
# 59 "QC_SingleValueIterator.qpp"
self->setPrivate(CID_SINGLEVALUEITERATOR, new SingleValueIterator(v));
}

// SingleValueIterator::copy() {}
static void SingleValueIterator_copy(QoreObject* self, QoreObject* old, SingleValueIterator* i, ExceptionSink* xsink) {
# 69 "QC_SingleValueIterator.qpp"
self->setPrivate(CID_SINGLEVALUEITERATOR, new SingleValueIterator(*i));
}

// any SingleValueIterator::getValue(){}
static AbstractQoreNode* SingleValueIterator_getValue(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 105 "QC_SingleValueIterator.qpp"
return i->checkValid(xsink) ? 0 : i->getValue();
}

// bool SingleValueIterator::next(){}
static bool SingleValueIterator_next(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 87 "QC_SingleValueIterator.qpp"
if (i->check(xsink))
      return false;
   return i->next();
}

// nothing SingleValueIterator::reset(){}
static AbstractQoreNode* SingleValueIterator_reset(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 132 "QC_SingleValueIterator.qpp"
if (i->check(xsink))
       return 0;
   i->reset();
   return 0;
}

// bool SingleValueIterator::valid(){}
static bool SingleValueIterator_valid(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, ExceptionSink* xsink) {
# 118 "QC_SingleValueIterator.qpp"
return i->valid();
}

DLLLOCAL QoreClass* initSingleValueIteratorClass(QoreNamespace& ns) {
   QC_SINGLEVALUEITERATOR = new QoreClass("SingleValueIterator", QDOM_DEFAULT);
   CID_SINGLEVALUEITERATOR = QC_SINGLEVALUEITERATOR->getID();

   // set parent class
   assert(QC_ABSTRACTITERATOR);
   QC_SINGLEVALUEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

   // SingleValueIterator::constructor(any v) {}
   QC_SINGLEVALUEITERATOR->setConstructorExtended3(SingleValueIterator_constructor_Va, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, anyTypeInfo, NULL, "v");

   // SingleValueIterator::copy() {}
   QC_SINGLEVALUEITERATOR->setCopy((q_copy_t)SingleValueIterator_copy);

   // any SingleValueIterator::getValue(){}
   QC_SINGLEVALUEITERATOR->addMethodExtended3("getValue", (q_method_t)SingleValueIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo);

   // bool SingleValueIterator::next(){}
   QC_SINGLEVALUEITERATOR->addMethodExtended3("next", (q_method_bool_t)SingleValueIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

   // nothing SingleValueIterator::reset(){}
   QC_SINGLEVALUEITERATOR->addMethodExtended3("reset", (q_method_t)SingleValueIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

   // bool SingleValueIterator::valid(){}
   QC_SINGLEVALUEITERATOR->addMethodExtended3("valid", (q_method_bool_t)SingleValueIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

   return QC_SINGLEVALUEITERATOR;
}
