/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_TermIOS.qpp
  
  Qore Programming Language
  
  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/QC_TermIOS.h>
/* Qore class Qore::TermIOS */

qore_classid_t CID_TERMIOS;
QoreClass* QC_TERMIOS;

// TermIOS::constructor() {}
static void TermIOS_constructor(QoreObject* self, const QoreListNode* args, ExceptionSink* xsink) {
# 578 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   self->setPrivate(CID_TERMIOS, new QoreTermIOS);
#else
   missing_method_error("TermIOS::constructor", "TERMIOS", xsink);
#endif
}

// TermIOS::copy() {}
static void TermIOS_copy(QoreObject* self, QoreObject* old, QoreTermIOS*s, ExceptionSink* xsink) {
# 595 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   self->setPrivate(CID_TERMIOS, new QoreTermIOS(*s));
#else
   missing_method_error("TermIOS::copy", "TERMIOS", xsink);
#endif
}

// int TermIOS::getCC(softint cc){}
static int64 TermIOS_getCC_vi(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   int64 cc = HARD_QORE_INT(args, 0);
# 797 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   return s->get_cc(cc, xsink);
#else
   missing_method_error("TermIOS::getCC", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

// int TermIOS::getCFlag(){}
static int64 TermIOS_getCFlag(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
# 634 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   return s->get_cflag();
#else
   missing_method_error("TermIOS::getCFlag", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

// int TermIOS::getIFlag(){}
static int64 TermIOS_getIFlag(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
# 674 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   return s->get_iflag();
#else
   missing_method_error("TermIOS::getIFlag", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

// int TermIOS::getLFlag(){}
static int64 TermIOS_getLFlag(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
# 614 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   return s->get_lflag();
#else
   missing_method_error("TermIOS::getLFlag", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

// int TermIOS::getOFlag(){}
static int64 TermIOS_getOFlag(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
# 654 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   return s->get_oflag();
#else
   missing_method_error("TermIOS::getOFlag", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

// bool TermIOS::isEqual(Termios termios){}
static bool TermIOS_isEqual_C7Termios(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   HARD_QORE_OBJ_DATA(termios, QoreTermIOS, args, 0, CID_TERMIOS, "TermIOS::isEqual()", "Termios", xsink);
   if (*xsink)
      return 0;
# 844 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   ReferenceHolder<QoreTermIOS> holder(termios, xsink);
   return s->is_equal(termios);
#else
   termios = 0;
   missing_method_error("TermIOS::isEqual", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

// nothing TermIOS::setCC(softint offset, softint value){}
static AbstractQoreNode* TermIOS_setCC_vivi(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   int64 offset = HARD_QORE_INT(args, 0);
   int64 value = HARD_QORE_INT(args, 1);
# 824 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   s->set_cc(offset, (cc_t)value, xsink);
   return 0;
#else
   return missing_method_error("TermIOS::setCC", "TERMIOS", xsink);
#endif
   return 0;
}

// nothing TermIOS::setCFlag(softint flag){}
static AbstractQoreNode* TermIOS_setCFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   int64 flag = HARD_QORE_INT(args, 0);
# 724 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   s->set_cflag((int)flag);
   return 0;
#else
   return missing_method_error("TermIOS::setCFlag", "TERMIOS", xsink);
#endif
   return 0;
}

// nothing TermIOS::setIFlag(softint flag){}
static AbstractQoreNode* TermIOS_setIFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   int64 flag = HARD_QORE_INT(args, 0);
# 770 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   s->set_iflag((int)flag);
   return 0;
#else
   return missing_method_error("TermIOS::setIFlag", "TERMIOS", xsink);
#endif
   return 0;
}

// nothing TermIOS::setLFlag(softint flag){}
static AbstractQoreNode* TermIOS_setLFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   int64 flag = HARD_QORE_INT(args, 0);
# 701 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   s->set_lflag((int)flag);
   return 0;
#else
   return missing_method_error("TermIOS::setLFlag", "TERMIOS", xsink);
#endif
   return 0;
}

// nothing TermIOS::setOFlag(softint flag){}
static AbstractQoreNode* TermIOS_setOFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreListNode* args, ExceptionSink* xsink) {
   int64 flag = HARD_QORE_INT(args, 0);
# 747 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   s->set_oflag((int)flag);
   return 0;
#else
   return missing_method_error("TermIOS::setOFlag", "TERMIOS", xsink);
#endif
   return 0;
}

// static hash TermIOS::getWindowSize(){}
static AbstractQoreNode* static_TermIOS_getWindowSize(const QoreListNode* args, ExceptionSink* xsink) {
# 867 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   int rows, columns;

   if (QoreTermIOS::getWindowSize(rows, columns, xsink))
      return 0;

   QoreHashNode *rv = new QoreHashNode;
   rv->setKeyValue("rows", new QoreBigIntNode(rows), xsink);
   rv->setKeyValue("columns", new QoreBigIntNode(columns), xsink);
   return rv;
#else
   missing_method_error("TermIOS::getWindowSize", "TERMIOS", xsink);
   return 0;
#endif
   return 0;
}

DLLLOCAL QoreClass* initTermIOSClass(QoreNamespace& ns) {
   QC_TERMIOS = new QoreClass("TermIOS", QDOM_TERMINAL_IO);
   CID_TERMIOS = QC_TERMIOS->getID();

   // TermIOS::constructor() {}
   QC_TERMIOS->setConstructorExtended3(TermIOS_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

   // TermIOS::copy() {}
   QC_TERMIOS->setCopy((q_copy_t)TermIOS_copy);

   // int TermIOS::getCC(softint cc){}
   QC_TERMIOS->addMethodExtended3("getCC", (q_method_int64_t)TermIOS_getCC_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "cc");

   // int TermIOS::getCFlag(){}
   QC_TERMIOS->addMethodExtended3("getCFlag", (q_method_int64_t)TermIOS_getCFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // int TermIOS::getIFlag(){}
   QC_TERMIOS->addMethodExtended3("getIFlag", (q_method_int64_t)TermIOS_getIFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // int TermIOS::getLFlag(){}
   QC_TERMIOS->addMethodExtended3("getLFlag", (q_method_int64_t)TermIOS_getLFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // int TermIOS::getOFlag(){}
   QC_TERMIOS->addMethodExtended3("getOFlag", (q_method_int64_t)TermIOS_getOFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

   // bool TermIOS::isEqual(Termios termios){}
   QC_TERMIOS->addMethodExtended3("isEqual", (q_method_bool_t)TermIOS_isEqual_C7Termios, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TERMIOS->getTypeInfo(), NULL, "termios");

   // nothing TermIOS::setCC(softint offset, softint value){}
   QC_TERMIOS->addMethodExtended3("setCC", (q_method_t)TermIOS_setCC_vivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, softBigIntTypeInfo, NULL, "offset", softBigIntTypeInfo, NULL, "value");

   // nothing TermIOS::setCFlag(softint flag){}
   QC_TERMIOS->addMethodExtended3("setCFlag", (q_method_t)TermIOS_setCFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

   // nothing TermIOS::setIFlag(softint flag){}
   QC_TERMIOS->addMethodExtended3("setIFlag", (q_method_t)TermIOS_setIFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

   // nothing TermIOS::setLFlag(softint flag){}
   QC_TERMIOS->addMethodExtended3("setLFlag", (q_method_t)TermIOS_setLFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

   // nothing TermIOS::setOFlag(softint flag){}
   QC_TERMIOS->addMethodExtended3("setOFlag", (q_method_t)TermIOS_setOFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

   // static hash TermIOS::getWindowSize(){}
   QC_TERMIOS->addStaticMethodExtended3("getWindowSize", (q_func_t)static_TermIOS_getWindowSize, false, QC_RET_VALUE_ONLY, QDOM_TERMINAL_IO, hashTypeInfo);
# 28 "QC_TermIOS.qpp"
#ifndef ECHOKE
#define ECHOKE 0
#endif
# 35 "QC_TermIOS.qpp"
#ifndef ECHOE
#define ECHOE 0
#endif
# 41 "QC_TermIOS.qpp"
#ifndef ECHO
#define ECHO 0
#endif
# 47 "QC_TermIOS.qpp"
#ifndef ECHONL
#define ECHONL 0
#endif
# 53 "QC_TermIOS.qpp"
#ifndef ECHOPRT
#define ECHOPRT 0
#endif
# 59 "QC_TermIOS.qpp"
#ifndef ECHOCTL
#define ECHOCTL 0
#endif
# 65 "QC_TermIOS.qpp"
#ifndef ISIG
#define ISIG 0
#endif
# 71 "QC_TermIOS.qpp"
#ifndef ICANON
#define ICANON 0
#endif
# 77 "QC_TermIOS.qpp"
#ifndef ALTWERASE
#define ALTWERASE 0
#endif
# 83 "QC_TermIOS.qpp"
#ifndef IEXTEN
#define IEXTEN 0
#endif
# 89 "QC_TermIOS.qpp"
#ifndef EXTPROC
#define EXTPROC 0
#endif
# 95 "QC_TermIOS.qpp"
#ifndef TOSTOP
#define TOSTOP 0
#endif
# 101 "QC_TermIOS.qpp"
#ifndef FLUSHO
#define FLUSHO 0
#endif
# 107 "QC_TermIOS.qpp"
#ifndef NOKERNINFO
#define NOKERNINFO 0
#endif
# 113 "QC_TermIOS.qpp"
#ifndef PENDIN
#define PENDIN 0
#endif
# 119 "QC_TermIOS.qpp"
#ifndef NOFLSH
#define NOFLSH 0
#endif
   ns.addConstant("ALTWERASE", new QoreBigIntNode(ALTWERASE));
   ns.addConstant("ECHO", new QoreBigIntNode(ECHO));
   ns.addConstant("ECHOCTL", new QoreBigIntNode(ECHOCTL));
   ns.addConstant("ECHOE", new QoreBigIntNode(ECHOE));
   ns.addConstant("ECHOKE", new QoreBigIntNode(ECHOKE));
   ns.addConstant("ECHONL", new QoreBigIntNode(ECHONL));
   ns.addConstant("ECHOPRT", new QoreBigIntNode(ECHOPRT));
   ns.addConstant("EXTPROC", new QoreBigIntNode(EXTPROC));
   ns.addConstant("FLUSHO", new QoreBigIntNode(FLUSHO));
   ns.addConstant("ICANON", new QoreBigIntNode(ICANON));
   ns.addConstant("IEXTEN", new QoreBigIntNode(IEXTEN));
   ns.addConstant("ISIG", new QoreBigIntNode(ISIG));
   ns.addConstant("NOFLSH", new QoreBigIntNode(NOFLSH));
   ns.addConstant("NOKERNINFO", new QoreBigIntNode(NOKERNINFO));
   ns.addConstant("PENDIN", new QoreBigIntNode(PENDIN));
   ns.addConstant("TOSTOP", new QoreBigIntNode(TOSTOP));
# 129 "QC_TermIOS.qpp"
#ifndef CSIZE
#define CSIZE 0
#endif
# 136 "QC_TermIOS.qpp"
#ifndef CS5
#define CS5 0
#endif
# 142 "QC_TermIOS.qpp"
#ifndef CS6
#define CS6 0
#endif
# 148 "QC_TermIOS.qpp"
#ifndef CS7
#define CS7 0
#endif
# 154 "QC_TermIOS.qpp"
#ifndef CS8
#define CS8 0
#endif
# 160 "QC_TermIOS.qpp"
#ifndef CSTOPB
#define CSTOPB 0
#endif
# 166 "QC_TermIOS.qpp"
#ifndef CREAD
#define CREAD 0
#endif
# 172 "QC_TermIOS.qpp"
#ifndef PARENB
#define PARENB 0
#endif
# 178 "QC_TermIOS.qpp"
#ifndef PARODD
#define PARODD 0
#endif
# 184 "QC_TermIOS.qpp"
#ifndef HUPCL
#define HUPCL 0
#endif
# 190 "QC_TermIOS.qpp"
#ifndef CLOCAL
#define CLOCAL 0
#endif
# 196 "QC_TermIOS.qpp"
#ifndef CCTS_OFLOW
#define CCTS_OFLOW 0
#endif
# 202 "QC_TermIOS.qpp"
#ifndef CRTSCTS
#define CRTSCTS 0
#endif
# 208 "QC_TermIOS.qpp"
#ifndef CRTS_IFLOW
#define CRTS_IFLOW 0
#endif
# 214 "QC_TermIOS.qpp"
#ifndef CDSR_OFLOW
#define CDSR_OFLOW 0
#endif
# 220 "QC_TermIOS.qpp"
#ifndef CCAR_OFLOW
#define CCAR_OFLOW 0
#endif
# 226 "QC_TermIOS.qpp"
#ifndef MDMBUF
#define MDMBUF 0
#endif
   ns.addConstant("CCAR_OFLOW", new QoreBigIntNode(CCAR_OFLOW));
   ns.addConstant("CCTS_OFLOW", new QoreBigIntNode(CCTS_OFLOW));
   ns.addConstant("CDSR_OFLOW", new QoreBigIntNode(CDSR_OFLOW));
   ns.addConstant("CLOCAL", new QoreBigIntNode(CLOCAL));
   ns.addConstant("CREAD", new QoreBigIntNode(CREAD));
   ns.addConstant("CRTSCTS", new QoreBigIntNode(CRTSCTS));
   ns.addConstant("CRTS_IFLOW", new QoreBigIntNode(CRTS_IFLOW));
   ns.addConstant("CS5", new QoreBigIntNode(CS5));
   ns.addConstant("CS6", new QoreBigIntNode(CS6));
   ns.addConstant("CS7", new QoreBigIntNode(CS7));
   ns.addConstant("CS8", new QoreBigIntNode(CS8));
   ns.addConstant("CSIZE", new QoreBigIntNode(CSIZE));
   ns.addConstant("CSTOPB", new QoreBigIntNode(CSTOPB));
   ns.addConstant("HUPCL", new QoreBigIntNode(HUPCL));
   ns.addConstant("MDMBUF", new QoreBigIntNode(MDMBUF));
   ns.addConstant("PARENB", new QoreBigIntNode(PARENB));
   ns.addConstant("PARODD", new QoreBigIntNode(PARODD));
# 236 "QC_TermIOS.qpp"
#ifndef OPOST
#define OPOST 0
#endif
# 243 "QC_TermIOS.qpp"
#ifndef ONLCR
#define ONLCR 0
#endif
# 249 "QC_TermIOS.qpp"
#ifndef OXTABS
#define OXTABS 0
#endif
# 255 "QC_TermIOS.qpp"
#ifndef ONOEOT
#define ONOEOT 0
#endif
# 261 "QC_TermIOS.qpp"
#ifndef OCRNL
#define OCRNL 0
#endif
# 267 "QC_TermIOS.qpp"
#ifndef OLCUC
#define OLCUC 0
#endif
# 273 "QC_TermIOS.qpp"
#ifndef ONOCR
#define ONOCR 0
#endif
# 279 "QC_TermIOS.qpp"
#ifndef ONLRET
#define ONLRET 0
#endif
   ns.addConstant("OCRNL", new QoreBigIntNode(OCRNL));
   ns.addConstant("OLCUC", new QoreBigIntNode(OLCUC));
   ns.addConstant("ONLCR", new QoreBigIntNode(ONLCR));
   ns.addConstant("ONLRET", new QoreBigIntNode(ONLRET));
   ns.addConstant("ONOCR", new QoreBigIntNode(ONOCR));
   ns.addConstant("ONOEOT", new QoreBigIntNode(ONOEOT));
   ns.addConstant("OPOST", new QoreBigIntNode(OPOST));
   ns.addConstant("OXTABS", new QoreBigIntNode(OXTABS));
# 289 "QC_TermIOS.qpp"
#ifndef IGNBRK
#define IGNBRK 0
#endif
# 296 "QC_TermIOS.qpp"
#ifndef BRKINT
#define BRKINT 0
#endif
# 302 "QC_TermIOS.qpp"
#ifndef IGNPAR
#define IGNPAR 0
#endif
# 308 "QC_TermIOS.qpp"
#ifndef PARMRK
#define PARMRK 0
#endif
# 314 "QC_TermIOS.qpp"
#ifndef INPCK
#define INPCK 0
#endif
# 320 "QC_TermIOS.qpp"
#ifndef ISTRIP
#define ISTRIP 0
#endif
# 326 "QC_TermIOS.qpp"
#ifndef INLCR
#define INLCR 0
#endif
# 332 "QC_TermIOS.qpp"
#ifndef IGNCR
#define IGNCR 0
#endif
# 338 "QC_TermIOS.qpp"
#ifndef ICRNL
#define ICRNL 0
#endif
# 344 "QC_TermIOS.qpp"
#ifndef IXON
#define IXON 0
#endif
# 350 "QC_TermIOS.qpp"
#ifndef IXOFF
#define IXOFF 0
#endif
# 356 "QC_TermIOS.qpp"
#ifndef IXANY
#define IXANY 0
#endif
# 362 "QC_TermIOS.qpp"
#ifndef IMAXBEL
#define IMAXBEL 0
#endif
# 368 "QC_TermIOS.qpp"
#ifndef IUCLC
#define IUCLC 0
#endif
   ns.addConstant("BRKINT", new QoreBigIntNode(BRKINT));
   ns.addConstant("ICRNL", new QoreBigIntNode(ICRNL));
   ns.addConstant("IGNBRK", new QoreBigIntNode(IGNBRK));
   ns.addConstant("IGNCR", new QoreBigIntNode(IGNCR));
   ns.addConstant("IGNPAR", new QoreBigIntNode(IGNPAR));
   ns.addConstant("IMAXBEL", new QoreBigIntNode(IMAXBEL));
   ns.addConstant("INLCR", new QoreBigIntNode(INLCR));
   ns.addConstant("INPCK", new QoreBigIntNode(INPCK));
   ns.addConstant("ISTRIP", new QoreBigIntNode(ISTRIP));
   ns.addConstant("IUCLC", new QoreBigIntNode(IUCLC));
   ns.addConstant("IXANY", new QoreBigIntNode(IXANY));
   ns.addConstant("IXOFF", new QoreBigIntNode(IXOFF));
   ns.addConstant("IXON", new QoreBigIntNode(IXON));
   ns.addConstant("PARMRK", new QoreBigIntNode(PARMRK));
# 378 "QC_TermIOS.qpp"
#ifndef VEOF
#define VEOF 0
#endif
# 385 "QC_TermIOS.qpp"
#ifndef VEOL
#define VEOL 0
#endif
# 391 "QC_TermIOS.qpp"
#ifndef VEOL2
#define VEOL2 0
#endif
# 397 "QC_TermIOS.qpp"
#ifndef VERASE
#define VERASE 0
#endif
# 403 "QC_TermIOS.qpp"
#ifndef VWERASE
#define VWERASE 0
#endif
# 409 "QC_TermIOS.qpp"
#ifndef VKILL
#define VKILL 0
#endif
# 415 "QC_TermIOS.qpp"
#ifndef VREPRINT
#define VREPRINT 0
#endif
# 421 "QC_TermIOS.qpp"
#ifndef VINTR
#define VINTR 0
#endif
# 427 "QC_TermIOS.qpp"
#ifndef VQUIT
#define VQUIT 0
#endif
# 433 "QC_TermIOS.qpp"
#ifndef VSUSP
#define VSUSP 0
#endif
# 439 "QC_TermIOS.qpp"
#ifndef VDSUSP
#define VDSUSP 0
#endif
# 445 "QC_TermIOS.qpp"
#ifndef VSTART
#define VSTART 0
#endif
# 451 "QC_TermIOS.qpp"
#ifndef VSTOP
#define VSTOP 0
#endif
# 457 "QC_TermIOS.qpp"
#ifndef VLNEXT
#define VLNEXT 0
#endif
# 463 "QC_TermIOS.qpp"
#ifndef VDISCARD
#define VDISCARD 0
#endif
# 469 "QC_TermIOS.qpp"
#ifndef VMIN
#define VMIN 0
#endif
# 475 "QC_TermIOS.qpp"
#ifndef VTIME
#define VTIME 0
#endif
# 481 "QC_TermIOS.qpp"
#ifndef VSTATUS
#define VSTATUS 0
#endif
# 487 "QC_TermIOS.qpp"
#ifndef _POSIX_VDISABLE
#define _POSIX_VDISABLE 0xff
#endif
   ns.addConstant("VDISCARD", new QoreBigIntNode(VDISCARD));
   ns.addConstant("VDSUSP", new QoreBigIntNode(VDSUSP));
   ns.addConstant("VEOF", new QoreBigIntNode(VEOF));
   ns.addConstant("VEOL", new QoreBigIntNode(VEOL));
   ns.addConstant("VEOL2", new QoreBigIntNode(VEOL2));
   ns.addConstant("VERASE", new QoreBigIntNode(VERASE));
   ns.addConstant("VINTR", new QoreBigIntNode(VINTR));
   ns.addConstant("VKILL", new QoreBigIntNode(VKILL));
   ns.addConstant("VLNEXT", new QoreBigIntNode(VLNEXT));
   ns.addConstant("VMIN", new QoreBigIntNode(VMIN));
   ns.addConstant("VQUIT", new QoreBigIntNode(VQUIT));
   ns.addConstant("VREPRINT", new QoreBigIntNode(VREPRINT));
   ns.addConstant("VSTART", new QoreBigIntNode(VSTART));
   ns.addConstant("VSTATUS", new QoreBigIntNode(VSTATUS));
   ns.addConstant("VSTOP", new QoreBigIntNode(VSTOP));
   ns.addConstant("VSUSP", new QoreBigIntNode(VSUSP));
   ns.addConstant("VTIME", new QoreBigIntNode(VTIME));
   ns.addConstant("VWERASE", new QoreBigIntNode(VWERASE));
   ns.addConstant("_POSIX_VDISABLE", new QoreBigIntNode(_POSIX_VDISABLE));
# 497 "QC_TermIOS.qpp"
#ifndef TCSANOW
#define TCSANOW 0
#endif
# 504 "QC_TermIOS.qpp"
#ifndef TCSADRAIN
#define TCSADRAIN 0
#endif
# 510 "QC_TermIOS.qpp"
#ifndef TCSAFLUSH
#define TCSAFLUSH 0
#endif
# 516 "QC_TermIOS.qpp"
#ifndef TCSASOFT
#define TCSASOFT 0
#endif
   ns.addConstant("TCSADRAIN", new QoreBigIntNode(TCSADRAIN));
   ns.addConstant("TCSAFLUSH", new QoreBigIntNode(TCSAFLUSH));
   ns.addConstant("TCSANOW", new QoreBigIntNode(TCSANOW));
   ns.addConstant("TCSASOFT", new QoreBigIntNode(TCSASOFT));

   return QC_TERMIOS;
}
