/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_context.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/Context.h>

static void cx_err(const char* func, ExceptionSink* xsink) {
   xsink->raiseException("CONTEXT-ERROR", "function %s() called without a current data iteration context", func);
}
# 32 "ql_context.qpp"
# 67 "ql_context.qpp"
# 93 "ql_context.qpp"
# 119 "ql_context.qpp"
# 144 "ql_context.qpp"
// bool cx_first() {}
static bool f_cx_first(const QoreListNode* args, ExceptionSink* xsink) {
# 83 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c) {
      cx_err("cx_first", xsink);
      return 0;
   }

   return c->isFirst();
}

// bool cx_last() {}
static bool f_cx_last(const QoreListNode* args, ExceptionSink* xsink) {
# 109 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c) {
      cx_err("cx_last", xsink);
      return 0;
   }

   return c->isLast();
}

// int cx_pos() {}
static int64 f_cx_pos(const QoreListNode* args, ExceptionSink* xsink) {
# 134 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c) {
      cx_err("cx_pos", xsink);
      return 0;
   }

   return c->getPos();
}

// int cx_total() {}
static int64 f_cx_total(const QoreListNode* args, ExceptionSink* xsink) {
# 159 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c) {
      cx_err("cx_total", xsink);
      return 0;
   }

   return c->getTotal();
}

// any cx_value(string key) {}
static AbstractQoreNode* f_cx_value_Vs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreStringNode* key = HARD_QORE_STRING(args, 0);
# 53 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c) {
      cx_err("cx_value", xsink);
      return 0;
   }

   TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
   if (!tmp)
      return 0;

   return c->evalValue(tmp->getBuffer(), xsink);
}


DLLLOCAL void init_context_functions(QoreNamespace& ns) {
   // bool cx_first() {}
   ns.addBuiltinVariant("cx_first", (q_func_bool_t)f_cx_first, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);
   // bool cx_last() {}
   ns.addBuiltinVariant("cx_last", (q_func_bool_t)f_cx_last, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);
   // int cx_pos() {}
   ns.addBuiltinVariant("cx_pos", (q_func_int64_t)f_cx_pos, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);
   // int cx_total() {}
   ns.addBuiltinVariant("cx_total", (q_func_int64_t)f_cx_total, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);
   // any cx_value(string key) {}
   ns.addBuiltinVariant("cx_value", (q_func_t)f_cx_value_Vs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 1, stringTypeInfo, NULL, "key");

}
