/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
 ql_list.qpp
 
 Qore Programming Language
 
 Copyright 2003 - 2013 David Nichols
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <qore/Qore.h>
#include <qore/intern/ql_list.h>
#include <qore/intern/qore_program_private.h>

ResolvedCallReferenceNode *getCallReference(const QoreString *str, ExceptionSink *xsink) {
   // ensure string is in default encoding                                                                                                                            
   TempEncodingHelper tmp(str, QCS_DEFAULT, xsink);
   if (!tmp)
      return 0;
   return qore_program_private::runtimeGetCallReference(getProgram(), tmp->getBuffer(), xsink);
}

bool inlist_intern(const AbstractQoreNode *arg, const QoreListNode *l, ExceptionSink *xsink) {
   ConstListIterator li(l);
   while (li.next()) {
      bool b = QoreLogicalEqualsOperatorNode::softEqual(arg, li.getValue(), xsink);
      if (*xsink)
         return false;
      if (b)
         return true;
   }
   return false;
}

QoreListNode *range_intern(int64 start, int64 stop, int64 step, ExceptionSink *xsink) {
    if (step < 1) {
        xsink->raiseException("RANGE-ERROR", "Value of the 'step' argument has to be greater than 0");
        return 0;
    }

    QoreListNode *l = new QoreListNode();
    if (start < stop) {
        for (int64 i = start; i <= stop; i += step) {
            l->push(new QoreBigIntNode(i));
        }
    }
    else {
        for (int64 i = start; i >= stop; i -= step) {
            l->push(new QoreBigIntNode(i));
        }
    }
    return l;
}
# 69 "ql_list.qpp"
# 80 "ql_list.qpp"
# 101 "ql_list.qpp"
# 123 "ql_list.qpp"
# 145 "ql_list.qpp"
# 152 "ql_list.qpp"
# 173 "ql_list.qpp"
# 195 "ql_list.qpp"
# 217 "ql_list.qpp"
# 224 "ql_list.qpp"
# 245 "ql_list.qpp"
# 267 "ql_list.qpp"
# 289 "ql_list.qpp"
# 296 "ql_list.qpp"
# 317 "ql_list.qpp"
# 339 "ql_list.qpp"
# 361 "ql_list.qpp"
# 379 "ql_list.qpp"
# 398 "ql_list.qpp"
# 417 "ql_list.qpp"
# 435 "ql_list.qpp"
# 453 "ql_list.qpp"
# 472 "ql_list.qpp"
# 491 "ql_list.qpp"
# 509 "ql_list.qpp"
# 515 "ql_list.qpp"
# 532 "ql_list.qpp"
# 539 "ql_list.qpp"
# 546 "ql_list.qpp"
# 565 "ql_list.qpp"
# 572 "ql_list.qpp"
# 579 "ql_list.qpp"
# 620 "ql_list.qpp"
# 650 "ql_list.qpp"
// bool inlist() {}
static bool f_inlist(const QoreListNode* args, ExceptionSink* xsink) {
# 535 "ql_list.qpp"
   return false;
}

// bool inlist(any arg, nothing x) {}
static bool f_inlist_VaVn(const QoreListNode* args, ExceptionSink* xsink) {
# 542 "ql_list.qpp"
   return false;
}

// bool inlist(any arg, softlist l) {}
static bool f_inlist_Vavl(const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
   const QoreListNode* l = HARD_QORE_LIST(args, 1);
# 561 "ql_list.qpp"
   return inlist_intern(arg, l, xsink);
}

// bool inlist_hard() {}
static bool f_inlist_hard(const QoreListNode* args, ExceptionSink* xsink) {
# 568 "ql_list.qpp"
   return false;
}

// bool inlist_hard(any arg, nothing x) {}
static bool f_inlist_hard_VaVn(const QoreListNode* args, ExceptionSink* xsink) {
# 575 "ql_list.qpp"
   return false;
}

// bool inlist_hard(any arg, softlist l) {}
static bool f_inlist_hard_Vavl(const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
   const QoreListNode* l = HARD_QORE_LIST(args, 1);
# 594 "ql_list.qpp"
   bool arg_is_nothing = is_nothing(arg);

   ConstListIterator li(l);
   while (li.next()) {
      const AbstractQoreNode *lp = li.getValue();

      bool b;

      // do hard comparison inline
      if (is_nothing(lp))
	 b = arg_is_nothing;
      else if (arg_is_nothing)
	 b = false;
      else {
	 b = arg->is_equal_hard(lp, xsink);
	 if (*xsink)
	    return 0;
      }

      if (b)
	 return true;
   }
   return false;
}

// any max(list l) {}
static AbstractQoreNode* f_max_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 449 "ql_list.qpp"
   return l->max();
}

// any max(list l, string func) {}
static AbstractQoreNode* f_max_VlVs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const QoreStringNode* func = HARD_QORE_STRING(args, 1);
# 467 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->max(*fr, xsink);
}

// any max(list l, code f) {}
static AbstractQoreNode* f_max_VlVc(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const ResolvedCallReferenceNode* f = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
# 487 "ql_list.qpp"
   return l->max(f, xsink);
}

// any max(...) {}
static AbstractQoreNode* f_max_VV(const QoreListNode* args, ExceptionSink* xsink) {
# 505 "ql_list.qpp"
   return args ? args->max() : 0;
}

// any min(list l) {}
static AbstractQoreNode* f_min_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 375 "ql_list.qpp"
   return l->min();
}

// any min(list l, string func) {}
static AbstractQoreNode* f_min_VlVs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const QoreStringNode* func = HARD_QORE_STRING(args, 1);
# 393 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->min(*fr, xsink);
}

// any min(list l, code f) {}
static AbstractQoreNode* f_min_VlVc(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const ResolvedCallReferenceNode* f = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
# 413 "ql_list.qpp"
   return l->min(f, xsink);
}

// any min(...) {}
static AbstractQoreNode* f_min_VV(const QoreListNode* args, ExceptionSink* xsink) {
# 431 "ql_list.qpp"
   return args ? args->min() : 0;
}

// list range(int start, int stop, int step = 1) {}
static AbstractQoreNode* f_range_ViViVi(const QoreListNode* args, ExceptionSink* xsink) {
   int64 start = HARD_QORE_INT(args, 0);
   int64 stop = HARD_QORE_INT(args, 1);
   int64 step = HARD_QORE_INT(args, 2);
# 646 "ql_list.qpp"
    return range_intern(start, stop, step, xsink);
}

// list range(int stop) {}
static AbstractQoreNode* f_range_Vi(const QoreListNode* args, ExceptionSink* xsink) {
   int64 stop = HARD_QORE_INT(args, 0);
# 670 "ql_list.qpp"
    return range_intern(0, stop, 1, xsink);
}

// nothing reverse() {}
static AbstractQoreNode* f_reverse(const QoreListNode* args, ExceptionSink* xsink) {
# 512 "ql_list.qpp"

   return 0;
}

// list reverse(list l) {}
static AbstractQoreNode* f_reverse_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 528 "ql_list.qpp"
   return l->reverse();
}

// any sort(any arg) {}
static AbstractQoreNode* f_sort_Va(const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
# 76 "ql_list.qpp"
   return arg ? arg->refSelf() : 0;
}

// list sort(list l) {}
static AbstractQoreNode* f_sort_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 97 "ql_list.qpp"
   return l->sort();
}

// list sort(list l, string func) {}
static AbstractQoreNode* f_sort_VlVs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const QoreStringNode* func = HARD_QORE_STRING(args, 1);
# 118 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sort(*fr, xsink);
}

// list sort(list l, code f) {}
static AbstractQoreNode* f_sort_VlVc(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const ResolvedCallReferenceNode* f = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
# 141 "ql_list.qpp"
   return l->sort(f, xsink);
}

// any sortDescending(any arg) {}
static AbstractQoreNode* f_sortDescending_Va(const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
# 148 "ql_list.qpp"
   return arg ? arg->refSelf() : 0;
}

// list sortDescending(list l) {}
static AbstractQoreNode* f_sortDescending_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 169 "ql_list.qpp"
   return l->sortDescending();
}

// list sortDescending(list l, string func) {}
static AbstractQoreNode* f_sortDescending_VlVs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const QoreStringNode* func = HARD_QORE_STRING(args, 1);
# 190 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortDescending(*fr, xsink);
}

// list sortDescending(list l, code f) {}
static AbstractQoreNode* f_sortDescending_VlVc(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const ResolvedCallReferenceNode* f = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
# 213 "ql_list.qpp"
   return l->sortDescending(f, xsink);
}

// any sortDescendingStable(any arg) {}
static AbstractQoreNode* f_sortDescendingStable_Va(const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
# 292 "ql_list.qpp"
   return arg ? arg->refSelf() : 0;
}

// list sortDescendingStable(list l) {}
static AbstractQoreNode* f_sortDescendingStable_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 313 "ql_list.qpp"
   return l->sortDescendingStable();
}

// list sortDescendingStable(list l, string func) {}
static AbstractQoreNode* f_sortDescendingStable_VlVs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const QoreStringNode* func = HARD_QORE_STRING(args, 1);
# 334 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortDescendingStable(*fr, xsink);
}

// list sortDescendingStable(list l, code f) {}
static AbstractQoreNode* f_sortDescendingStable_VlVc(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const ResolvedCallReferenceNode* f = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
# 357 "ql_list.qpp"
   return l->sortDescendingStable(f, xsink);
}

// any sortStable(any arg) {}
static AbstractQoreNode* f_sortStable_Va(const QoreListNode* args, ExceptionSink* xsink) {
   const AbstractQoreNode* arg = get_param(args, 0);
# 220 "ql_list.qpp"
   return arg ? arg->refSelf() : 0;
}

// list sortStable(list l) {}
static AbstractQoreNode* f_sortStable_Vl(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
# 241 "ql_list.qpp"
   return l->sortStable();
}

// list sortStable(list l, string func) {}
static AbstractQoreNode* f_sortStable_VlVs(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const QoreStringNode* func = HARD_QORE_STRING(args, 1);
# 262 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortStable(*fr, xsink);
}

// list sortStable(list l, code f) {}
static AbstractQoreNode* f_sortStable_VlVc(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreListNode* l = HARD_QORE_LIST(args, 0);
   const ResolvedCallReferenceNode* f = reinterpret_cast<const ResolvedCallReferenceNode*>(get_param(args, 1));
# 285 "ql_list.qpp"
   return l->sortStable(f, xsink);
}


DLLLOCAL void init_list_functions(QoreNamespace& ns) {
   // bool inlist() {}
   ns.addBuiltinVariant("inlist", (q_func_bool_t)f_inlist, QC_NOOP, QDOM_DEFAULT, boolTypeInfo);
   // bool inlist(any arg, nothing x) {}
   ns.addBuiltinVariant("inlist", (q_func_bool_t)f_inlist_VaVn, QC_NOOP, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, NULL, "arg", nothingTypeInfo, NULL, "x");
   // bool inlist(any arg, softlist l) {}
   ns.addBuiltinVariant("inlist", (q_func_bool_t)f_inlist_Vavl, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, NULL, "arg", softListTypeInfo, NULL, "l");
   // bool inlist_hard() {}
   ns.addBuiltinVariant("inlist_hard", (q_func_bool_t)f_inlist_hard, QC_NOOP, QDOM_DEFAULT, boolTypeInfo);
   // bool inlist_hard(any arg, nothing x) {}
   ns.addBuiltinVariant("inlist_hard", (q_func_bool_t)f_inlist_hard_VaVn, QC_NOOP, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, NULL, "arg", nothingTypeInfo, NULL, "x");
   // bool inlist_hard(any arg, softlist l) {}
   ns.addBuiltinVariant("inlist_hard", (q_func_bool_t)f_inlist_hard_Vavl, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, NULL, "arg", softListTypeInfo, NULL, "l");
   // any max(list l) {}
   ns.addBuiltinVariant("max", (q_func_t)f_max_Vl, QC_CONSTANT, QDOM_DEFAULT, anyTypeInfo, 1, listTypeInfo, NULL, "l");
   // any max(list l, string func) {}
   ns.addBuiltinVariant("max", (q_func_t)f_max_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
   // any max(list l, code f) {}
   ns.addBuiltinVariant("max", (q_func_t)f_max_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
   // any max(...) {}
   ns.addBuiltinVariant("max", (q_func_t)f_max_VV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, anyTypeInfo);
   // any min(list l) {}
   ns.addBuiltinVariant("min", (q_func_t)f_min_Vl, QC_CONSTANT, QDOM_DEFAULT, anyTypeInfo, 1, listTypeInfo, NULL, "l");
   // any min(list l, string func) {}
   ns.addBuiltinVariant("min", (q_func_t)f_min_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
   // any min(list l, code f) {}
   ns.addBuiltinVariant("min", (q_func_t)f_min_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, anyTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
   // any min(...) {}
   ns.addBuiltinVariant("min", (q_func_t)f_min_VV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, anyTypeInfo);
   // list range(int start, int stop, int step = 1) {}
   ns.addBuiltinVariant("range", (q_func_t)f_range_ViViVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 3, bigIntTypeInfo, NULL, "start", bigIntTypeInfo, NULL, "stop", bigIntTypeInfo, new QoreBigIntNode(1), "step");
   // list range(int stop) {}
   ns.addBuiltinVariant("range", (q_func_t)f_range_Vi, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, bigIntTypeInfo, NULL, "stop");
   // nothing reverse() {}
   ns.addBuiltinVariant("reverse", (q_func_t)f_reverse, QC_NOOP, QDOM_DEFAULT, nothingTypeInfo);
   // list reverse(list l) {}
   ns.addBuiltinVariant("reverse", (q_func_t)f_reverse_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
   // any sort(any arg) {}
   ns.addBuiltinVariant("sort", (q_func_t)f_sort_Va, QC_NOOP, QDOM_DEFAULT, anyTypeInfo, 1, anyTypeInfo, NULL, "arg");
   // list sort(list l) {}
   ns.addBuiltinVariant("sort", (q_func_t)f_sort_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
   // list sort(list l, string func) {}
   ns.addBuiltinVariant("sort", (q_func_t)f_sort_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
   // list sort(list l, code f) {}
   ns.addBuiltinVariant("sort", (q_func_t)f_sort_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
   // any sortDescending(any arg) {}
   ns.addBuiltinVariant("sortDescending", (q_func_t)f_sortDescending_Va, QC_NOOP, QDOM_DEFAULT, anyTypeInfo, 1, anyTypeInfo, NULL, "arg");
   // list sortDescending(list l) {}
   ns.addBuiltinVariant("sortDescending", (q_func_t)f_sortDescending_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
   // list sortDescending(list l, string func) {}
   ns.addBuiltinVariant("sortDescending", (q_func_t)f_sortDescending_VlVs, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
   // list sortDescending(list l, code f) {}
   ns.addBuiltinVariant("sortDescending", (q_func_t)f_sortDescending_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
   // any sortDescendingStable(any arg) {}
   ns.addBuiltinVariant("sortDescendingStable", (q_func_t)f_sortDescendingStable_Va, QC_NOOP, QDOM_DEFAULT, anyTypeInfo, 1, anyTypeInfo, NULL, "arg");
   // list sortDescendingStable(list l) {}
   ns.addBuiltinVariant("sortDescendingStable", (q_func_t)f_sortDescendingStable_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
   // list sortDescendingStable(list l, string func) {}
   ns.addBuiltinVariant("sortDescendingStable", (q_func_t)f_sortDescendingStable_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
   // list sortDescendingStable(list l, code f) {}
   ns.addBuiltinVariant("sortDescendingStable", (q_func_t)f_sortDescendingStable_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
   // any sortStable(any arg) {}
   ns.addBuiltinVariant("sortStable", (q_func_t)f_sortStable_Va, QC_NOOP, QDOM_DEFAULT, anyTypeInfo, 1, anyTypeInfo, NULL, "arg");
   // list sortStable(list l) {}
   ns.addBuiltinVariant("sortStable", (q_func_t)f_sortStable_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
   // list sortStable(list l, string func) {}
   ns.addBuiltinVariant("sortStable", (q_func_t)f_sortStable_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
   // list sortStable(list l, code f) {}
   ns.addBuiltinVariant("sortStable", (q_func_t)f_sortStable_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");

}
