/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_math.qpp

  Qore Programming Language

  Copyright 2003 - 2013 David Nichols

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qore/Qore.h>
#include <qore/intern/ql_math.h>
#include <qore/intern/qore_number_private.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

static double minus_infinity() {
#ifdef INFINITY
   return -INFINITY;
#else
   // returns -infinity
   return log10((double)0.0);
#endif
}
# 65 "ql_math.qpp"
# 70 "ql_math.qpp"
# 84 "ql_math.qpp"
# 106 "ql_math.qpp"
# 113 "ql_math.qpp"
# 130 "ql_math.qpp"
# 147 "ql_math.qpp"
# 154 "ql_math.qpp"
# 169 "ql_math.qpp"
# 184 "ql_math.qpp"
# 191 "ql_math.qpp"
# 209 "ql_math.qpp"
# 237 "ql_math.qpp"
# 254 "ql_math.qpp"
# 271 "ql_math.qpp"
# 288 "ql_math.qpp"
# 295 "ql_math.qpp"
# 311 "ql_math.qpp"
# 327 "ql_math.qpp"
# 334 "ql_math.qpp"
# 349 "ql_math.qpp"
# 364 "ql_math.qpp"
# 371 "ql_math.qpp"
# 386 "ql_math.qpp"
# 401 "ql_math.qpp"
# 408 "ql_math.qpp"
# 423 "ql_math.qpp"
# 438 "ql_math.qpp"
# 445 "ql_math.qpp"
# 460 "ql_math.qpp"
# 475 "ql_math.qpp"
# 482 "ql_math.qpp"
# 497 "ql_math.qpp"
# 512 "ql_math.qpp"
# 519 "ql_math.qpp"
# 534 "ql_math.qpp"
# 549 "ql_math.qpp"
# 556 "ql_math.qpp"
# 571 "ql_math.qpp"
# 586 "ql_math.qpp"
# 601 "ql_math.qpp"
# 616 "ql_math.qpp"
# 623 "ql_math.qpp"
# 639 "ql_math.qpp"
# 655 "ql_math.qpp"
# 662 "ql_math.qpp"
# 677 "ql_math.qpp"
# 692 "ql_math.qpp"
# 699 "ql_math.qpp"
# 714 "ql_math.qpp"
# 729 "ql_math.qpp"
# 736 "ql_math.qpp"
# 751 "ql_math.qpp"
# 766 "ql_math.qpp"
# 773 "ql_math.qpp"
# 788 "ql_math.qpp"
# 803 "ql_math.qpp"
# 810 "ql_math.qpp"
# 825 "ql_math.qpp"
# 840 "ql_math.qpp"
# 847 "ql_math.qpp"
# 862 "ql_math.qpp"
# 877 "ql_math.qpp"
# 884 "ql_math.qpp"
# 899 "ql_math.qpp"
# 906 "ql_math.qpp"
# 921 "ql_math.qpp"
# 936 "ql_math.qpp"
# 943 "ql_math.qpp"
# 958 "ql_math.qpp"
# 977 "ql_math.qpp"
# 984 "ql_math.qpp"
# 999 "ql_math.qpp"
# 1014 "ql_math.qpp"
// int abs(int i) {}
static int64 f_abs_Vi(const QoreListNode* args, ExceptionSink* xsink) {
   int64 i = HARD_QORE_INT(args, 0);
# 250 "ql_math.qpp"
   return i < 0 ? -i : i;
}

// number abs(number n) {}
static AbstractQoreNode* f_abs_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 267 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_abs);
}

// float abs(softfloat f) {}
static double f_abs_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 284 "ql_math.qpp"
   return fabs(f);
}

// float abs() {}
static double f_abs(const QoreListNode* args, ExceptionSink* xsink) {
# 291 "ql_math.qpp"
   return 0;
}

// number acos(number n) {}
static AbstractQoreNode* f_acos_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 567 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_acos);
}

// float acos(softfloat f) {}
static double f_acos_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 582 "ql_math.qpp"
   return acos(f);
}

// number asin(number n) {}
static AbstractQoreNode* f_asin_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 530 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_asin);
}

// float asin(softfloat f) {}
static double f_asin_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 545 "ql_math.qpp"
   return asin(f);
}

// float asin() {}
static double f_asin(const QoreListNode* args, ExceptionSink* xsink) {
# 552 "ql_math.qpp"
   return 0;
}

// number atan(number n) {}
static AbstractQoreNode* f_atan_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 597 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_atan);
}

// float atan(softfloat f) {}
static double f_atan_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 612 "ql_math.qpp"
   return atan(f);
}

// float atan() {}
static double f_atan(const QoreListNode* args, ExceptionSink* xsink) {
# 619 "ql_math.qpp"
   return 0;
}

// number atan2(number y, number x) {}
static AbstractQoreNode* f_atan2_VnVn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* y = HARD_QORE_NUMBER(args, 0);
   const QoreNumberNode* x = HARD_QORE_NUMBER(args, 1);
# 635 "ql_math.qpp"
   return qore_number_private::doBinary(*y, mpfr_atan2, *x);
}

// float atan2(softfloat y, softfloat x) {}
static double f_atan2_vfvf(const QoreListNode* args, ExceptionSink* xsink) {
   double y = HARD_QORE_FLOAT(args, 0);
   double x = HARD_QORE_FLOAT(args, 1);
# 651 "ql_math.qpp"
   return atan2(y, x);
}

// float atan2() {}
static double f_atan2(const QoreListNode* args, ExceptionSink* xsink) {
# 658 "ql_math.qpp"
   return 0;
}

// number cbrt(number n) {}
static AbstractQoreNode* f_cbrt_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 382 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_cbrt);
}

// float cbrt(softfloat f) {}
static double f_cbrt_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 397 "ql_math.qpp"
   return cbrt(f);
}

// float cbrt() {}
static double f_cbrt(const QoreListNode* args, ExceptionSink* xsink) {
# 404 "ql_math.qpp"
   return 0;
}

// number ceil(number n) {}
static AbstractQoreNode* f_ceil_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 126 "ql_math.qpp"
   return qore_number_private::doUnaryNR(*n, mpfr_ceil);
}

// float ceil(softfloat f) {}
static double f_ceil_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 143 "ql_math.qpp"
   return ceil(f);
}

// float ceil() {}
static double f_ceil(const QoreListNode* args, ExceptionSink* xsink) {
# 150 "ql_math.qpp"
   return 0;
}

// number cos(number n) {}
static AbstractQoreNode* f_cos_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 456 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_cos);
}

// float cos(float f) {}
static double f_cos_Vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 471 "ql_math.qpp"
   return cos(f);
}

// float cos() {}
static double f_cos(const QoreListNode* args, ExceptionSink* xsink) {
# 478 "ql_math.qpp"
   return 1;
}

// number cosh(number n) {}
static AbstractQoreNode* f_cosh_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 710 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_cosh);
}

// float cosh(softfloat f) {}
static double f_cosh_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 725 "ql_math.qpp"
   return cosh(f);
}

// float cosh() {}
static double f_cosh(const QoreListNode* args, ExceptionSink* xsink) {
# 732 "ql_math.qpp"
   return 1;
}

// number exp(number n) {}
static AbstractQoreNode* f_exp_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 917 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_exp);
}

// float exp(softfloat f) {}
static double f_exp_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 932 "ql_math.qpp"
   return exp(f);
}

// float exp() {}
static double f_exp(const QoreListNode* args, ExceptionSink* xsink) {
# 939 "ql_math.qpp"
   return 1;
}

// number exp2(number n) {}
static AbstractQoreNode* f_exp2_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 954 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_exp2);
}

// float exp2(softfloat f) {}
static double f_exp2_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 969 "ql_math.qpp"
#ifdef HAVE_EXP2
   return exp2(f);
#else
   return pow(2, f);
#endif
   return 0;
}

// float exp2() {}
static double f_exp2(const QoreListNode* args, ExceptionSink* xsink) {
# 980 "ql_math.qpp"
   return 1;
}

// number expm1(number n) {}
static AbstractQoreNode* f_expm1_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 995 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_expm1);
}

// float expm1(softfloat f) {}
static double f_expm1_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 1010 "ql_math.qpp"
   return expm1(f);
}

// float expm1() {}
static double f_expm1(const QoreListNode* args, ExceptionSink* xsink) {
# 1017 "ql_math.qpp"
   return 0;
}

// number floor(softnumber n) {}
static AbstractQoreNode* f_floor_vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 165 "ql_math.qpp"
   return qore_number_private::doUnaryNR(*n, mpfr_floor);
}

// float floor(softfloat f) {}
static double f_floor_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 180 "ql_math.qpp"
   return floor(f);
}

// float floor() {}
static double f_floor(const QoreListNode* args, ExceptionSink* xsink) {
# 187 "ql_math.qpp"
   return 0;
}

// number hypot(number x, number y) {}
static AbstractQoreNode* f_hypot_VnVn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* x = HARD_QORE_NUMBER(args, 0);
   const QoreNumberNode* y = HARD_QORE_NUMBER(args, 1);
# 307 "ql_math.qpp"
   return qore_number_private::doBinary(*x, mpfr_hypot, *y);
}

// float hypot(softfloat x, softfloat y) {}
static double f_hypot_vfvf(const QoreListNode* args, ExceptionSink* xsink) {
   double x = HARD_QORE_FLOAT(args, 0);
   double y = HARD_QORE_FLOAT(args, 1);
# 323 "ql_math.qpp"
   return hypot(x, y);
}

// float hypot() {}
static double f_hypot(const QoreListNode* args, ExceptionSink* xsink) {
# 330 "ql_math.qpp"
   return 0;
}

// number log10(number n) {}
static AbstractQoreNode* f_log10_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 821 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_log10);
}

// float log10(softfloat f) {}
static double f_log10_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 836 "ql_math.qpp"
   return log10(f);
}

// float log10() {}
static double f_log10(const QoreListNode* args, ExceptionSink* xsink) {
# 843 "ql_math.qpp"
   return minus_infinity();
}

// number log1p(number n) {}
static AbstractQoreNode* f_log1p_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 858 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_log1p);
}

// float log1p(softfloat f) {}
static double f_log1p_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 873 "ql_math.qpp"
   return log1p(f);
}

// float log1p() {}
static double f_log1p(const QoreListNode* args, ExceptionSink* xsink) {
# 880 "ql_math.qpp"
   return 0;
}

// float logb(softfloat f) {}
static double f_logb_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 895 "ql_math.qpp"
   return logb(f);
}

// float logb() {}
static double f_logb(const QoreListNode* args, ExceptionSink* xsink) {
# 902 "ql_math.qpp"
   return minus_infinity();
}

// number nlog(number n) {}
static AbstractQoreNode* f_nlog_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 784 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_log);
}

// float nlog(softfloat f) {}
static double f_nlog_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 799 "ql_math.qpp"
   return log(f);
}

// float nlog() {}
static double f_nlog(const QoreListNode* args, ExceptionSink* xsink) {
# 806 "ql_math.qpp"
   return minus_infinity();
}

// number pow(number x, number y) {}
static AbstractQoreNode* f_pow_VnVn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* x = HARD_QORE_NUMBER(args, 0);
   const QoreNumberNode* y = HARD_QORE_NUMBER(args, 1);
# 205 "ql_math.qpp"
   return qore_number_private::doBinary(*x, mpfr_pow, *y, xsink);
}

// float pow(softfloat x = 0.0, softfloat y = 0.0) {}
static double f_pow_vfvf(const QoreListNode* args, ExceptionSink* xsink) {
   double x = HARD_QORE_FLOAT(args, 0);
   double y = HARD_QORE_FLOAT(args, 1);
# 224 "ql_math.qpp"
   if (x == 0.0 && y < 0) {
      xsink->raiseException("DIVISION-BY-ZERO", "pow(x, y) if x = 0 then y must be a non-negative value");
      return 0;
   }
   if (x < 0 && y != ceil(y)) {
      xsink->raiseException("INVALID-POW-ARGUMENTS", "pow(x, y) x cannot be negative when y is not an integer value");
      return 0;
   }

   return pow(x, y);
}

// number round(number n) {}
static AbstractQoreNode* f_round_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 80 "ql_math.qpp"
   return qore_number_private::doUnaryNR(*n, mpfr_round);
}

// float round(softfloat f) {}
static double f_round_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 97 "ql_math.qpp"
#ifdef HAVE_ROUND
   return round(f);
#else
   missing_function_error("round", "ROUND", xsink);
   return 0;
#endif
   return 0;
}

// float round() {}
static double f_round(const QoreListNode* args, ExceptionSink* xsink) {
# 109 "ql_math.qpp"
   return 0;
}

// number sin(number n) {}
static AbstractQoreNode* f_sin_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 419 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_sin);
}

// float sin(softfloat f) {}
static double f_sin_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 434 "ql_math.qpp"
   return sin(f);
}

// float sin() {}
static double f_sin(const QoreListNode* args, ExceptionSink* xsink) {
# 441 "ql_math.qpp"
   return 0;
}

// number sinh(number n) {}
static AbstractQoreNode* f_sinh_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 673 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_sinh);
}

// float sinh(softfloat f) {}
static double f_sinh_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 688 "ql_math.qpp"
   return sinh(f);
}

// float sinh() {}
static double f_sinh(const QoreListNode* args, ExceptionSink* xsink) {
# 695 "ql_math.qpp"
   return 0;
}

// number sqrt(number n) {}
static AbstractQoreNode* f_sqrt_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 345 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_sqrt);
}

// float sqrt(softfloat f) {}
static double f_sqrt_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 360 "ql_math.qpp"
   return sqrt(f);
}

// float sqrt() {}
static double f_sqrt(const QoreListNode* args, ExceptionSink* xsink) {
# 367 "ql_math.qpp"
   return 0;
}

// number tan(number n) {}
static AbstractQoreNode* f_tan_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 493 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_tan);
}

// float tan(softfloat f) {}
static double f_tan_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 508 "ql_math.qpp"
   return tan(f);
}

// float tan() {}
static double f_tan(const QoreListNode* args, ExceptionSink* xsink) {
# 515 "ql_math.qpp"
   return 0;
}

// number tanh(number n) {}
static AbstractQoreNode* f_tanh_Vn(const QoreListNode* args, ExceptionSink* xsink) {
   const QoreNumberNode* n = HARD_QORE_NUMBER(args, 0);
# 747 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_tanh);
}

// float tanh(softfloat f) {}
static double f_tanh_vf(const QoreListNode* args, ExceptionSink* xsink) {
   double f = HARD_QORE_FLOAT(args, 0);
# 762 "ql_math.qpp"
   return tanh(f);
}

// float tanh() {}
static double f_tanh(const QoreListNode* args, ExceptionSink* xsink) {
# 769 "ql_math.qpp"
   return 0;
}


DLLLOCAL void init_math_functions(QoreNamespace& ns) {
   // int abs(int i) {}
   ns.addBuiltinVariant("abs", (q_func_int64_t)f_abs_Vi, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, NULL, "i");
   // number abs(number n) {}
   ns.addBuiltinVariant("abs", (q_func_t)f_abs_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float abs(softfloat f) {}
   ns.addBuiltinVariant("abs", (q_func_double_t)f_abs_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float abs() {}
   ns.addBuiltinVariant("abs", (q_func_double_t)f_abs, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number acos(number n) {}
   ns.addBuiltinVariant("acos", (q_func_t)f_acos_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float acos(softfloat f) {}
   ns.addBuiltinVariant("acos", (q_func_double_t)f_acos_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // number asin(number n) {}
   ns.addBuiltinVariant("asin", (q_func_t)f_asin_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float asin(softfloat f) {}
   ns.addBuiltinVariant("asin", (q_func_double_t)f_asin_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float asin() {}
   ns.addBuiltinVariant("asin", (q_func_double_t)f_asin, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number atan(number n) {}
   ns.addBuiltinVariant("atan", (q_func_t)f_atan_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float atan(softfloat f) {}
   ns.addBuiltinVariant("atan", (q_func_double_t)f_atan_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float atan() {}
   ns.addBuiltinVariant("atan", (q_func_double_t)f_atan, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number atan2(number y, number x) {}
   ns.addBuiltinVariant("atan2", (q_func_t)f_atan2_VnVn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, NULL, "y", numberTypeInfo, NULL, "x");
   // float atan2(softfloat y, softfloat x) {}
   ns.addBuiltinVariant("atan2", (q_func_double_t)f_atan2_vfvf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, NULL, "y", softFloatTypeInfo, NULL, "x");
   // float atan2() {}
   ns.addBuiltinVariant("atan2", (q_func_double_t)f_atan2, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number cbrt(number n) {}
   ns.addBuiltinVariant("cbrt", (q_func_t)f_cbrt_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float cbrt(softfloat f) {}
   ns.addBuiltinVariant("cbrt", (q_func_double_t)f_cbrt_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float cbrt() {}
   ns.addBuiltinVariant("cbrt", (q_func_double_t)f_cbrt, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number ceil(number n) {}
   ns.addBuiltinVariant("ceil", (q_func_t)f_ceil_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float ceil(softfloat f) {}
   ns.addBuiltinVariant("ceil", (q_func_double_t)f_ceil_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float ceil() {}
   ns.addBuiltinVariant("ceil", (q_func_double_t)f_ceil, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number cos(number n) {}
   ns.addBuiltinVariant("cos", (q_func_t)f_cos_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float cos(float f) {}
   ns.addBuiltinVariant("cos", (q_func_double_t)f_cos_Vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, floatTypeInfo, NULL, "f");
   // float cos() {}
   ns.addBuiltinVariant("cos", (q_func_double_t)f_cos, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number cosh(number n) {}
   ns.addBuiltinVariant("cosh", (q_func_t)f_cosh_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float cosh(softfloat f) {}
   ns.addBuiltinVariant("cosh", (q_func_double_t)f_cosh_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float cosh() {}
   ns.addBuiltinVariant("cosh", (q_func_double_t)f_cosh, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number exp(number n) {}
   ns.addBuiltinVariant("exp", (q_func_t)f_exp_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float exp(softfloat f) {}
   ns.addBuiltinVariant("exp", (q_func_double_t)f_exp_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float exp() {}
   ns.addBuiltinVariant("exp", (q_func_double_t)f_exp, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number exp2(number n) {}
   ns.addBuiltinVariant("exp2", (q_func_t)f_exp2_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float exp2(softfloat f) {}
   ns.addBuiltinVariant("exp2", (q_func_double_t)f_exp2_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float exp2() {}
   ns.addBuiltinVariant("exp2", (q_func_double_t)f_exp2, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number expm1(number n) {}
   ns.addBuiltinVariant("expm1", (q_func_t)f_expm1_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float expm1(softfloat f) {}
   ns.addBuiltinVariant("expm1", (q_func_double_t)f_expm1_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float expm1() {}
   ns.addBuiltinVariant("expm1", (q_func_double_t)f_expm1, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number floor(softnumber n) {}
   ns.addBuiltinVariant("floor", (q_func_t)f_floor_vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, softNumberTypeInfo, NULL, "n");
   // float floor(softfloat f) {}
   ns.addBuiltinVariant("floor", (q_func_double_t)f_floor_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float floor() {}
   ns.addBuiltinVariant("floor", (q_func_double_t)f_floor, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number hypot(number x, number y) {}
   ns.addBuiltinVariant("hypot", (q_func_t)f_hypot_VnVn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, NULL, "x", numberTypeInfo, NULL, "y");
   // float hypot(softfloat x, softfloat y) {}
   ns.addBuiltinVariant("hypot", (q_func_double_t)f_hypot_vfvf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, NULL, "x", softFloatTypeInfo, NULL, "y");
   // float hypot() {}
   ns.addBuiltinVariant("hypot", (q_func_double_t)f_hypot, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number log10(number n) {}
   ns.addBuiltinVariant("log10", (q_func_t)f_log10_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float log10(softfloat f) {}
   ns.addBuiltinVariant("log10", (q_func_double_t)f_log10_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float log10() {}
   ns.addBuiltinVariant("log10", (q_func_double_t)f_log10, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number log1p(number n) {}
   ns.addBuiltinVariant("log1p", (q_func_t)f_log1p_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float log1p(softfloat f) {}
   ns.addBuiltinVariant("log1p", (q_func_double_t)f_log1p_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float log1p() {}
   ns.addBuiltinVariant("log1p", (q_func_double_t)f_log1p, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // float logb(softfloat f) {}
   ns.addBuiltinVariant("logb", (q_func_double_t)f_logb_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float logb() {}
   ns.addBuiltinVariant("logb", (q_func_double_t)f_logb, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number nlog(number n) {}
   ns.addBuiltinVariant("nlog", (q_func_t)f_nlog_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float nlog(softfloat f) {}
   ns.addBuiltinVariant("nlog", (q_func_double_t)f_nlog_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float nlog() {}
   ns.addBuiltinVariant("nlog", (q_func_double_t)f_nlog, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number pow(number x, number y) {}
   ns.addBuiltinVariant("pow", (q_func_t)f_pow_VnVn, QC_RET_VALUE_ONLY, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, NULL, "x", numberTypeInfo, NULL, "y");
   // float pow(softfloat x = 0.0, softfloat y = 0.0) {}
   ns.addBuiltinVariant("pow", (q_func_double_t)f_pow_vfvf, QC_RET_VALUE_ONLY, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, zero_float(), "x", softFloatTypeInfo, zero_float(), "y");
   // number round(number n) {}
   ns.addBuiltinVariant("round", (q_func_t)f_round_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float round(softfloat f) {}
   ns.addBuiltinVariant("round", (q_func_double_t)f_round_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float round() {}
   ns.addBuiltinVariant("round", (q_func_double_t)f_round, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number sin(number n) {}
   ns.addBuiltinVariant("sin", (q_func_t)f_sin_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float sin(softfloat f) {}
   ns.addBuiltinVariant("sin", (q_func_double_t)f_sin_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float sin() {}
   ns.addBuiltinVariant("sin", (q_func_double_t)f_sin, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number sinh(number n) {}
   ns.addBuiltinVariant("sinh", (q_func_t)f_sinh_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float sinh(softfloat f) {}
   ns.addBuiltinVariant("sinh", (q_func_double_t)f_sinh_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float sinh() {}
   ns.addBuiltinVariant("sinh", (q_func_double_t)f_sinh, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number sqrt(number n) {}
   ns.addBuiltinVariant("sqrt", (q_func_t)f_sqrt_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float sqrt(softfloat f) {}
   ns.addBuiltinVariant("sqrt", (q_func_double_t)f_sqrt_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float sqrt() {}
   ns.addBuiltinVariant("sqrt", (q_func_double_t)f_sqrt, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number tan(number n) {}
   ns.addBuiltinVariant("tan", (q_func_t)f_tan_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float tan(softfloat f) {}
   ns.addBuiltinVariant("tan", (q_func_double_t)f_tan_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float tan() {}
   ns.addBuiltinVariant("tan", (q_func_double_t)f_tan, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
   // number tanh(number n) {}
   ns.addBuiltinVariant("tanh", (q_func_t)f_tanh_Vn, QC_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, NULL, "n");
   // float tanh(softfloat f) {}
   ns.addBuiltinVariant("tanh", (q_func_double_t)f_tanh_vf, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, NULL, "f");
   // float tanh() {}
   ns.addBuiltinVariant("tanh", (q_func_double_t)f_tanh, QC_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);

}

DLLLOCAL void init_math_constants(QoreNamespace& ns) {
# 42 "ql_math.qpp"
# 49 "ql_math.qpp"
# 54 "ql_math.qpp"
   ns.addConstant("MAXINT", new QoreBigIntNode(LLONG_MAX));
   ns.addConstant("MININT", (new QoreBigIntNode(-LLONG_MAX - 1)));
   ns.addConstant("M_PI", new QoreFloatNode(3.14159265358979323846));
   ns.addConstant("M_PIn", (pi_number()));
# 65 "ql_math.qpp"
# 70 "ql_math.qpp"
# 84 "ql_math.qpp"
# 106 "ql_math.qpp"
# 113 "ql_math.qpp"
# 130 "ql_math.qpp"
# 147 "ql_math.qpp"
# 154 "ql_math.qpp"
# 169 "ql_math.qpp"
# 184 "ql_math.qpp"
# 191 "ql_math.qpp"
# 209 "ql_math.qpp"
# 237 "ql_math.qpp"
# 254 "ql_math.qpp"
# 271 "ql_math.qpp"
# 288 "ql_math.qpp"
# 295 "ql_math.qpp"
# 311 "ql_math.qpp"
# 327 "ql_math.qpp"
# 334 "ql_math.qpp"
# 349 "ql_math.qpp"
# 364 "ql_math.qpp"
# 371 "ql_math.qpp"
# 386 "ql_math.qpp"
# 401 "ql_math.qpp"
# 408 "ql_math.qpp"
# 423 "ql_math.qpp"
# 438 "ql_math.qpp"
# 445 "ql_math.qpp"
# 460 "ql_math.qpp"
# 475 "ql_math.qpp"
# 482 "ql_math.qpp"
# 497 "ql_math.qpp"
# 512 "ql_math.qpp"
# 519 "ql_math.qpp"
# 534 "ql_math.qpp"
# 549 "ql_math.qpp"
# 556 "ql_math.qpp"
# 571 "ql_math.qpp"
# 586 "ql_math.qpp"
# 601 "ql_math.qpp"
# 616 "ql_math.qpp"
# 623 "ql_math.qpp"
# 639 "ql_math.qpp"
# 655 "ql_math.qpp"
# 662 "ql_math.qpp"
# 677 "ql_math.qpp"
# 692 "ql_math.qpp"
# 699 "ql_math.qpp"
# 714 "ql_math.qpp"
# 729 "ql_math.qpp"
# 736 "ql_math.qpp"
# 751 "ql_math.qpp"
# 766 "ql_math.qpp"
# 773 "ql_math.qpp"
# 788 "ql_math.qpp"
# 803 "ql_math.qpp"
# 810 "ql_math.qpp"
# 825 "ql_math.qpp"
# 840 "ql_math.qpp"
# 847 "ql_math.qpp"
# 862 "ql_math.qpp"
# 877 "ql_math.qpp"
# 884 "ql_math.qpp"
# 899 "ql_math.qpp"
# 906 "ql_math.qpp"
# 921 "ql_math.qpp"
# 936 "ql_math.qpp"
# 943 "ql_math.qpp"
# 958 "ql_math.qpp"
# 977 "ql_math.qpp"
# 984 "ql_math.qpp"
# 999 "ql_math.qpp"
# 1014 "ql_math.qpp"

}
